package edu.bu.signstream.codingScheme.ui.importLegacyCodingScheme;

import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetContext;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class TreeDropTarget implements DropTargetListener {
	private DropTarget target;

	private JTree targetTree;

	private JDialog parentDialog;

	public TreeDropTarget(JTree tree, JDialog parentDialog) {
		this.targetTree = tree;
		this.parentDialog = parentDialog;
		target = new DropTarget(targetTree, this);
	}

	/*
	 * Drop Event Handlers
	 */
	private TreeNode getNodeForEvent(DropTargetDragEvent dtde) {
		Point p = dtde.getLocation();
		DropTargetContext dtc = dtde.getDropTargetContext();
		JTree tree = (JTree) dtc.getComponent();
		TreePath path = tree.getClosestPathForLocation(p.x, p.y);
		return (TreeNode) path.getLastPathComponent();
	}

	@Override
	public void dragEnter(DropTargetDragEvent dtde) {
		TreeNode node = getNodeForEvent(dtde);
		if (node.isLeaf()) {
			dtde.rejectDrag();
		} else {
			// start by supporting move operations
			// dtde.acceptDrag(DnDConstants.ACTION_MOVE);
			dtde.acceptDrag(dtde.getDropAction());
		}
	}

	@Override
	public void dragOver(DropTargetDragEvent dtde) {
//		TreeNode node = getNodeForEvent(dtde);
//		if (node.isLeaf()) {
//			dtde.rejectDrag();
//		} else {
			// start by supporting move operations
			// dtde.acceptDrag(DnDConstants.ACTION_MOVE);
			dtde.acceptDrag(dtde.getDropAction());
		// }
	}

	@Override
	public void dragExit(DropTargetEvent dte) {
	}

	@Override
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}

	@Override
	public void drop(DropTargetDropEvent dtde) {
		Point pt = dtde.getLocation();
		DropTargetContext dtc = dtde.getDropTargetContext();
		JTree tree = (JTree) dtc.getComponent();
		TreePath parentpath = tree.getClosestPathForLocation(pt.x, pt.y);
		DefaultMutableTreeNode parent = (DefaultMutableTreeNode) parentpath
				.getLastPathComponent();

		//
		System.err.println("pt = " + pt + ", parentpath = " + parentpath
				+ ", PARENT OBJECT = " + parent.getUserObject());
		//

		try {
			Transferable tr = dtde.getTransferable();

			DataFlavor[] flavors = tr.getTransferDataFlavors();
			for (int i = 0; i < flavors.length; i++) {
				if (tr.isDataFlavorSupported(flavors[i])) {
					dtde.acceptDrop(dtde.getDropAction());
					Object obj = tr.getTransferData(flavors[i]);
					Object parentObject = parent.getUserObject();
					//
					System.err.println("obj = " + obj + ", parentObject = "
							+ parentObject);
					//

					if (parentObject instanceof SS3Field
							&& obj instanceof SS3Field) {

						JOptionPane
								.showMessageDialog(
										parentDialog,
										"Can't add Field to Field. \n Only Value can be added to Field.",
										"Information dialog",
										JOptionPane.INFORMATION_MESSAGE);

						dtde.dropComplete(false);
					} else if (parentObject instanceof SS3CodingScheme
							&& obj instanceof SS3FieldValue) {
						JOptionPane
								.showMessageDialog(
										parentDialog,
										"Can't add Value to CodingScheme. \n Only Field can be added to CodinghScheme.",
										"Information dialog",
										JOptionPane.INFORMATION_MESSAGE);
					} else if (parentObject instanceof SS3FieldValue
							&& obj instanceof SS3FieldValue) {
						//
						System.err
								.println("--- associate value with value ---");
						//
						SS3FieldValue value = (SS3FieldValue) obj;
						SS3FieldValue parentValue = (SS3FieldValue) parentObject;
						String legacyID = parentValue.getLegacyID();

						//
						System.err.println("legacyID = " + legacyID);
						//

						if (legacyID == null || legacyID.length() == 0) {
							parentValue.setLegacyID(value.getId());
							dtde.dropComplete(true);
						} else {
							// value can be associated only with one legacy
							// value.
							// can't associate because selected value is already
							// associated.
							JOptionPane.showMessageDialog(parentDialog,
									"Can't associte with selected value.",
									"Information dialog",
									JOptionPane.INFORMATION_MESSAGE);

							dtde.dropComplete(false);
						}
					} else if (parentObject instanceof SS3Field
							&& obj instanceof SS3FieldValue) {

						SS3FieldValue value = (SS3FieldValue) obj;

						SS3Field field = (SS3Field) parent.getUserObject();
						SS3FieldValue exValue = field.getFieldValue(value
								.getId());

						if (exValue != null) {
							String newKey = Util.getLongUniqueNumber();
							value.setLegacyID(value.getId());
							value.setId(newKey);
						}
                  //
                  System.out.println("update from TreeDropTarget: value = " + value);
                  //
						field.addValue(value);

						DefaultMutableTreeNode node = new DefaultMutableTreeNode(
								value);
						DefaultTreeModel model = (DefaultTreeModel) tree
								.getModel();
						model.insertNodeInto(node, parent, 0);

						dtde.dropComplete(true);
						return;
					} else if (parentObject instanceof SS3CodingScheme
							&& obj instanceof SS3Field) {

						SS3Field field = (SS3Field) obj;
						SS3CodingScheme cs = (SS3CodingScheme) parentObject;
						SS3Field exField = cs.getField(field.getId());

						if (exField != null) {
							field.setLegacyID(field.getId());
							field.setId(Util.getLongUniqueNumber());
						}

						cs.addField(field);

						DefaultMutableTreeNode node = new DefaultMutableTreeNode(
								field);
						ArrayList values = field.getValuesAsList();

						for (int j = 0; j < values.size(); j++) {
							SS3FieldValue value = (SS3FieldValue) values.get(j);
							DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(
									value);
							node.add(childNode);
						}

						DefaultTreeModel model = (DefaultTreeModel) tree
								.getModel();
						model.insertNodeInto(node, parent, 0);
						dtde.dropComplete(true);
						return;
					}
				}
			}
			dtde.rejectDrop();
		} catch (Exception e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			dtde.rejectDrop();
		}
	}

	public SS3CodingScheme getSS3CodingScheme() {
		DefaultTreeModel model = (DefaultTreeModel) targetTree.getModel();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
		SS3CodingScheme cScheme = (SS3CodingScheme) root.getUserObject();
		return cScheme;
	}
}
