package edu.bu.signstream.common.db;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.MediaFile;
import edu.bu.signstream.common.util.vo.Profile;
import edu.bu.signstream.common.util.vo.Segment;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.Utterance;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DBProfile;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Selection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.participants.Background;
import edu.bu.signstream.common.util.vo.ss3.participants.Education;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SS2toSS3Adaptor {

	private ParticipantsLibrary prtLibrary = new ParticipantsLibrary();
	private HashMap convertedMediaFiles = new HashMap();
	private HashMap ss3Databases = new HashMap();
	private SS3DBProfile ss3Profile = new SS3DBProfile();
	private String fileName = "";

	public SS2toSS3Adaptor(SignStreamDocument ss2Document) {
		this.fileName = ss2Document.getFileName();

		SS3SignStreamApplication.logger.log(Level.INFO,
				"Loading SignStream2 to SignStream3 database adaptor...");

		HashMap codingScheme = ss2Document.getCodingScheme();

		HashMap mediaFiles = ss2Document.getMediaFiles();
		convertMediaFiles(mediaFiles);

		HashMap utterances = ss2Document.getUtterances();
		convertUtterances(utterances);

		HashMap participants = ss2Document.getParticipants();
		convertParticipantsLibrary(participants);

		Profile profile = ss2Document.getProfile();

		ss3Profile.setDatabaseVersion(profile.getDatabaseVersion());
		ss3Profile.setDistributor(profile.getDistributor());
		ss3Profile.setSource(profile.getSourcePath());
		ss3Profile.setSignStreamVersion(profile.getSignStreamVersion());
	}

	public HashMap getSS3Databeses() {
		return ss3Databases;
	}

	private void convertParticipantsLibrary(HashMap participants) {
		Set keys = participants.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {

			String key = (String) it.next();
			Object obj = participants.get(key);

			edu.bu.signstream.common.util.vo.Participant participant = (edu.bu.signstream.common.util.vo.Participant) obj;

			SS3Participant ss3Participant = new SS3Participant();
			// ss3Participant.setAge(participant.getAge());
			Background background = new Background();
			Education education = new Education();
			//
			ArrayList bckgr = participant.getBackground();
			// System.out.println("background = ");
			for (int i = 0; i < bckgr.size(); i++) {
				String str = (String) bckgr.get(i);
				// System.out.println("i = " + i + ", " + str);
			}
			//
			background.setEducation(education);
			// ss3Participant.setBackground(background);
			// ss3Participant.setDominantHand(participant.)

			// ss3Participant.setFirstName(participant.getName());
			// ss3Participant.setLastName();
			ss3Participant.setGender(participant.getGender());
			ss3Participant.setId("" + participant.getParticipantId());
			ss3Participant.setLabel(participant.getLabel());
			ArrayList languages = new ArrayList();
			languages.add(participant.getLanguage());
			// ss3Participant.setLanguages(languages);

			prtLibrary.addGlobalParticipant(ss3Participant);
		}
	}

	private void convertUtterances(HashMap utterances) {
		Set keys = utterances.keySet();
		Iterator it = keys.iterator();

		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = utterances.get(key);

			Utterance utterance = (Utterance) obj;

			SS3Database ss3DB = new SS3Database(utterance.getStartTimeInfo(),
					utterance.getEndTimeInfo());

			SS3GlossWindow window = new SS3GlossWindow();
			ss3DB.setSS3GlossWindow(window);
			SS3TemporalPartition partition = new SS3TemporalPartition();
			partition.setLabel("Temporal Partition");
			window.addTemporalPartition(partition);

			SS3GlossWindowSegment gSegment = new SS3GlossWindowSegment();
			gSegment.setLabel("Segment");
			partition.addTemporalPartitionSegment(gSegment);
			// sd
			// SS3GlossWindowItem item = new
			// SS3GlossWindowItem(partition.getLabel(), gSegment.getLabel());
			// item.setStartTimeInfo(utterance.getStartTimeInfo());
			// item.setEndTimeInfo(utterance.getEndTimeInfo());
			// item.setLabel(utterance.getExcerpt());

			String utteranceID = utterance.getId();
			while (utteranceID.length() < 3) {
				utteranceID = "0" + utteranceID;
			}

			ss3DB.setDirectory(fileName);
			ss3DB.setFileName(fileName + "_" + utteranceID + ".xml");

			int id = Integer.parseInt(utteranceID);
			ss3DB.setNumber("" + id);

			ArrayList segments = utterance.getSegments();

			// selection represents utteranse. Selection has one or more
			// statement
			// (number or statements equal number of segments)
			SS3Selection selection = new SS3Selection();
			selection.setLabel(utterance.getExcerpt());

			Excerpt excerpt = new Excerpt(utterance.getId(), utterance
					.getExcerpt());
			excerpt.setMediaFiles(utterance.getMediaFilesAsCollection());

			ArrayList mFiles = utterance.getMediaFilesAsCollection();
			for (int i = 0; i < mFiles.size(); i++) {
				Object mmm = mFiles.get(i);
				// System.out.println("MEDIA FILE = " + mmm);
				MediaFile mediaFile = (MediaFile) mmm;
				SS3MediaFile ss3MediaFile = (SS3MediaFile) convertedMediaFiles
						.get(mediaFile.getId());
				// //
				// System.err.println(
				// "get media file: id = "
				// + mediaFile.getId()
				// + ", ss3MediaFile = "
				// + ss3MediaFile);
				// //
				if (ss3MediaFile != null)
					ss3DB.addMediaFile(ss3MediaFile);
			}
			// create statement
			for (int i = 0; i < segments.size(); i++) {
				Segment segment = (Segment) segments.get(i);
				String participantID = segment.getParticipantId();

				// add tracks to statement
				HashMap tracks = segment.getTracks();
				SS3GlossWindowItem statement = getPopulatedSS3StatementsetTracksToSS3Database(
						tracks, ss3DB);

				gSegment.addSS3GlossWindowItem(statement);

				// private ArrayList<SS3StatementField> fields = new
				// ArrayList<SS3StatementField>();

				statement.setLabel(utterance.getExcerpt());

				// set participant ID
				statement.addParticipant(participantID);

				// add statement to selection
				selection.addStatementID(statement.getId());

				ss3DB.addStatement(statement);
				ss3DB.addSelection(selection);

				// ss3Databases.put(excerpt, ss3DB);
				ss3Databases.put(utterance.getId(), ss3DB);
			}
		}
	}

	private SS3GlossWindowItem getPopulatedSS3StatementsetTracksToSS3Database(
			HashMap tracks, SS3Database ss3DB) {

		SS3GlossWindowItem statement = new SS3GlossWindowItem("", "");
		statement.setId(Util.getLongUniqueNumber());

		Set set = tracks.keySet();
		Iterator it = set.iterator();
		double startTime = -1;
		double endTime = -1;
		while (it.hasNext()) {
			SS3StatementField statementField = new SS3StatementField();
			String key = (String) it.next();
			Object obj = tracks.get(key);

			edu.bu.signstream.common.util.vo.Track track = (edu.bu.signstream.common.util.vo.Track) obj;
			SS3Track ss3Track = convertTrackToSS3Track(track);

			double st = ss3Track.getStartTime();
			if (startTime == -1) {
				startTime = st;
			} else {
				startTime = startTime < st ? startTime : st;
			}

			double et = ss3Track.getEndTime();
			if (endTime == -1) {
				endTime = et;
			} else {
				endTime = endTime > et ? endTime : et;
			}

			ss3DB.addTrack(ss3Track);
			statementField.setId(ss3Track.getId());
			
			ArrayList entities = ss3Track.getEntities();
			ArrayList<SS3Entity> statementFieldEntitiesList = new ArrayList<SS3Entity>();
			for (int i = 0; i < entities.size(); i++) {
				SS3Entity entity = (SS3Entity) entities.get(i);
				statementFieldEntitiesList.add(entity);
			}
			statementField.setSS3Entities(statementFieldEntitiesList);

			statement.addField(statementField);
		}

		Time startTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(startTime), TimeUnit.TIMESTAMP);
//		TimeInfo startTimeInfo = new TimeInfo();
//		startTimeInfo.setUnitValue(startTime);

        Time endTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(endTime), TimeUnit.TIMESTAMP);
//		TimeInfo endTimeInfo = new TimeInfo();
//		endTimeInfo.setUnitValue(endTime);

		statement.setStartTimeInfo(startTimeInfo);
		statement.setEndTimeInfo(endTimeInfo);
		return statement;
	}

	private SS3Track convertTrackToSS3Track(Track track) {
		System.err.println("=========================== ");
		SS3Track nTrack = new SS3Track();
		nTrack.setId(track.getFieldId());

		ArrayList trackValues = track.getTrackValues();
		
//		// sort values by time
//		int[] timesToSort = new int[trackValues1.size()]; 
//		for (int j = 0; j < trackValues1.size(); j++) {
//			TrackValue tv = (TrackValue) trackValues1.get(j);
//			timesToSort[j] = tv.getStartTimeInfo().getMovieTime();
//		}
//		Sorter.sort(timesToSort);
//		
//		ArrayList trackValues = new ArrayList();
//		
//		for(int k = 0; k < timesToSort.length; k++) {
//			int t = timesToSort[k];
//			
//			for(int tt = 0; tt < trackValues1.size(); tt++) {
//				TrackValue tvv = (TrackValue) trackValues1.get(tt);
//				if(tvv.getEndTimeInfo().getMovieTime() == t) {
//					trackValues.add(tvv);
//					tt = trackValues1.size();
//				}
//			}
//		}
		
		// we do have only final holds in SignStream2; we will separate them in an array
		
		ArrayList<TrackValue> holds = new ArrayList<TrackValue>();
		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
		
		for (int ii = 0; ii < trackValues.size(); ii++) {
			TrackValue value = (TrackValue) trackValues.get(ii);
			int valueId = value.getValueId();
			if (valueId == Constants.LEGACY_OFFSET_CODE || valueId == Constants.LEGACY_HOLD_CODE) {
				holds.add(value);
				//
				System.err.println("HOLD VALUE = " + value.getStartTimeInfo().getFrameNumber());
				//
			} else {
				values.add(value);
			}
		}
		
//		for (int v = 0; v < values.size(); v++) {
//			TrackValue mvalue = values.get(v);
//			SS3Entity entity = new SS3Entity();
//			entity.setColor(Color.MAGENTA);
//			entity.setID(Util.getLongUniqueNumber());
//			SS3Item item = new SS3Item(mvalue);
//			item.setColor(Color.ORANGE);
//			entity.addItem(new SS3Item(mvalue));
//			int endTimeFrameNumber = mvalue.getEndTimeInfo().getFrameNumber();
//			
//			for(int h = 0; h < holds.size(); h++) {
//				TrackValue hold = holds.get(h);
//				if(hold.getStartTimeInfo().getFrameNumber() == endTimeFrameNumber) {
//					SS3Item holdItem = new SS3Item(hold);
//					entity.addItem(holdItem);
//					
//					h = holds.size();
//					holds.remove(hold);
//				}
//			}
//			
//			nTrack.addEntity(entity);
//		}

		for (int i = 0; i < trackValues.size(); i++) {
			TrackValue trackValue = (TrackValue) trackValues.get(i);
			int valueId = trackValue.getValueId();

			if (valueId != Constants.LEGACY_ONSET_CODE
					&& valueId != Constants.LEGACY_OFFSET_CODE
					&& valueId != Constants.LEGACY_HOLD_CODE) {
				TrackValue value = trackValue;

				SS3Entity entity = new SS3Entity();
				entity.setColor(Color.MAGENTA);
				entity.setID(Util.getLongUniqueNumber());
				SS3Item item = new SS3Item(value);
				item.setColor(Color.ORANGE);
				entity.addItem(new SS3Item(value));
/*
				if (i > 0) {
					TrackValue onset = (TrackValue) trackValues.get(i - 1);
					if (onset.getValueId() == Constants.ONSET_CODE) {
						SS3Item onsetItem = new SS3Item(onset);
						entity.addItem(onsetItem);
					}
				}

				if (i + 1 < trackValues.size()) {
					TrackValue offset = (TrackValue) trackValues.get(i + 1);
					if (offset.getValueId() == Constants.OFFSET_CODE
							|| offset.getValueId() == Constants.HOLD_CODE) {
						SS3Item offsetItem = new SS3Item(offset);
						entity.addItem(offsetItem);
						i++;
					}
				}
*/				
				//
				int endTimeFrameNumber = value.getEndTimeInfo().getFrameNumber();
				//
				//
				for(int h = 0; h < holds.size(); h++) {
					TrackValue hold = holds.get(h);
					if(hold.getStartTimeInfo().getFrameNumber() == endTimeFrameNumber) {
						SS3Item holdItem = new SS3Item(hold);
						entity.addItem(holdItem);
						
						h = holds.size();
						holds.remove(hold);
					}
				}
				//
				nTrack.addEntity(entity);
			}			
		}
	
		return nTrack;
	}

	private void convertMediaFiles(HashMap mediaFiles) {
		Set keys = mediaFiles.keySet();
		Iterator it = keys.iterator();

		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = mediaFiles.get(key);

			MediaFile mediaFile = (MediaFile) obj;
			SS3MediaFile ss3MediaFile = new SS3MediaFile();
			ss3MediaFile.setId(mediaFile.getId());
			ss3MediaFile.setFile(mediaFile.getPath());

			convertedMediaFiles.put(ss3MediaFile.getId(), ss3MediaFile);
		}
	}

	public SS3Database loadSS2Database(Excerpt excerpt) {
		int dbSize = ss3Databases.size();
		String id = excerpt.getID();
		SS3Database db = (SS3Database) ss3Databases.get(id);

		return db;
	}
}
