package edu.bu.signstream.common.util;

import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

/**
 * This class provides useful methods for setting the appearance 
 * of components on the UI.
 */
public class UserInterfaceUtil {
	private static Font dialogTitleFont = null;
	private static Color dialogTitleColor = null;

	public static int ALL_FILES_FILTER = -1;
	public static int VIDEO_FILES_FILTER = 0;
	public static int GRAPH_FILES_FILTER = 1;
    public static int AUDIO_FILES_FILTER = 2;
    public static int TEXTGRID_FILES_FILTER = 3;

	public static Font getDialogTitleFont() {
		if (dialogTitleFont == null)
			dialogTitleFont = new Font("Times-Roman", Font.BOLD + Font.ITALIC,
					16);
		return dialogTitleFont;
	}

	public static void setGridBagConstraints(GridBagConstraints gbc, int gridx,
			int gridy, int gridwidth, int gridheight, int fill) {
		gbc.gridx = gridx;
		gbc.gridy = gridy;
		gbc.gridwidth = gridwidth;
		gbc.gridheight = gridheight;
		gbc.fill = fill;
	}

	public static Color getDialogTitleColor() {
		if (dialogTitleColor == null)
			dialogTitleColor = new Color(40, 61, 112);
		return dialogTitleColor;
	}
	
	public static void setCheckBoxLNF(JCheckBox cb) {
		Font cbFont = new Font(cb.getFont().getName(), Font.PLAIN, 10);
		cb.setFont(cbFont);
	}
	
	public static void setRadioBtnLNF(JRadioButton rb) {
		Font rbFont = new Font(rb.getFont().getName(), Font.PLAIN, 10);
		rb.setFont(rbFont);
		rb.setPreferredSize(rb.getMinimumSize());
		rb.setMaximumSize(rb.getMinimumSize());
		rb.setBorder(BorderFactory.createLineBorder(Color.red));
	}
	
	public static void setComboBoxLNF(JComboBox cb) {
		Font lblFont = new Font(cb.getFont().getName(), Font.PLAIN, 10);
		FontMetrics lblFontMetrics = cb.getFontMetrics(lblFont);
		
		int addLblHeight = lblFontMetrics.getHeight();
		int addLblWidth = 75;
		
		if (cb.getSelectedItem() != null)
			addLblWidth = lblFontMetrics.stringWidth(cb.getSelectedItem().toString());

		cb.setFont(lblFont);

		Dimension addLblMinSize = new Dimension(addLblWidth + 10,
				addLblHeight + 5);
		cb.setMaximumSize(addLblMinSize);
		cb.setPreferredSize(addLblMinSize);
	}
	
	public static void setDropDownButtonsLNF(JButton btn) {
		Font btnFont = new Font(btn.getFont().getName(), Font.PLAIN, 11);
		btn.setFont(btnFont);				
		btn.setIcon(new DropDownIcon());
		btn.setVerticalTextPosition(SwingConstants.CENTER);
		btn.setHorizontalTextPosition(SwingConstants.LEFT);
		btn.setBorder(BorderFactory.createLineBorder(Color.GRAY));
	}
	
	public static void setDropDownButtonNoBorderLNF(JButton btn) {		
		btn.setIcon(new DropDownIcon2());
		btn.setVerticalTextPosition(SwingConstants.CENTER);
		btn.setHorizontalTextPosition(SwingConstants.RIGHT);
		//btn.setBorder(BorderFactory.createEmptyBorder());
		Insets insets = btn.getMargin();
		
		btn.setMargin(new Insets(insets.bottom,0, insets.right, insets.top));
		
		//Use the method setMargin and specify an Insets object with (bottom, left, right, top) margins.
		
		btn.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		
		Dimension d = btn.getMinimumSize(); //.getPreferredSize();
		
		btn.setPreferredSize(d);
		btn.setMaximumSize(d);
		btn.setMaximumSize(d);
	}
	
	public static void setMenuItemLNF(JComponent btn) {
		Font btnFont = new Font(btn.getFont().getName(), Font.PLAIN, 11);
		btn.setFont(btnFont);	
		btn.setBorder(BorderFactory.createLineBorder(UIPalette.getBackground()));
	}

	/**
	 * A method to set the look and feel of a button.
	 * @param btn - The JButton to set the look and feel of
	 */
	public static void setButtonsLNF(JButton btn) {
		Font btnFont = new Font(btn.getFont().getName(), Font.PLAIN, 10);
		FontMetrics btnFontMetrics = btn.getFontMetrics(btnFont);
		int addBtnHeight = btnFontMetrics.getHeight();
		int addBtnWidth = btnFontMetrics.stringWidth(btn.getText());

		btn.setFont(btnFont);
		btn.setBorder(BorderFactory.createLineBorder(Color.darkGray));

		Dimension addBtnMinSize = new Dimension(addBtnWidth + 10,
				addBtnHeight + 5);
		btn.setMaximumSize(addBtnMinSize);
		btn.setPreferredSize(addBtnMinSize);
		btn.setMaximumSize(addBtnMinSize);

		btn.setFocusable(false);
	}
	
	public static void setButtonsLargeFontLNF(JButton btn) {
		Font btnFont = new Font(btn.getFont().getName(), Font.PLAIN, 12);
		FontMetrics btnFontMetrics = btn.getFontMetrics(btnFont);
		int addBtnHeight = btnFontMetrics.getHeight();
		int addBtnWidth = btnFontMetrics.stringWidth(btn.getText());

		btn.setFont(btnFont);
		btn.setBorder(BorderFactory.createLineBorder(Color.darkGray));

		Dimension addBtnMinSize = new Dimension(addBtnWidth + 10,
				addBtnHeight + 5);
		btn.setMaximumSize(addBtnMinSize);
		btn.setPreferredSize(addBtnMinSize);
		btn.setMaximumSize(addBtnMinSize);
	}

	public static void setLabelLNF(JLabel label) {
		Font lblFont = new Font(label.getFont().getName(), Font.PLAIN, 11);
		label.setFont(lblFont);
		label.setText("  " + label.getText());
	}

	public static void setTextFieldLNF(JTextField txtField) {
		Font fldFont = new Font(txtField.getFont().getName(), Font.PLAIN, 11);
		txtField.setFont(fldFont);
	}

	public static void setTextAreaLNF(JTextArea txtField) {
		Font fldFont = new Font(txtField.getFont().getName(), Font.PLAIN, 11);
		txtField.setFont(fldFont);
	}

	public static void setListLNF(JList list) {
		Font font = new Font(list.getFont().getName(), Font.PLAIN, 11);
		list.setFont(font);
	}
	
	public static File selectDirectory(JFrame frame, File file, JComponent component,
			String title) {
		
		File selectedFile = null;

		if ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName())) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			String absPath = file.getAbsolutePath();
			fc.setSelectedFile(new File(absPath));

			int returnVal = fc.showOpenDialog(component);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				selectedFile = fc.getSelectedFile();
			}
		} else {
			System.setProperty("apple.awt.fileDialogForDirectories", "true");

			FileDialog fd = new FileDialog(frame, title, FileDialog.LOAD);
			String absPath = file.getAbsolutePath();
			fd.setDirectory(absPath);
			fd.setFile(absPath);

            SS3Singleton.getSignStreamApplication().getJMenuBar().setEnabled(false);
			fd.setVisible(true);
            SS3Singleton.getSignStreamApplication().getJMenuBar().setEnabled(true);

			if (fd.getFile() != null) {
				String selectedFileName = fd.getDirectory() + File.separator + fd.getFile();
				selectedFile = new File(selectedFileName);
			}
		}
		
		return selectedFile;
	}

	public static File selectFile(JFrame frame, File file, JComponent component, int filter, 
			String title) {
		
		File selectedFile = null;

		if ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName())) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			String absPath = file.getAbsolutePath();
			fc.setSelectedFile(new File(absPath));
			
			if(filter == UserInterfaceUtil.VIDEO_FILES_FILTER) {

                class VideoFileFilter1 extends javax.swing.filechooser.FileFilter {
                    @Override
                    public boolean accept(File file) {
                        String filename = file.getName();
                        return (filename.endsWith(".mov") || filename.endsWith(".mpg") || filename.endsWith("avi") || filename.endsWith(".mp4"));
                    }

                    @Override
                    public String getDescription() {
                        return "*.mov, *.mpg, *.mp4, *.avi";
                    }
                }

                fc.addChoosableFileFilter(new VideoFileFilter1());
            } else if (filter == UserInterfaceUtil.AUDIO_FILES_FILTER) {

                class AudioFilter extends javax.swing.filechooser.FileFilter {
                    @Override
                    public boolean accept(File file) {
                        String filename = file.getName();
                        return filename.endsWith(".wav")
                            || filename.endsWith(".mov")
                            || filename.endsWith(".mpg")
                            || filename.endsWith("avi")
                            || filename.endsWith(".mp4");
                    }

                    @Override
                    public String getDescription() {
                        return "*.wav, *.mov, *.mpg, *.mp4, *.avi";
                    }
                }

                fc.addChoosableFileFilter(new AudioFilter());
			} else if (filter == UserInterfaceUtil.GRAPH_FILES_FILTER) {
 				
 			    class VideoFileFilter1 extends javax.swing.filechooser.FileFilter {
 			        @Override
 					public boolean accept(File file) {
 			            String filename = file.getName();
 			            return (filename.endsWith(".txt"));
 			        }
 			        @Override
 					public String getDescription() {
 			            return "*.txt";
 			        }
 			    }
 			    
 			    fc.addChoosableFileFilter(new VideoFileFilter1());

            } else if (filter == UserInterfaceUtil.TEXTGRID_FILES_FILTER) {

                class TextGridFilter extends javax.swing.filechooser.FileFilter {
                    @Override
                    public boolean accept(File file) {
                        String filename = file.getName();
                        return filename.endsWith(".TextGrid");
                    }

                    @Override
                    public String getDescription() { return "*.TextGrid"; }
                }

                fc.addChoosableFileFilter(new TextGridFilter());
            }

			int returnVal = fc.showOpenDialog(component);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				selectedFile = fc.getSelectedFile();
			}
		} else {
			System.setProperty("apple.awt.fileDialogForDirectories", "false");

			FileDialog fd = new FileDialog(frame, title, FileDialog.LOAD);
			String absPath = file.getAbsolutePath();
			fd.setDirectory(absPath);
			fd.setFile(absPath);

			if(filter == UserInterfaceUtil.VIDEO_FILES_FILTER) {
                fd.setFilenameFilter(new java.io.FilenameFilter() {
                    @Override
                    public boolean accept(File dir, String filename) {
                        return (filename.endsWith(".mov") || filename.endsWith(".mpg") || filename.endsWith("avi") || filename.endsWith(".mp4"));
                    }
                });
            } else if (filter == UserInterfaceUtil.AUDIO_FILES_FILTER) {
                fd.setFilenameFilter(new java.io.FilenameFilter() {
                    @Override
                    public boolean accept(File dir, String filename) {
                        return filename.endsWith(".wav")
                            || filename.endsWith(".mov")
                            || filename.endsWith(".mpg")
                            || filename.endsWith("avi")
                            || filename.endsWith(".mp4");
                    }
                });
			} else if (filter == UserInterfaceUtil.GRAPH_FILES_FILTER) {
				fd.setFilenameFilter(new java.io.FilenameFilter() {
					@Override
					public boolean accept(File dir, String filename) {
						return (filename.endsWith(".txt"));
					}
				});
			} else if (filter == UserInterfaceUtil.TEXTGRID_FILES_FILTER) {
			    fd.setFilenameFilter((dir, name) -> name.endsWith(".TextGrid"));
            }

            SS3Singleton.getSignStreamApplication().getJMenuBar().setEnabled(false);
			fd.setVisible(true);
            SS3Singleton.getSignStreamApplication().getJMenuBar().setEnabled(true);

			if (fd.getFile() != null) {
				String selectedFileName = fd.getDirectory() + File.separator + fd.getFile();
				selectedFile = new File(selectedFileName);
			}
		}
		
		return selectedFile;
	}
}

class DropDownIcon implements Icon {   
    private int width = 11;
    private int height = 15;
    
    public DropDownIcon() {
    	super();
    }
    
    @Override
	public int getIconWidth() {
        return width;
    }
    
    @Override
	public int getIconHeight() {
        return height;
    }

	@Override
	public void paintIcon(java.awt.Component c, Graphics g, int x, int y) {
		Graphics2D g2d = (Graphics2D) g.create();  
		int x1 = x-3;
		g2d.drawLine(x1+2, 6, x1+6, 10);
		g2d.drawLine(x1+6, 10, x1+10, 6);
		g2d.drawLine(x1+2, 6, x1+10, 6);	
        g2d.dispose();		
	}
}

class DropDownIcon1 implements Icon {   
    private int width = 11;
    private int height = 15;
    
    public DropDownIcon1() {
    	super();
    }
    
    @Override
	public int getIconWidth() {
        return width;
    }
    
    @Override
	public int getIconHeight() {
        return height;
    }

	@Override
	public void paintIcon(java.awt.Component c, Graphics g, int x, int y) {
		Graphics2D g2d = (Graphics2D) g.create(); 
		g2d.setColor(Color.DARK_GRAY);
		int x1 = x;
		Triangle triangle = new Triangle(new Point(x1+1, 3), new Point(x1+6, 9), new Point(x1+12, 3));
		g2d.fill(triangle);		
        g2d.dispose();		
	}
}

class DropDownIcon2 implements Icon {   
    private int width = 11;
    private int height = 15;
    
    public DropDownIcon2() {
    	super();
    }
    
    @Override
	public int getIconWidth() {
        return width;
    }
    
    @Override
	public int getIconHeight() {
        return height;
    }

	@Override
	public void paintIcon(java.awt.Component c, Graphics g, int x, int y) {
		Graphics2D g2d = (Graphics2D) g.create(); 
		g2d.setColor(Color.DARK_GRAY);
		int x1 = x;
		Triangle triangle = new Triangle(new Point(x1+1, 14), new Point(x1+1, 3), new Point(x1+6, 9));
		g2d.fill(triangle);		
        g2d.dispose();		
	}
}


class Triangle implements Shape {
	
  private Polygon poly;

  public Triangle(Polygon p) {
    poly = p;
  }

  public Triangle(Point p1, Point p2, Point p3) {
    poly = new Polygon();
    poly.addPoint( p1.x, p1.y );
    poly.addPoint( p2.x, p2.y );
    poly.addPoint( p3.x, p3.y );
  }
  
  public void draw(Graphics g) {
    g.drawPolygon( poly );
  }

  public void fill(Graphics g) {
    g.fillPolygon( poly );
  }

  //methods implemented from interface Shape

  @Override
public Rectangle getBounds()
  {
    return poly.getBounds();
  }

  @Override
public Rectangle2D getBounds2D()
  {
    return poly.getBounds2D();
  }

  @Override
public boolean contains(double x, double y)
  {
    return poly.contains(x, y);
  }

  @Override
public boolean contains(Point2D p)
  {
    return poly.contains(p);
  }

  @Override
public boolean intersects(double x, double y, double w, double h)
  {
    return poly.intersects(x, y, w, h);
  }

  @Override
public boolean intersects(Rectangle2D r)
  {
    return poly.intersects(r);
  }

  @Override
public boolean contains(double x, double y, double w, double h)
  {
    return poly.contains(x, y, w, h);
  }

  @Override
public boolean contains(Rectangle2D r)
  {
    return poly.intersects(r);
  }

  @Override
public PathIterator getPathIterator(AffineTransform at)
  {
    return poly.getPathIterator(at);
  }

  @Override
public PathIterator getPathIterator(AffineTransform at, double flatness)
  {
    return poly.getPathIterator(at, flatness);
  }

}

