package edu.bu.signstream.common.util.signbank;

public class SignBankResource {
	
	private final int INVALID_PRIORITY = -999;
	
	private int priority;
	
	private String signBankFileName = null;
	
	private String signBankAbsolutePath = null;
	
	protected boolean nonEditableResource = false;
	
	private String signBankAuthor = null;
	private String dateCreated = null;
	private String dateModified = null;
	private String redirect = null;
	private String schemaVersion = null;
	private String dataVersion = null;
	
	public SignBankResource(String signBankFileName, String signBankAbsolutePath) {
		this.priority = this.INVALID_PRIORITY;
		this.signBankFileName = signBankFileName;
		this.signBankAbsolutePath = signBankAbsolutePath;
	}
	
	public String getSignBankFileName() {
		return this.signBankFileName;
	}
	
	public String getSignBankAbsolutePath() {
		return this.signBankAbsolutePath;
	}
	
	public boolean isNonEditableResource() {
		return this.nonEditableResource;
	}
	
	/**
	 * Set the priority of this SignBankResource.
	 * This method is intended to be called only by SignBankResourceCollection.setResourcesList(...)
	 * @param priority The priority to reset the SignBankResource to
	 */
	protected void setPriority(int priority) {
		this.priority = priority;
	}
	
	/**
	 * Get the priority of this SIgnBankResource.
	 * This method is intended to be called only by SIgnBankResource.ResourceComparator.compare(...)
	 * @return The priority of this resource
	 */
	protected int getPriority() {
		return this.priority;
	}
	
	/**
	 * Check whether this resource is valid as determined by whether its priority has been properly set
	 * @return A boolean representing whether this resource is valid
	 */
	protected boolean isValidPriority() {
		return this.priority != this.INVALID_PRIORITY;
	}
	
	public String getSignBankAuthor() {
		return this.signBankAuthor;
	}
	
	public void setSignBankAuthor(String signBankAuthor) {
		this.signBankAuthor = signBankAuthor;
	}
	
	public String getDateCreated() {
		return this.dateCreated;
	}
	
	public void setDateCreated(String dateCreated) {
		this.dateCreated = dateCreated;
	}
	
	public String getDateModified() {
		return this.dateModified;
	}
	
	public void setDateModified(String dateModified) {
		this.dateModified = dateModified;
	}
	
	public String  getRedirect() {
		return this.redirect;
	}
	
	public void setRedirect(String redirect) {
		this.redirect = redirect;
	}

	public String getSchemaVersion() {
		return this.schemaVersion;
	}

	public String getDataVersion() {
		return this.dataVersion;
	}

	public void setCurrentSchemaVersion(String resourceVersion) {
		this.schemaVersion = resourceVersion;
	}

	public void setCurrentDataVersion(String dataVersion) {
		this.dataVersion = dataVersion;
	}


	
	@Override
	public SignBankResource clone() {
		SignBankResource clone = new SignBankResource(this.signBankFileName, this.signBankAbsolutePath);

		clone.priority = this.priority;
		clone.nonEditableResource = this.nonEditableResource;
		clone.signBankAuthor = this.signBankAuthor;
		clone.dateCreated = this.dateCreated;
		clone.dateModified = this.dateModified;
		clone.redirect = this.redirect;
		clone.schemaVersion = this.schemaVersion;
		clone.dataVersion = this.dataVersion;

		return clone;
	}
	
	@Override
	public String toString() {
		String ret = "SignBankResource[";
		ret += "SignBankFileName=" + this.signBankFileName;
		ret += ", Priority=" + this.priority;
		ret += ", SignBankAbsolutePath=" + this.signBankAbsolutePath;
		ret += ", Editabable=" + !this.nonEditableResource;
		ret += "]";
		return ret;
	}
}
