package edu.bu.signstream.common.util.vo.ss3.db;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.apexField.ApexEntityProperties;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEntityProperties;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

public class SS3Entity {

	private String id = "";
	private String refEntityId = "";
	@Deprecated
	private String participantID = "";
	@Deprecated
	private String ref = "";
	private String startCompoundEntityID = "";
	private String endCompoundEntityID = "";
	private ArrayList<String> parentEntityIDs = new ArrayList<String>();

	private boolean lockedGlossPhoneAlignments = false;
	private boolean isLockedDomNdomAlignments = false;
	private boolean lockedGesturePhoneAlignments = false;
	private boolean lockedApexPhoneAlignments = false;
	
	private String dependentApexEntityID;
	private String dependentHandEntityID;
	private String dependentHandShapeEntityID;
	private String dependentOrientationFingerTipEntityID;
	private String dependentOrientationPalmEntityID;
	private String dependentGlossEntityID;
	private String dependentGestureEntityID;

	private String dependentLocationEntityID;
	private String dependentSignTypeEntityID;
	private String dependentGesturePropertyEntityID;
	
	private boolean incompleteGesture = false;
	private boolean showUniqueNumber = false;
	private String uniqueNumber;
	
	private String primaryEntry = null;
	
	private boolean validTime = false;	// for now, only applies to free text field (id 30000)
	
	public boolean isLockedGlossPhoneAlignments() {
		return lockedGlossPhoneAlignments;
	}

	public void setLockedGlossPhoneAlignments(boolean lockedGlossPhoneAlignments) {
		this.lockedGlossPhoneAlignments = lockedGlossPhoneAlignments;
	}

	public boolean isLockedDomNdomAlignments() {
		return isLockedDomNdomAlignments;
	}

	public void setLockedDomNdomAlignments(boolean isLockedDomNdomAlignments) {
		this.isLockedDomNdomAlignments = isLockedDomNdomAlignments;
	}
	
	public String getDependentApexEntityID() {
		return dependentApexEntityID;
	}

	public void setDependentApexEntityID(String dependentApexEntityID) {
		this.dependentApexEntityID = dependentApexEntityID;
	}

	public String getDependentHandEntityID() {
		return dependentHandEntityID;
	}

	public void setDependentHandEntityID(String dependentHandEntityID) {
		this.dependentHandEntityID = dependentHandEntityID;
	}

	public String getDependentHandShapeEntityID() {
		return dependentHandShapeEntityID;
	}

	public void setDependentHandShapeEntityID(String dependentHandShapeEntityID) {
		this.dependentHandShapeEntityID = dependentHandShapeEntityID;
	}
	
	public String getDependentOrientationFingerTipEntityID() {
		 return dependentOrientationFingerTipEntityID;
	}

	public String getDependentOrientationPalmEntityID() {
		return dependentOrientationPalmEntityID;
	}
	
	public void setDepenentOrientationFingerTipEntityID(String dependentOrientationFingerTipEntityIDEntityID) {
		this.dependentOrientationFingerTipEntityID = dependentOrientationFingerTipEntityIDEntityID;
	}

	public void setDependentOrientationPalmEntityID(String dependentOrientationPalmEntityID) {
		this.dependentOrientationPalmEntityID = dependentOrientationPalmEntityID;
	}

	public String getDependentGlossEntityID() {
		return dependentGlossEntityID;
	}

	public void setDependentGlossEntityID(String dependentGlossEntityID) {
		this.dependentGlossEntityID = dependentGlossEntityID;
	}
	
	public String getDependentGestureEntityID() {
		return dependentGestureEntityID;
	}

	public void setDependentGestureEntityID(String dependentGestureEntityID) {
		this.dependentGestureEntityID = dependentGestureEntityID;
	}

	public String getDependentLocationEntityID() {
		return dependentLocationEntityID;
	}

	public void setDependentLocationEntityID(String dependentLocationEntityID) {
		this.dependentLocationEntityID = dependentLocationEntityID;
	}
	
	public String getDependentSignTypeEntityID() {
		return dependentSignTypeEntityID;
	}

	public void setDependentSignTypeEntityID(String dependentSignTypeEntityID) {
		this.dependentSignTypeEntityID = dependentSignTypeEntityID;
	}
	
	public String getDependentGesturePropertyEntityID() {
		return dependentGesturePropertyEntityID;
	}
	
	public void setDependentGesturePropertyEntityID(String dependentGesturePropertyEntityID) {
		this.dependentGesturePropertyEntityID = dependentGesturePropertyEntityID;
	}

	@Override
	public String toString() {
		return "Entity ID = " + id + ", startTime = " + getStartTime() + ", endTime = " + getEndTime();
	}

	private Color color = null;

	LinkedHashMap<Integer, SS3Item> items = new LinkedHashMap<Integer, SS3Item>();

	private GlossEntityProperties properties = null;

	public GlossEntityProperties getGlossEntityProperties() {
		return properties;
	}

	public void setGlossEntityProperties(GlossEntityProperties properties) {
		this.properties = properties;
	}
	
	private GestureEntityProperties gestureProperties = null;

	public GestureEntityProperties getGestureEntityProperties() {
		return gestureProperties;
	}

	public void setGestureEntityProperties(GestureEntityProperties properties) {
		this.gestureProperties = properties;
	}
	
	private ApexEntityProperties apexProperties = null;

	public ApexEntityProperties getApexEntityProperties() {
		return apexProperties;
	}

	public void setApexEntityProperties(ApexEntityProperties apexProperties) {
		this.apexProperties = apexProperties;
	}

	public void setID(String id) {
		this.id = id;
	}

	public String getID() {
		if (id == null || id.length() == 0)
			id = "" + Util.getLongUniqueNumber();
		return id;
	}

	public void setRefEntityId(String refEntityId) {
		this.refEntityId = refEntityId;
	}

	public String getRefEntityId() {
		return refEntityId;
	}

	public void setReference(String ref) {
		this.ref = ref;
	}

	public String getReference() {
		return ref;
	}
	
	public void setParentEntityIDs(ArrayList<String> parentEntityIDs) {
		this.parentEntityIDs = (ArrayList<String>) parentEntityIDs.clone();
	}
	
	public void setParentEntities(ArrayList<PresentationEventsEntity> parentEntities) {
		ArrayList<String> parentEntityIDs = new ArrayList<String>();
		for (PresentationEventsEntity parentEntity: parentEntities) {
			parentEntityIDs.add(parentEntity.getID());
		}
		this.setParentEntityIDs(parentEntityIDs);
	}
	
	public ArrayList<String> getParentEntityIDs() {
		return parentEntityIDs;
	}

	// GD: TODO: this is the same as getItems() below - condense to one func
	public ArrayList<SS3Item> getItemsAsList() {
		ArrayList<SS3Item> ret = new ArrayList<SS3Item>();
		items.values().forEach(item -> ret.add(item));
		return ret;
	}
	
	public ArrayList<SS3Item> getOrderedItemsAsList() {
		
		
		return null;
	}

	public ArrayList<SS3Item> getItems() {
		ArrayList<SS3Item> ret = new ArrayList<SS3Item>();
		items.values().forEach(item -> ret.add(item));
		return ret;
	}

	@Deprecated
	public String getParticipantID() {
		return participantID;
	}

	 // GD: commenting as unused 2021-07-15
//	public void setItems(ArrayList<SS3Item> items) {
//		this.items = items;
//	}

	@Deprecated
	public void setParticipantID(String string) {
		participantID = string;
	}

	public void addItem(SS3Item item) {
		int maxOrder = 0;
		if (items.size() > 0) {
			maxOrder = Collections.max(items.keySet());
		}
		
		int newMax = maxOrder + 1;
		items.put(newMax, item);
	}

	public double getStartTime() {
        return getItemsAsList().stream()
        .map(SS3Item::getStartTimeInfo)
        .reduce(Time::min).orElseThrow()
        .in(SS3Singleton.getVideoTimeRange()).as(TimeUnit.TIMESTAMP, Rounding.NEAREST);
    }
	
	public Time getStartTimeInfo() {
        return getItemsAsList().stream()
            .map(SS3Item::getStartTimeInfo)
            .reduce(Time::min).orElseThrow();
	}

	public double getEndTime() {
        return getItemsAsList().stream()
        .map(SS3Item::getEndTimeInfo)
        .reduce(Time::max).orElseThrow()
        .in(SS3Singleton.getVideoTimeRange()).as(TimeUnit.TIMESTAMP, Rounding.NEAREST);
    }
	
	public Time getEndTimeInfo() {
        return getItemsAsList().stream()
            .map(SS3Item::getEndTimeInfo)
            .reduce(Time::max).orElseThrow();
	}

	public Interval interval() {
		return getStartTimeInfo().to(getEndTimeInfo());
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public String getEndCompoundEntityID() {
		return endCompoundEntityID;
	}

	public void setEndCompoundEntityID(String string) {
		endCompoundEntityID = string;
	}

	public String getStartCompoundEntityID() {
		return startCompoundEntityID;
	}

	public void setStartCompoundEntityID(String string) {
		startCompoundEntityID = string;
	}
	
	public boolean isValidTime() {
		return validTime;
	}
	
	public void setValidTime(boolean validTime) {
		this.validTime = validTime;
	}

	public boolean isLockedGesturePhoneAlignments() {
		return lockedGesturePhoneAlignments;
	}

	public void setLockedGesturePhoneAlignments(boolean lockedGesturePhoneAlignments) {
		this.lockedGesturePhoneAlignments = lockedGesturePhoneAlignments;
	}
	
	public boolean isLockedApexPhoneAlignments() {
		return lockedApexPhoneAlignments;
	}
	
	public boolean isIncompleteGesture() {
		return incompleteGesture;
	}
	
	public void setIncompleteGesture(boolean incompleteGesture) {
		this.incompleteGesture = incompleteGesture;
	}

	public boolean getShowUniqueNumber() {
		return showUniqueNumber;
	}

	public void setShowUniqueNumber(boolean showUniqueNumber) {
		this.showUniqueNumber = showUniqueNumber;
	}

	public String getUniqueNumber() {
		return uniqueNumber;
	}

	public void setUniqueNumber(String uniqueNumber) {
		this.uniqueNumber = uniqueNumber;
	}

	public String getPrimaryEntry() {
		return primaryEntry;
	}

	public void setPrimaryEntry(String primaryEntry) {
		this.primaryEntry = primaryEntry;
	}
}
