package edu.bu.signstream.common.util.vo.ss3.db;

import java.util.*;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.time.TimeRange;

public class SS3GlossWindowItem extends SS3Element {

	private Time startTimeInfo;
	private Time endTimeInfo;
	private String label = "";
	private String noteID = "";
	private ArrayList<String> participantIDs = new ArrayList<String>();
	private ArrayList<String> anotatorIDs = new ArrayList<String>();

	private String partitionLabel = null;
	private String segmentLabel = null;
	private String utteranceLabel = "";

	private PresentationEvent event = null;
	
	// Collection of SS3StatementField objects
	private ArrayList<SS3StatementField> fields = new ArrayList<SS3StatementField>();

	// Segment level items. Must contain SS3StatementField collection.
	// Must be added to Wrappers and updated in updateDatabase method.
	private ArrayList<SS3GlossWindowItem> segmentItems = new ArrayList<SS3GlossWindowItem>();
	
	private Set<String> permanentlyVisibleFieldIDs = new HashSet<>();

	public SS3GlossWindowItem(String partitionLabel, String segmentLabel) {
		this.partitionLabel = partitionLabel;
		this.segmentLabel = segmentLabel;
	}

	public void addSS3GlossWindowItem(SS3GlossWindowItem item) {
		segmentItems.add(item);
	}

	public SS3GlossWindowItem getSS3GlossWindowItem(String id) {
		for (SS3GlossWindowItem item : segmentItems) {
			if (item.getEvent().getID().equalsIgnoreCase(id))
				return item;
		}
		return null;
	}

	public ArrayList<SS3GlossWindowItem> getSS3GlossWindowItems() {
		return segmentItems;
	}

	public PresentationEvent getEvent() {
		return event;
	}

	public void setEvent(PresentationEvent event) {
		this.event = event;
	}

	public void synchronize() {
		if (event != null) {
			this.startTimeInfo = event.getStartTimeInfo();
			this.endTimeInfo = event.getEndTimeInfo();
			this.setId(event.getID());
			this.setLabel(event.getText());
		}
	}

	public void setUtteranceLabel(String utteranceLabel) {
		this.utteranceLabel = utteranceLabel;
	}

	public String getUtteranceLabel() {
		return utteranceLabel;
	}

	public void setPartitionLabel(String partitionLabel) {
		this.partitionLabel = partitionLabel;
	}

	public String getPartitionLabel() {
		return partitionLabel;
	}

	public void setSegmentLabel(String segmentLabel) {
		this.segmentLabel = segmentLabel;
	}

	public String getSegmentLabel() {
		return segmentLabel;
	}

	@Override
	public SS3GlossWindowItem clone() {
		SS3GlossWindowItem item = new SS3GlossWindowItem(partitionLabel, segmentLabel);
		String label1 = label;
		item.setLabel(label1);
		item.setNoteID(noteID);
		item.setStartTimeInfo(startTimeInfo);
		item.setEndTimeInfo(endTimeInfo);

		ArrayList<String> clonedParticipants = new ArrayList<String>();
		for (String participant : participantIDs) {
			clonedParticipants.add(participant);
		}

		ArrayList<String> clonedCoders = new ArrayList<String>();
		for (String coder : anotatorIDs) {
			clonedCoders.add(coder);
		}

		ArrayList<SS3StatementField> statementFields = new ArrayList<SS3StatementField>();
		for (SS3StatementField field : fields) {
			statementFields.add(field);
		}

		item.setCoders(clonedCoders);
		item.setParticipants(clonedParticipants);
		item.setFields(statementFields);

		return item;
	}

	public String getNoteID() {
		return noteID;
	}

	public void setNoteID(String noteID) {
		this.noteID = noteID;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String string) {
		label = string;
	}

	public void setParticipants(ArrayList<String> participants) {
		this.participantIDs = participants;
	}

	public void addParticipant(String participantID) {
		participantIDs.add(participantID);
	}

	public void setCoders(ArrayList<String> coders) {
		this.anotatorIDs = coders;
	}

	public void setFields(ArrayList<SS3StatementField> fields) {
		this.fields = fields;
	}

	public void addField(SS3StatementField field) {
		fields.add(field);
	}

	public void removeAllFields() {
		fields.clear();
	}

	public ArrayList<String> getParticipants() {
		return participantIDs;
	}

	public ArrayList<String> setCoders() {
		return anotatorIDs;
	}

	public ArrayList<SS3StatementField> getStatementFields() {
		return fields;
	}

	public SS3StatementField getSS3StatementField(String fieldID) {
		for (int i = 0; i < fields.size(); i++) {
			SS3StatementField field = fields.get(i);
			if (fieldID.equalsIgnoreCase(field.getId())) {
				return field;
			}
		}
		return null;
	}

	public Time getEndTimeInfo() {
		return endTimeInfo;
	}

	public Time getStartTimeInfo() {
		return startTimeInfo;
	}

	public Interval interval() { return startTimeInfo.to(endTimeInfo); }

	public void setEndTimeInfo(Time info) {
		endTimeInfo = info;
	}

	public void setStartTimeInfo(Time info) {
		startTimeInfo = info;
	}

	public void resetTranslationStartTime(String parentID, Time time) {
		// need to do this utteranceView section, since the wrapper data will overwrite the track data in SignStreamSegmentPanel.updateLoadedDatabase()
		var uttView = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView();
		if (uttView != null) {
			ArrayList<Segment> segments = uttView.getSegments();

			for (Segment segment : segments) {
				RootLabelObject rootLabelObject = segment.getRootLabelObject();
				LabelObject glossLabelObject = rootLabelObject.getGlossLabelObject();
				LabelObject nonManualLabelObject = rootLabelObject.getNonManualLabelObject();
				LabelObject gestureLabelObject = rootLabelObject.getGestureLabelObject();
				LabelObject customLabelObject = rootLabelObject.getCustomLabelObject();

				ArrayList<LabelObject> allObjects = nonManualLabelObject.getAllChildren();
				allObjects.addAll(glossLabelObject.getAllChildren());
				allObjects.addAll(gestureLabelObject.getAllChildren());
				allObjects.addAll(customLabelObject.getAllChildren());

				for (LabelObject obj : allObjects) {
					var wrapper = obj.getWrapper();
					var field = wrapper.getField();
					if (field.getFieldID().equals(Util.FREE_TEXT_FIELD_ID)) {
						ArrayList<EventsEntity> entities = field.getEntities();
						if (entities != null && ! entities.isEmpty()) {
							for (EventsEntity entity: entities) {
								if (entity.getParentEntities().size() == 1) {
									if (entity.getParentEntities().get(0).getID().equals(parentID)) {
										entity.setStartTimeInfo(time);
									}
								}
							}
						}
					}
				}
			}
			
			return;
		}
		
	    var db = SS3Singleton.getMediaToolBar().segmentPanel().getSS3Database();
	    var track = db.getTrack(Util.FREE_TEXT_FIELD_ID);
	    for (var entity : track.getEntities()) {
	    		if (entity.getParentEntityIDs().size() == 1) {
		        if (entity.getParentEntityIDs().get(0).equals(parentID)) {
		            for (var item : entity.getItems()) {
		            	
		                item.setStartTimeInfo(time);
	                }
	            }
	    		}
        }
	}

	public void resetTranslationEndTime(String parentID, Time time) {
		// need to do this utteranceView section, since the wrapper data will overwrite the track data in SignStreamSegmentPanel.updateLoadedDatabase()
		var uttView = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView();
		if (uttView != null) {
			ArrayList<Segment> segments = uttView.getSegments();

			for (Segment segment : segments) {
				RootLabelObject rootLabelObject = segment.getRootLabelObject();
				LabelObject glossLabelObject = rootLabelObject.getGlossLabelObject();
				LabelObject nonManualLabelObject = rootLabelObject.getNonManualLabelObject();
				LabelObject gestureLabelObject = rootLabelObject.getGestureLabelObject();
				LabelObject customLabelObject = rootLabelObject.getCustomLabelObject();

				ArrayList<LabelObject> allObjects = nonManualLabelObject.getAllChildren();
				allObjects.addAll(glossLabelObject.getAllChildren());
				allObjects.addAll(gestureLabelObject.getAllChildren());
				allObjects.addAll(customLabelObject.getAllChildren());

				for (LabelObject obj : allObjects) {
					var wrapper = obj.getWrapper();
					var field = wrapper.getField();
					if (field.getFieldID().equals(Util.FREE_TEXT_FIELD_ID)) {
						ArrayList<EventsEntity> entities = field.getEntities();
						if (entities != null && ! entities.isEmpty()) {
							for (EventsEntity entity: entities) {
								if (entity.getParentEntities().size() == 1) {
									if (entity.getParentEntities().get(0).getID().equals(parentID)) {
										entity.setEndTimeInfo(time);
									}
								}
							}
						}
					}
				}
			}
			
			return;
		}
		
        var db = SS3Singleton.getMediaToolBar().segmentPanel().getSS3Database();
        var track = db.getTrack(Util.FREE_TEXT_FIELD_ID);
        for (var entity : track.getEntities()) {
        		if (entity.getParentEntityIDs().size() == 1) {
	            if (entity.getParentEntityIDs().get(0).equals(parentID)) {
	                for (var item : entity.getItems()) {
	                    item.setEndTimeInfo(time);
	                }
	            }
        		}
        }
	}

	@Deprecated
	public Time maximalStartTime() {
	    return fields.stream()
            .filter(f -> !f.getId().equals(Util.FREE_TEXT_FIELD_ID)) // ignore translation
            .map(f ->
                f.getSS3Entities().stream()
                    .map(SS3Entity::getStartTimeInfo)
                    .min(Time::compareTo)
                    .orElse(TimeRange.ABSOLUTE.end()))
            .min(Time::compareTo)
            .orElse(TimeRange.ABSOLUTE.end());
    }

	@Deprecated
    public Time minimalEndTime() {
        return fields.stream()
            .filter(f -> !f.getId().equals(Util.FREE_TEXT_FIELD_ID)) // ignore translation
            .map(f ->
                f.getSS3Entities().stream()
                    .map(SS3Entity::getEndTimeInfo)
                    .max(Time::compareTo)
                    .orElse(TimeRange.ABSOLUTE.zero()))
            .max(Time::compareTo)
            .orElse(TimeRange.ABSOLUTE.zero());
    }

	@Override
	public String toString() {
		return label;
	}
	
	public void addPermanentlyVisibleFieldAsID(String fieldID) {
		permanentlyVisibleFieldIDs.add(fieldID);
	}
	
	public void removePermanentlyVisibleFieldAsID(String fieldID) {
		permanentlyVisibleFieldIDs.remove(fieldID);
	}
	
	public void removeAllPermanentFields() {
		permanentlyVisibleFieldIDs.clear();
	}
	
	public Set<String> getPermanentVisibleFieldIDs() {
		return permanentlyVisibleFieldIDs;
	}
}
