package edu.bu.signstream.common.util.vo.ss3.db;

import java.util.*;
import java.util.stream.Collectors;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.Panel;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ViewMode;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.view.ViewDataManager;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentTemplate;

public class SS3GlossWindowSegment {

    public SS3GlossWindowSegment() {}

	private String ID = "";
	private String label = "";
	private boolean isDefault = false;
	private SS3Participant participant = null;
	private String segmentControlPanelIndexFieldID = null;

	private ArrayList<SS3GlossWindowItem> glossWindowItems = new ArrayList<SS3GlossWindowItem>();

	// Collection of SS3StatementField objects
	private ArrayList<SS3StatementField> fields = new ArrayList<SS3StatementField>();
	
	private Set<String> permanentlyVisibleFieldIDs = new HashSet<>();
	private Set<String> visiblePanelIDs = SS3Singleton.getSS3CodingSchemeCollection()
        .getDefaultCodingScheme()
        .getDefaultViewMode()
        .getPanels().stream()
        .map(p -> p.id)
        .collect(Collectors.toSet());

	private SS3GlossWindowItemNote note = new SS3GlossWindowItemNote();
	
	private int currentGestureCounter = 1;
	private boolean showAllGestureCounters = true;
	private boolean defaultShowGestureCounters = true;

	private ViewDataManager viewDataManager;

	public String getID() {
		return ID;
	}

	public void setID(String id) {
		ID = id;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
		
		for (SS3GlossWindowItem item: glossWindowItems) {
			item.setSegmentLabel(label);
		}
	}
	
	public boolean isDefault() {
		return isDefault;
	}
	
	/**
	 * Sets this segment tier to be default. It is using access level protected
	 * because it is intended only to be called by SS3TemporalPartition, 
	 * as part of a sequence in which all segment tiers are set simultaneously
	 * to ensure that one and only one are set to default.
	 * @param isDefault setting to either true for default or false for not default
	 */
	protected void setDefault(boolean isDefault) {
		this.isDefault = isDefault;
	}
	
	public SS3Participant getParticipant() {
		return participant;
	}
	
	public void setParticipant(SS3Participant participant) {
		this.participant = participant;
	}

	public boolean isDominantHandRight() {
		if (participant == null) return true;
		return participant.isDominantHandRight();
	}
	
	public String getParticipantLabel() {		
		return participant != null ? participant.getLabel() : "";
	}
	
	public void delete(EventsEntity entity) {
		Time startTimeInfo = entity.getStartTimeInfo();
		Time endTimeInfo =  entity.getEndTimeInfo();
		
		ArrayList<SS3GlossWindowItem> itemsToRemove = new ArrayList<SS3GlossWindowItem>();
		
		for(SS3GlossWindowItem item : glossWindowItems) {
			Time sMovieTimeInfo = item.getStartTimeInfo();
			Time eMovieTimeInfo = item.getEndTimeInfo();
			
			if(sMovieTimeInfo.equals(startTimeInfo) && eMovieTimeInfo.equals(endTimeInfo)) {
				itemsToRemove.add(item);				
			}
		}
				
		for(SS3GlossWindowItem itemToRemove : itemsToRemove) {
            var db = SS3Singleton.getMediaToolBar().segmentPanel().getSS3Database();
            var track = db.getTrack(Util.FREE_TEXT_FIELD_ID);
            for (var ss3entity : track.getEntities()) {
            		if (ss3entity.getParentEntityIDs().size() == 1) {
	                if (ss3entity.getParentEntityIDs().get(0).equals(entity.getID())) {
	                    track.getEntities().remove(ss3entity);
	
	                    var field = fields.stream()
	                        .filter(f -> f.getId().equals(Util.FREE_TEXT_FIELD_ID))
	                        .findFirst().orElseThrow();
	
	                    field.getSS3Entities().remove(ss3entity);
	
	                    break;
	                }
            		}
            }

			glossWindowItems.remove(itemToRemove);
		}
	}
	
	//public void addOrUpdateGlossWindowItem(SS3GlossWindowItem item) {
	//	item.getId();
	//}

	public void addSS3GlossWindowItem(SS3GlossWindowItem item) {
		// if item is added after this segment was initially created, the label may have changed
		item.setSegmentLabel(label);
		glossWindowItems.add(item);
		Collections.sort(glossWindowItems, new Comparator<SS3GlossWindowItem>() {
			@Override
			public int compare(SS3GlossWindowItem o1, SS3GlossWindowItem o2) {
				return o1.getStartTimeInfo().compareTo(o2.getStartTimeInfo());
			}
			
		});
	}

	public ArrayList<SS3GlossWindowItem> getSS3GlossWindowItems() {
		return glossWindowItems;
	}

	public void setItems(ArrayList<SS3GlossWindowItem> items) {
		this.glossWindowItems = items;
		Collections.sort(glossWindowItems, new Comparator<SS3GlossWindowItem>() {
			@Override
			public int compare(SS3GlossWindowItem o1, SS3GlossWindowItem o2) {
				return o1.getStartTimeInfo().compareTo(o2.getStartTimeInfo());
			}

		});
	}



	public SS3GlossWindowItem getSS3GlossWindowItem(String id) {
		for (SS3GlossWindowItem glossWindowItem : glossWindowItems) {
			if (glossWindowItem.getEvent().getID().equalsIgnoreCase(id))
				return glossWindowItem;
		}
		return null;
	}
	
	@Override
	public String toString() {
		String ret = "SS3GlossWindowSegment: participantLabel =  " + getParticipantLabel() + ", label = " + getLabel();
		return ret;
	}
	
	public SS3GlossWindowItemNote getNote() {
		return note;
	}

	public void setNote(SS3GlossWindowItemNote note) {
		this.note = note;
	}
	
	public String getIndexFieldID() {
		return this.segmentControlPanelIndexFieldID;
	}
	
	public void setIndexFieldID(String fieldID) {
		this.segmentControlPanelIndexFieldID = fieldID;
	}
	

	public void setFields(ArrayList<SS3StatementField> fields) {
		this.fields = fields;
	}

	public void addField(SS3StatementField field) {
		SS3StatementField existingField = getSS3StatementField(field.getId());
		if (existingField == null)
			fields.add(field);
		else {
			ArrayList<SS3Entity> existingEntities = existingField.getSS3Entities();
			existingEntities.addAll(field.getSS3Entities());
			existingField.setSS3Entities(existingEntities);
		}
	}

	public void removeAllFields() {
		fields.clear();
	}
	
	public void removeField(String fieldID) {
		int indexToRemove = -1;
		
		for (int i = 0; i < fields.size(); i++) {
			SS3StatementField field = fields.get(i);
			if (fieldID.equalsIgnoreCase(field.getId())) {
				indexToRemove = i;
			}
		}
		
		if (indexToRemove != -1)
			fields.remove(indexToRemove);
	}

	public ArrayList<SS3StatementField> getStatementFields() {
		return fields;
	}

	public SS3StatementField getSS3StatementField(String fieldID) {
		for (int i = 0; i < fields.size(); i++) {
			SS3StatementField field = fields.get(i);
			if (fieldID.equalsIgnoreCase(field.getId())) {
				return field;
			}
		}
		return null;
	}
	
	public void addPermanentFieldAsID(String fieldID) {
		permanentlyVisibleFieldIDs.add(fieldID);
	}
	
	public void removePermanentFieldAsID(String fieldID) {
		permanentlyVisibleFieldIDs.remove(fieldID);
	}
	
	public void removeAllPermanentFields() {
		permanentlyVisibleFieldIDs.clear();
	}
	
	public Set<String> getPermanentFieldIDs() {
		return permanentlyVisibleFieldIDs;
	}

	public void setVisiblePanelIDs(Set<String> ids) {
	    visiblePanelIDs = ids;
    }
	public Set<String> getVisiblePanelIDs() {
	    return visiblePanelIDs;
    }

	public int getCurrentGestureCounter() {
		return currentGestureCounter;
	}

	public void setCurrentGestureCounter(int counter) {
		this.currentGestureCounter = counter;
	}
	
	public void increaseCurrentGestureCounter() {
		currentGestureCounter++;
	}

	public boolean getShowAllGestureCounters() {
		return showAllGestureCounters;
	}

	public void setShowAllGestureCounters(boolean show) {
		showAllGestureCounters = show;
	}

	public void setViewDataManager(ViewDataManager viewDataManager) {
		this.viewDataManager = viewDataManager;
	}

	public ViewDataManager getViewDataManager() {
		return this.viewDataManager;
	}
	
	public ViewMode getViewMode() {
		// create view mode from visible panel ids saved
		
		var myVisiblePanelIDS = getVisiblePanelIDs();
		
		Set<Panel> myViewModeSet = new HashSet<Panel>();
		
		List<Panel> panels = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme().panels();
		
		for (Panel panel: panels) {
			if (myVisiblePanelIDS.contains(panel.id)) {
				myViewModeSet.add(panel);
			}
		}
		
		return new ViewMode("VIEW_MODE_ID", "VIEW_MODE_LABEL", myViewModeSet);
	}


	
	public void showAllGestureCounters() {
		for (SegmentTemplate template: SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
					.getUtteranceView().getSegmentTemplates()) {
			String segmentID = template.getSegment().getParentField().getSS3GlossWindowSegment().getID(); 	// man, this is roundabout
			if (segmentID != null && segmentID.equals(getID())) {
				
				FieldWrappersCollection fwc = template.getSpeechView().getFieldWrappersCollection();
				FieldWrapper domGestureFieldWrp = fwc.getFieldWrapperWithFieldID(Util.DOM_GESTURE_FIELD_ID);
				FieldWrapper ndomGestureFieldWrp = fwc.getFieldWrapperWithFieldID(Util.NDOM_GESTURE_FIELD_ID);

				for (EventsEntity ent: domGestureFieldWrp.getField().getAllEntities()) {
					GestureChainedEventsEntity gEnt = (GestureChainedEventsEntity) ent;
					gEnt.setShowUniqueNumber(showAllGestureCounters);
				}
				
				for (EventsEntity ent: ndomGestureFieldWrp.getField().getAllEntities()) {
					GestureChainedEventsEntity gEnt = (GestureChainedEventsEntity) ent;
					gEnt.setShowUniqueNumber(showAllGestureCounters);
				}
				
				break;
			}
		}
	}

	public boolean getDefaultShowGestureCounters() {
		return defaultShowGestureCounters;
	}

	public void setDefaultShowGestureCounters(boolean setDefault) {
		defaultShowGestureCounters = setDefault;
	}
	
	public boolean setGestureUniqueNumber(GestureChainedEventsEntity entity, int potentialNumber) {
		// This method is called when a gesture is created.  It checks to see if the potential number is already in use.
		int highestNum = potentialNumber;
		
		for (SegmentTemplate template: SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
				.getUtteranceView().getSegmentTemplates()) {
			String segmentID = template.getSegment().getParentField().getSS3GlossWindowSegment().getID(); 	// man, this is roundabout
			if (segmentID != null && segmentID.equals(getID())) {

				FieldWrappersCollection fwc = template.getSpeechView().getFieldWrappersCollection();
				FieldWrapper domGestureFieldWrp = fwc.getFieldWrapperWithFieldID(Util.DOM_GESTURE_FIELD_ID);
				FieldWrapper ndomGestureFieldWrp = fwc.getFieldWrapperWithFieldID(Util.NDOM_GESTURE_FIELD_ID);

				for (EventsEntity ent: domGestureFieldWrp.getField().getAllEntities()) {
					GestureChainedEventsEntity gestureEnt = (GestureChainedEventsEntity) ent;
					if (gestureEnt.equals(entity))
						continue;

                    if (gestureEnt.equals(((GestureEventsEntity)entity).getDependentGestureEntity()))
                        continue;

					int num = Integer.parseInt(gestureEnt.getUniqueNumber());
					if (potentialNumber == num) {
						return false;
					}
					
					if (num > highestNum)
						highestNum = num;
				}

				for (EventsEntity ent: ndomGestureFieldWrp.getField().getAllEntities()) {
					GestureChainedEventsEntity gestureEnt = (GestureChainedEventsEntity) ent;
					if (gestureEnt.equals(entity))
						continue;

					if (gestureEnt.equals(((GestureEventsEntity)entity).getDependentGestureEntity()))
					    continue;

					int num = Integer.parseInt(gestureEnt.getUniqueNumber());
					if (potentialNumber == num) {
						return false;
					}
					
					if (num > highestNum)
						highestNum = num;
				}

				break;
			}
		}

		String numberStr = String.valueOf(potentialNumber);
		entity.setUniqueNumber(numberStr);
		
		setCurrentGestureCounter(highestNum);
		increaseCurrentGestureCounter();
		
		return true;
	}

	@Override
	public boolean equals(Object obj) {
		return (obj instanceof SS3GlossWindowSegment) && ((SS3GlossWindowSegment) obj).ID.equals(ID);
	}
}
