package edu.bu.signstream.grepresentation.fields.customField;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.MouseDblRightClickedDelegate;
import edu.bu.signstream.grepresentation.fields.nonManualField.NewChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.panels.codingScheme.NewCSValuePanel;

public class CustomSectionFieldWrapper extends FieldWrapper {
	protected final NonManualField nmField;
	private MouseDblRightClickedDelegate rDblClickDelegate;
	private Point lastPoint = null;

	public CustomSectionFieldWrapper(NonManualField nmField, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(nmField, segmentPanel, timeRange);
		this.nmField = nmField;
		rDblClickDelegate = new MouseDblRightClickedDelegate(nmField);
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public void unselectAll() {
		nmField.unselect();
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	// @TODO: implement
	@Override
	public void addEventsEntity(EventsEntity entity) {
		nmField.addChainedEventEntity((ChainedEventsEntity) entity);
	}

	// select event or entity located at given point
	@Override
	public void mouseClicked(Point point, JPanel panel) {

		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		EventsEntity entity = nmField.getEntityAt(point.x);
		Event event = nmField.getEventAt(point.x);
		if (event != null) {
			event.select();
			mediaTb.selectEvent(event, entity, nmField);
		}
	}

	// not functional
	@Override
	public void mouseRightClicked(Point point) {
	}

	// not functional
	@Override
	public void mouseLeftClicked(Point point) {
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	// add new chained event
	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
//		PresentationField parentField = nmField.getParentField();
//		Event parentEvent = parentField.getEventAt(point.x);

//		// if outside the bounds of any utterance, do not proceed
//		if (parentEvent == null) return;

        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()));

        NewChainedEvent newEvent = new NewChainedEvent(time, segmentPanel);
		int numberOfEntities = nmField.getEntities().size();
		
		if (numberOfEntities == 0) {
			nmField.setNewChainedEvent(newEvent);
		} else {
			EventsEntity nextEntity = nmField.getNextEntity(time);
			EventsEntity prevEntity = nmField.getPreviousEntity(time);

			if (prevEntity != null && prevEntity.getEndTimeInfo().greaterThan(time)) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}

			newEvent.setNextEntity(nextEntity);
			newEvent.setPreviousEntity(prevEntity);

			nmField.setNewChainedEvent(newEvent);
		}

		SS3Singleton.getMediaPlayer().seek(time);
	}

	// change field ID
	@Override
	public void mouseDblLeftClicked(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
			final String fieldID = field.getFieldID();

			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

			SS3Field field = dbScheme.getField(fieldID);
			final ArrayList<SS3FieldValue> values = field.getValuesAsList();
			final JPopupMenu popup = new JPopupMenu();
			final ChainedEvent cEvent = (ChainedEvent) event;

			ChainedEventsEntity entity = (ChainedEventsEntity) SS3Singleton.getMediaToolBar(null)
					.selectedEntity();

			if (cEvent.equals(entity.getValue())) {
				for (int i = 0; i < values.size(); i++) {
					final SS3FieldValue value = values.get(i);

					final JLabel label = new JLabel(value.getName());
					label.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseClicked(MouseEvent arg0) {
							if (cEvent != null) {
								String id = value.getId();
								cEvent.setID(id);
								SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
								panel.repaint();
							}
							popup.setVisible(false);
						}

						@Override
						public void mouseEntered(MouseEvent arg0) {
							label.setForeground(Color.RED);
						}

						@Override
						public void mouseExited(MouseEvent arg0) {
							label.setForeground(Color.BLACK);
						}
					});

					popup.add(label);
				}

				final JLabel addNew = new JLabel("Add new value");
				addNew.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent arg0) {
						if (cEvent != null) {
							popup.setVisible(false);
							
							final JDialog popup1 = new JDialog();
							popup1.setUndecorated(true);
							popup1.setResizable(false);
							NewCSValuePanel valuePanel = new NewCSValuePanel(fieldID, cEvent,
									nmField.getSignStreamSegmentPanel(), popup1);

							popup1.setSize(valuePanel.getPreferredSize());
							popup1.add(valuePanel);
							Dimension d = popup1.getPreferredSize();
							int width = (int) d.getWidth();
							int x = (int) point.getX() - width / 2;
							int y = (int) point.getY();

							Point loc = new Point(x, y);
							SwingUtilities.convertPointToScreen(loc, panel);
							popup1.setLocation(loc);
							popup1.setVisible(true);

							// SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup1);

							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
							panel.repaint();
						}
					}

					@Override
					public void mouseEntered(MouseEvent arg0) {
						addNew.setForeground(Color.RED);
					}

					@Override
					public void mouseExited(MouseEvent arg0) {
						addNew.setForeground(Color.BLACK);
					}
				});

				popup.add(addNew);
				popup.setPreferredSize(popup.getPreferredSize());
				popup.show(panel, (int) point.getX(), (int) point.getY());
				popup.setVisible(true);

				SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
			}
		}
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
	}

	// add new event (hold. onset, offset) to selected entity
	public void mouseCtrlAltClicked(Point point, JPanel panel) {
		EventsEntity ent = nmField.getEntityAt(point.x);

		if (ent == null)
			return;

		ChainedEventsEntity entity = (ChainedEventsEntity) ent;

		if (isEntitySelected()) {
			rDblClickDelegate.entityDblClick(entity, point, panel);
		} else {
			Event event = nmField.getEventAt(point.x);
			ChainedEvent cEvent = (ChainedEvent) event;
			rDblClickDelegate.dblClick(cEvent, entity, point, panel);
		}
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		panel.setCursor(new Cursor(getCursorType(point)));

		Event event = nmField.getEventAt(point.x);
		if (event != null) {
			panel.setToolTipText(event.getToolTipText());
		} else {
			panel.setToolTipText("");
		}
	}

	public int getCursorType(Point point) {
		Event event = nmField.getSelectedEvent();

		if (event != null) {
			double startCoord = event.getStartTimeCoordinate();
			double endCoord = event.getEndTimeCoordinate();

			if (startCoord == endCoord) {
				if (startCoord - 3 <= point.x && point.x <= startCoord - 1) {
					MOVE_DIRECTION = MOVE_START_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else if (startCoord + 1 <= point.x && point.x <= startCoord + 3) {
					MOVE_DIRECTION = MOVE_END_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else {
					return Cursor.DEFAULT_CURSOR;
				}
			} else {
				if (startCoord - 3 < point.x && point.x < startCoord + 3) {
					MOVE_DIRECTION = MOVE_START_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else if (endCoord - 3 < point.x && point.x < endCoord + 3) {
					MOVE_DIRECTION = MOVE_END_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else {
					return Cursor.DEFAULT_CURSOR;
				}
			}
		}

		return Cursor.DEFAULT_CURSOR;
	}

	private boolean drawNewEvent(Point point, JPanel panel) {

		NewChainedEvent newEvent = nmField.getNewChainedEvent();
		if (newEvent != null) {
			double coord = point.x;

			EventsEntity nextEntity = newEvent.getNextEntity();
			EventsEntity prevEntity = newEvent.getPreviousEntity();

			if (prevEntity != null) {
				double startCoord = prevEntity.getEndTimeCoordinate();

				if (startCoord > coord) {
					coord = startCoord;
				}
			}

			if (nextEntity != null) {
				double endCoord = nextEntity.getStartTimeCoordinate();

				if (endCoord < coord) {
					coord = endCoord;
				}
			}

			newEvent.setEndTimeCoordinate(coord);

			return true;
		}
		return false;
	}

	@Override
	public void mouseDragged(Point point, JPanel panel) {
		if (lastPoint != null && lastPoint.x == point.x)
			return;

//		PresentationField parentField = nmField.getParentField();
//		Event parentEvent = parentField.getEventAt(point.x);

//		// if outside the bounds of any utterance, do not proceed
//		if (parentEvent == null) {
//			return;
//		}

//		// if outside the bounds of the starting utterance, do not proceed
//		if (lastPoint != null) {
//			Event lastParentEvent = parentField.getEventAt(lastPoint.x);
//
//			if (lastParentEvent != null && lastParentEvent != parentEvent)
//				return;
//		}

		lastPoint = point;

		boolean drawNewEvent = drawNewEvent(point, panel);

		if (!drawNewEvent) {
			ChainedEventsEntity entity = (ChainedEventsEntity) nmField.getSelectedEntity();
			if (entity != null) {

				CoordinateTimeConverter convertor = segmentPanel.getZoomer().getCoordinateTimeConvertor();

				double frameNumber = convertor.getFrameNumber(convertor.getAdjustedMovieTime(point.x));
				double movieTime = convertor.getCorrespondingMovieTime(frameNumber);

//				double utteranceStartFrameNumber = convertor
//						.getFrameNumber(convertor.getAdjustedMovieTime(parentEvent.getStartTimeCoordinate()));
//				double utteranceEndFrameNumber = convertor
//						.getFrameNumber(convertor.getAdjustedMovieTime(parentEvent.getEndTimeCoordinate()));

				if (MOVE_DIRECTION == MOVE_START_POINT) {

//					if (utteranceStartFrameNumber > frameNumber) {
//						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//						return;
//					}

                    var time = entity.getStartTimeInfo().range().time(Math.round(movieTime), TimeUnit.TIMESTAMP);
                    String resp = entity.canSetSelectedEventStartMovieTime(time);

					if (resp != null) {
						SS3Singleton.getErrorMessages().showErrorMessage(resp);
						return;
					}
					
					entity.setStartTimeInfo(time);
				}

				if (MOVE_DIRECTION == MOVE_END_POINT) {

//					if (utteranceEndFrameNumber < frameNumber) {
//						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//						return;
//					}

                    var time = entity.getEndTimeInfo().range().time(Math.round(movieTime), TimeUnit.TIMESTAMP);
                    String resp = entity.canSetSelectedEventEndMovieTime(time);		// GREG TODO fix hack

					if (resp != null) {
						SS3Singleton.getErrorMessages().showErrorMessage(resp);
						return;
					}
					
					entity.setEndTimeInfo(time);
				}
			}
		}
	}

	@Override
	public void mouseReleased(final Point point, final JPanel panel) {
		// clear the lastPoint since the user has finished dragging the mouse
		lastPoint = null;

		NewChainedEvent newEvent = nmField.getNewChainedEvent();
		if (newEvent != null) {

			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

			Time startTimeInfo = newEvent.getStartTimeInfo();
			Time endTimeInfo = newEvent.getEndTimeInfo();

			if (endTimeInfo.lessThan(startTimeInfo)) {
				Time temp = endTimeInfo;
				endTimeInfo = startTimeInfo;
				startTimeInfo = temp;
			}

			boolean invalidEventTiming = false;

			if (startTimeInfo.equals(endTimeInfo)) {
				invalidEventTiming = true;
			}

//			PresentationField parentField = nmField.getParentField();
//			Event parentEvent = parentField.getEventAt(point.x);
//			if (parentEvent != null) {
//				Time parentEventStartTimeInfo = parentEvent.getStartTimeInfo();
//				Time precedingEventStartTimeInfo = segmentPanel.getMultipleMovieController()
//						.getPrecedingFrameTimeInfo(parentEventStartTimeInfo);
//				Time parentEventEndTimeInfo = parentEvent.getEndTimeInfo();
//				Time followingEventEndTimeInfo = segmentPanel.getMultipleMovieController()
//						.getFollowingFrameTimeInfo(parentEventEndTimeInfo);
//				if (startTimeInfo.lessThan(precedingEventStartTimeInfo)) {
//					invalidEventTiming = true;
//				}
//				if (endTimeInfo.greaterThan(followingEventEndTimeInfo)) {
//					invalidEventTiming = true;
//				}
//			} else {
//				invalidEventTiming = true;
//			}

			if (invalidEventTiming) {
				nmField.setNewChainedEvent(null);
				return;
			}

			ChainedEventsCollection collection = new ChainedEventsCollection();
			final ChainedEvent event = new ChainedEvent(segmentPanel);
			event.setStartTimeInfo(startTimeInfo);
			event.setEndTimeInfo(endTimeInfo);

			final String fieldID = field.getFieldID();
			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
			SS3Field field = dbScheme.getField(fieldID);
			final ArrayList<SS3FieldValue> values = field.getValuesAsList();
			final JPopupMenu popup = new JPopupMenu();

			for (final SS3FieldValue value : values) {
				final JLabel label = new JLabel(value.getName());
				popup.add(label);
				label.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent arg0) {
						if (event != null) {
							String id = value.getId();
							event.setID(id);
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
							panel.repaint();
						}
						popup.setVisible(false);
					}

					@Override
					public void mouseEntered(MouseEvent arg0) {
						label.setForeground(Color.RED);
					}

					@Override
					public void mouseExited(MouseEvent arg0) {
						label.setForeground(Color.BLACK);
					}
				});
			}

			final JLabel addNew = new JLabel("Add new value");
			addNew.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseClicked(MouseEvent arg0) {
					if (event != null) {
						popup.setVisible(false);

						final JDialog popup1 = new JDialog();
						popup1.setUndecorated(true);
						popup1.setResizable(false);
						NewCSValuePanel valuePanel = new NewCSValuePanel(fieldID, event,
								nmField.getSignStreamSegmentPanel(), popup1);

						popup1.setSize(valuePanel.getPreferredSize());
						popup1.add(valuePanel);
						Dimension d = popup1.getPreferredSize();
						int width = (int) d.getWidth();
						int x = (int) point.getX() - width / 2;
						int y = (int) point.getY();

						Point loc = new Point(x, y);
						SwingUtilities.convertPointToScreen(loc, panel);
						popup1.setLocation(loc);
						popup1.setVisible(true);
						
						// SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup1);

						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						panel.repaint();
					}
				}

				@Override
				public void mouseEntered(MouseEvent arg0) {
					addNew.setForeground(Color.RED);
				}

				@Override
				public void mouseExited(MouseEvent arg0) {
					addNew.setForeground(Color.BLACK);
				}
			});

			popup.add(addNew);
			popup.setPreferredSize(popup.getPreferredSize());
			popup.show(panel, (int) point.getX(), (int) point.getY());
			popup.setVisible(true);

			SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

			collection.setValueEvent(event);
			event.select();

			ChainedEventsEntity entity = new ChainedEventsEntity(collection, segmentPanel, nmField);
//			entity.determineParentEntity();

			nmField.addChainedEventEntity(entity);
			nmField.setNewChainedEvent(null);

			popup.addPopupMenuListener(new PopupMenuListener() {
				@Override
				public void popupMenuWillBecomeVisible(PopupMenuEvent e) {}

				@Override
				public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
					nmField.deleteEmptyEntities();
				}

				@Override
				public void popupMenuCanceled(PopupMenuEvent e) {
					nmField.deleteEmptyEntities();
				}
			});

			SS3Singleton.getMediaToolBar(null).selectEvent(event, entity, nmField);
		}
	}

	@Override
	public void mouseHoldReleased(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
			final String fieldID = field.getFieldID();

			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

			SS3Field field = dbScheme.getField(fieldID);
			final ArrayList<SS3FieldValue> values = field.getValuesAsList();
			final JPopupMenu popup = new JPopupMenu();
			final ChainedEvent cEvent = (ChainedEvent) event;

			ChainedEventsEntity entity = (ChainedEventsEntity) SS3Singleton.getMediaToolBar(null)
					.selectedEntity();

			if (cEvent.equals(entity.getValue())) {
				for (int i = 0; i < values.size(); i++) {
					final SS3FieldValue value = values.get(i);

					final JLabel label = new JLabel(value.getName());
					label.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseClicked(MouseEvent arg0) {
							if (cEvent != null) {
								String id = value.getId();
								cEvent.setID(id);
								SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
								panel.repaint();
							}
							popup.setVisible(false);
						}

						@Override
						public void mouseEntered(MouseEvent arg0) {
							label.setForeground(Color.RED);
						}

						@Override
						public void mouseExited(MouseEvent arg0) {
							label.setForeground(Color.BLACK);
						}
					});

					popup.add(label);
				}

				final JLabel addNew = new JLabel("Add new value");
				addNew.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent arg0) {
						if (cEvent != null) {
							popup.setVisible(false);

							final JDialog popup1 = new JDialog();
							popup1.setUndecorated(true);
							popup1.setResizable(false);
							NewCSValuePanel valuePanel = new NewCSValuePanel(fieldID, cEvent,
									nmField.getSignStreamSegmentPanel(), popup1);

							popup1.setSize(valuePanel.getPreferredSize());
							popup1.add(valuePanel);
							Dimension d = popup1.getPreferredSize();
							int width = (int) d.getWidth();
							int x = (int) point.getX() - width / 2;
							int y = (int) point.getY();

							Point loc = new Point(x, y);
							SwingUtilities.convertPointToScreen(loc, panel);
							popup1.setLocation(loc);
							popup1.setVisible(true);

							// SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup1);

							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
							panel.repaint();
						}
					}

					@Override
					public void mouseEntered(MouseEvent arg0) {
						addNew.setForeground(Color.RED);
					}

					@Override
					public void mouseExited(MouseEvent arg0) {
						addNew.setForeground(Color.BLACK);
					}
				});

				popup.add(addNew);
				popup.setPreferredSize(popup.getPreferredSize());
				popup.show(panel, (int) point.getX(), (int) point.getY());
				popup.setVisible(true);

				SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
			}
		}
	}
}
