package edu.bu.signstream.grepresentation.fields.freeTextField;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.time.TimeUnit;

public class FreeTextFieldWrapper extends NonManualFieldWrapper { // FieldWrapper {

	public FreeTextFieldWrapper(FreeTextField ftField,
			SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(ftField, segmentPanel, timeRange);
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	// do nothing!!!
	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
        var time = TimeRange.ABSOLUTE.time(Math.round(point.getX()), TimeUnit.COORDINATE);

        field.getAllEntities().stream()
            .filter(e -> e.interval().contains(time, Bound.INCLUSIVE))
            .findFirst()
            .ifPresent(e -> {
                var popup = new JPopupMenu();

                var delete = new JMenuItem("Delete");
                UserInterfaceUtil.setMenuItemLNF(delete);
                delete.addActionListener(l -> {
                    field.deleteEntity(e);
                    SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
                    panel.repaint();
                });
                popup.add(delete);

                JMenuItem cancel = new JMenuItem("Cancel");
                UserInterfaceUtil.setMenuItemLNF(cancel);
                popup.add(cancel);

                popup.setPreferredSize(popup.getPreferredSize());
                popup.show(panel, point.x, point.y);
                popup.setVisible(true);
                SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
            });
    }

    @Override
    public void mouseDblLeftClicked(Point point, JPanel panel) {
        mouseDblClicked(point, panel);
	}

	@Override
	public void mouseClicked(Point point, JPanel panel) {
	}

	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
		final JPopupMenu popup = new JPopupMenu();
		
		int xcoord = point.x;
		final Time movieTimeClicked = this.getSignStreamSegmentPanel()
				.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTimeInfo(xcoord);

		String str = "";
		FreeTextEntity ent = null;
		ArrayList<FreeTextEntity> entities = field.getEntities();
		for (int i = 0; i < entities.size(); i++) {
			FreeTextEntity currentEnt = entities.get(i);
			Time entStartTimeInfo = currentEnt.getStartTimeInfo();
			Time entEndTimeInfo = currentEnt.getEndTimeInfo();
			
			if (entStartTimeInfo.lessThanOrEq(movieTimeClicked) && movieTimeClicked.lessThanOrEq(entEndTimeInfo)) {
				ent =currentEnt;
				str = ent.getTextValue().getText();
				break;
			}
		}
		
		Event utterance = null;
		
		if (ent == null || ent.getParentEntities() == null) {
			PresentationField parentField = field.getParentField();
			utterance = parentField.getEventAt(xcoord);
		} else {
			if (ent.getParentEntities().size() == 1) {
				utterance = ent.getParentEntities().get(0);
			}
		}
		
		if (utterance == null)
			return;

		final Event parentEvent = utterance;

		final JTextField labelTextField = new JTextField();
		labelTextField.setText(str);
		labelTextField.setPreferredSize(new Dimension(700, 17));
		labelTextField.setBounds(new Rectangle(0, 0, 0, 0));
		labelTextField.setBackground(UIPalette.getBackground());
		labelTextField.setForeground(Color.black);
		labelTextField.setSize(5, 10);
		popup.setBounds(0, 0, 0, 0);
		popup.setBorder(BorderFactory.createEmptyBorder());
		popup.setBackground(UIPalette.getBackground());
		popup.add(labelTextField);

		PopupMenuListener popupMenuListener = new NewTextPopupMenuListener(
				labelTextField, segmentPanel);
		popup.addPopupMenuListener(popupMenuListener);

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				labelTextField.requestFocus();
			}
		});

		labelTextField.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == 10) {
					String txtStr = labelTextField.getText();
					FreeTextEntity ent = null;

					ArrayList<FreeTextEntity> entities = field.getEntities();
					for (int i = 0; i < entities.size(); i++) {
						FreeTextEntity currentEnt = entities.get(i);
						Time entStartTimeInfo = currentEnt.getStartTimeInfo();
						Time entEndTimeInfo = currentEnt.getEndTimeInfo();
						
						if (entStartTimeInfo.lessThanOrEq(movieTimeClicked) && movieTimeClicked.lessThanOrEq(entEndTimeInfo)) {
							ent =currentEnt;
							break;
						}
					}
					
					if (ent != null) {
						// if blank text on enter, it means we want to delete the free text entity from the wrapper and the database
						if (txtStr == null || txtStr.isEmpty()) {
							field.deleteEntity(ent);
						}
					} else if (ent == null) {
						ChainedEventsCollection collection = new ChainedEventsCollection();
						SS3Item item = new SS3Item();
						item.setText(txtStr);
						
						ToolBarPanels panels = segmentPanel.getToolBarPanels();
						MediaToolBar mediaTb = panels.getMediaToolBar();
						Event pEvent = parentEvent;
						
						item.setStartTimeInfo(pEvent.getStartTimeInfo());
						item.setEndTimeInfo(pEvent.getEndTimeInfo());
								
						GlossChainedEvent textValue = new GlossChainedEvent(item, segmentPanel);
						collection.setTextValueEvent(textValue);
						ent = new FreeTextEntity(collection, field.getFieldID(),
								segmentPanel, field);
						ent.updateParentEntity();
						ent.setValidTime(true);
						((FreeTextField) field).addFreeTextEntity(ent);
					}

					ent.getTextValue().setText(txtStr);

					segmentPanel.repaint();

					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(true);		
					
					popup.setVisible(false);			
				}
			}

			@Override
			public void keyTyped(KeyEvent e) {
			}

			@Override
			public void keyPressed(KeyEvent e) {
			}
		});

		popup.setPreferredSize(popup.getPreferredSize());
		int y = field.getY() - 11;

		popup.show(panel, xcoord, y);
		popup.setVisible(true);
		
		labelTextField.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent e) {
			}
			@Override
			public void focusLost(FocusEvent e) {
				popup.setVisible(false);
			}		
		});
	}

	public void mouseDblClicked(Point point, JPanel panel) {
		mousePressedAlt(point, panel);
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		if (!isEntitySelected()) {
			Event event = nmField.getEventAt(point.x);
			if (event != null) {
				GlossChainedEvent glsEvent = (GlossChainedEvent) event;
				panel.setToolTipText(glsEvent.getText());
			} else {
				panel.setToolTipText(null);
			}
			panel.setCursor(new Cursor(getCursorType(point)));
		}
	}

	@Override
	public int getCursorType(Point point) {
		return Cursor.DEFAULT_CURSOR;
	}
}

class NewTextPopupMenuListener implements PopupMenuListener {
	private SignStreamSegmentPanel segmentPanel = null;
	private JTextField labelTextField = null;

	NewTextPopupMenuListener(JTextField labelTextField, //Field nmField,
			SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		this.labelTextField = labelTextField;
	}

	@Override
	public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
		segmentPanel.repaint();
	}

	@Override
	public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
	}

	private void removeListeners(PopupMenuEvent popupMenuEvent) {
		java.awt.event.KeyListener[] listeners = labelTextField
				.getKeyListeners();
		for (int i = 0; i < listeners.length; i++) {
			java.awt.event.KeyListener listener = listeners[i];
			labelTextField.removeKeyListener(listener);
		}

		labelTextField = null;

		JPopupMenu menu = (JPopupMenu) popupMenuEvent.getSource();
		menu.removePopupMenuListener(this);

		System.gc();
	}

	@Override
	public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
	}
}