package edu.bu.signstream.grepresentation.fields.gestureField;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.Event;

public class GestureChainedEventsCollection {
   private GestureChainedEvent preparation = null;
   private GestureChainedEvent prestrokeHold = null;
   private GestureChainedEvent value = null;
   private GestureEvent textValue = null;
   private GestureChainedEvent poststrokeHold = null;
   private GestureChainedEvent retraction = null;
   private GestureChainedEvent relaxed = null;

   public GestureChainedEvent getPreparationEvent() {
      return preparation;
   }

   public GestureChainedEventsCollection cloneGestureChainedEventsCollection() {
      GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
      if (preparation != null)
         collection.setPreparationEvent(preparation.cloneGestureChainedEvent());
      if (prestrokeHold != null)
         collection.setPrestrokeHoldEvent(prestrokeHold.cloneGestureChainedEvent());
      if (value != null)
         collection.setValueEvent(value.cloneGestureChainedEvent());
      if (textValue != null)
         collection.setTextValueEvent(textValue.cloneGestureChainedEvent());
      if (poststrokeHold != null)
         collection.setPoststrokeHoldEvent(poststrokeHold.cloneGestureChainedEvent());
      if (retraction != null)
         collection.setRetractionEvent(retraction.cloneGestureChainedEvent());
      return collection;
   }
   
   public Event getNextEvent(Event event) {
      ArrayList<GestureChainedEvent> collection = getCollection();
      for(int i = 0; i < collection.size()-1; i++) {
         Event cEvent = collection.get(i);
         if(event.equals(cEvent)) {
            return collection.get(i+1);
         }
      }
      return null;
   }
   
   public Event getPreviousEvent(Event event) {
      ArrayList<GestureChainedEvent> collection = getCollection();
      for(int i = 1; i < collection.size(); i++) {
         Event cEvent = collection.get(i);
         if(event.equals(cEvent)) {
            return collection.get(i-1);
         }
      }
      return null;
   }

//   public ArrayList<NoteItem> getNoteItems(String entityID) {
//      ArrayList<NoteItem> noteItems = new ArrayList<NoteItem>();
//      ArrayList<GestureChainedEvent> col = getCollection();
//
//      for (GestureChainedEvent event : col) {
//         if (event != null) {
//            String noteID = event.getNoteID();
//            if (noteID != null && noteID.length() > 0) {
//               String valueID = event.getID();
//               String text = null;
//               if (event instanceof GlossGestureChainedEvent)
//                  text = event.getText();
//               NoteItem item = new NoteItem();
//               item.setNoteID(noteID);
//               item.setEntityID(entityID);
//               item.setValueID(valueID);
//               item.setText(text);
//               noteItems.add(item);
//            }
//         }
//      }
//      return noteItems;
//   }

   public ArrayList<GestureChainedEvent> getCollection() {
      ArrayList<GestureChainedEvent> list = new ArrayList<GestureChainedEvent>();
      addEvent(list, preparation);
      addEvent(list, prestrokeHold);
      addEvent(list, value);
      addEvent(list, textValue);
      addEvent(list, poststrokeHold);
      addEvent(list, retraction);
      addEvent(list, relaxed);
      return list;
   }
   
   public GestureChainedEvent getEventAt(double x) {
      ArrayList<GestureChainedEvent> list = getCollection();
      for(GestureChainedEvent event : list) {
    	  double startTimeCoord = event.getStartTimeCoordinate();
    	  double endTimeCoord = event.getEndTimeCoordinate();
    	  
    	  if(event.getStartTimeInfo().doubleValue() == event.getEndTimeInfo().doubleValue()) {
    		  startTimeCoord = startTimeCoord - 5;
    		  endTimeCoord = endTimeCoord + 5; 
    	  }
    	  
    	  if(startTimeCoord <= x && x <= endTimeCoord) {
    		  return event;
    	  }
      }
      return null;
   }

   private void addEvent(ArrayList<GestureChainedEvent> list, GestureChainedEvent event) {
      if (event != null)
         list.add(event);
   }

   public void setPreparationEvent(GestureChainedEvent onset) {
      this.preparation = onset;
      if (onset != null)
         onset.setText(Constants.PREPARATION);
   }

   public GestureChainedEvent getPrestrokeHoldEvent() {
      return prestrokeHold;
   }

   public void setPrestrokeHoldEvent(GestureChainedEvent initialHold) {
      this.prestrokeHold = initialHold;
      if (initialHold != null)
         initialHold.setText(Constants.PRESTROKE_HOLD);
   }

   public GestureChainedEvent getValueEvent() {
      return value;
   }

   public GestureChainedEvent getValueOrTextValueEvent() {
      return value == null ? textValue : value;
   }

   public void setValueEvent(GestureChainedEvent value) {
      this.value = value;
   }

   public GestureEvent getTextValueEvent() {
      return textValue;
   }

   public void setTextValueEvent(GestureEvent textValue) {
      this.textValue = textValue;
      if (textValue != null)
         textValue.setText(textValue.getText());
   }

   public GestureChainedEvent getPoststrokeHoldEvent() {
      return poststrokeHold;
   }

   public void setPoststrokeHoldEvent(GestureChainedEvent finalHold) {
      this.poststrokeHold = finalHold;
      if (finalHold != null) {
         finalHold.setText(Constants.POSTSTROKE_HOLD);
      }
   }

   public GestureChainedEvent getRetractionEvent() {
      return retraction;
   }

   public void setRetractionEvent(GestureChainedEvent offset) {
      this.retraction = offset;
      if (offset != null)
         offset.setText(Constants.RETRACTION);
   }
   
   public GestureChainedEvent getRelaxedEvent() {
	   return relaxed;
   }
   
   public void setRelaxedEvent(GestureChainedEvent relaxed) {
	   this.relaxed = relaxed;
	   if (relaxed != null) {
		  relaxed.setText(Constants.RELAXED);
	   }
   }

   @Override
public String toString() {
      StringBuffer buffer = new StringBuffer("GestureChainedEventsCollection: ");
      buffer.append("\n PREPARATION: ");
      buffer.append(preparation);
      buffer.append("\n PRESTROKE HOLD: ");
      buffer.append(prestrokeHold);
      buffer.append("\n VALUE: ");
      buffer.append(value);
      buffer.append("\n TEXT VALUE: ");
      buffer.append(textValue);
      buffer.append("\n POSTSTROKE HOLD: ");
      buffer.append(poststrokeHold);
      buffer.append("\n RETRACTION: ");
      buffer.append(retraction);
      buffer.append("\n RELAXED: ");
      buffer.append(relaxed);

      return buffer.toString();
   }
}
