package edu.bu.signstream.grepresentation.fields.gestureField;

import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GestureStartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;

public class GestureClassifications {

	public static final int SIGN_FINGERSPELLED = 0;
	public static final int SIGN_LOAN = 1;
	public static final int SIGN_LEXICAL = 2;
	public static final int SIGN_CLASSIFIER = 3;
	public static final int SIGN_GESTURE = 4;
	public static final int SIGN_NUMBERS = 5;

	public static final int SAME_START_END_HANDSHAPES = 0;
	public static final int DIFF_START_END_HANDSHAPES = 1;
	public static final int NOT_ENTERED_HANDSHAPES = 2;

	private int signClassifier = GestureClassifications.SIGN_LEXICAL;
	private boolean isNameSign = false;
	private boolean isOneFrameGesture = false;
	private boolean isPassiveBaseArm = false;
	private int numberOfHands = 1;
	private boolean sameHandshapes = true;
	private GestureStartEndHandshapesType domHandShapesSameDifferent = GestureStartEndHandshapesType.SAME;
	private GestureStartEndHandshapesType nondomHandShapesSameDifferent = GestureStartEndHandshapesType.SAME;
	private boolean markedNumberOfHands;
	private boolean isRightHandDominant = true;
	
	private boolean defaultClassificationValues = true;

	public boolean isDefaultClassificationValues() {
		return defaultClassificationValues;
	}
	
	public int getSignClassification() {
		return signClassifier;
	}

	public void setSignClassifications(int signClassifier) {
		this.signClassifier = signClassifier;
		this.defaultClassificationValues = false;
	}
	
	public boolean isSignClassificationFingerspelled() {
		return signClassifier == GestureClassifications.SIGN_FINGERSPELLED;
	}
	
	public boolean isSignClassificationLoan() {
		return signClassifier == GestureClassifications.SIGN_LOAN;
	}
	
	public boolean isSignClassificationLexical() {
		return signClassifier == GestureClassifications.SIGN_LEXICAL;
	}
	
	public boolean isSignClassificationClassifier() {
		return signClassifier == GestureClassifications.SIGN_CLASSIFIER;
	}
	
	public boolean isSignClassificationGesture() {
		return signClassifier == GestureClassifications.SIGN_GESTURE;
	}
	
	public boolean isSignClassificationNumber() {
		return signClassifier == GestureClassifications.SIGN_NUMBERS;
	}

	public boolean isNameSign() {
		return isNameSign;
	}

	public void setNameSign(boolean isNameSign) {
		this.isNameSign = isNameSign;
		this.defaultClassificationValues = false;
	}
	
	public boolean isOneFrameGesture() {
		return isOneFrameGesture;
	}
	
	public void setOneFrameGesture(boolean isOneFrameGesture) {
		this.isOneFrameGesture = isOneFrameGesture;
	}

	public boolean isPassiveBaseArm() {
		return isPassiveBaseArm;
	}

	public void setPassiveBaseArm(boolean isPassiveBaseArm) {
		this.isPassiveBaseArm = isPassiveBaseArm;
		this.defaultClassificationValues = false;
	}

	public int getNumberOfHands() {
		return numberOfHands;
	}
	
	public boolean isOneHanded() {
		return numberOfHands == 1;
	}

	public void setNumberOfHands(int numberOfHands) {
		this.numberOfHands = numberOfHands;
		this.defaultClassificationValues = false;
	}

	public boolean isSameHandshapes() {
		return sameHandshapes;
	}

	public void setSameHandshapes(boolean sameHandshapes) {
		this.sameHandshapes = sameHandshapes;
		this.defaultClassificationValues = false;
	}

	public GestureStartEndHandshapesType getDomHandShapesSameDifferent() {
		return domHandShapesSameDifferent;
	}

	public void setDomHandShapesSameDifferent(GestureStartEndHandshapesType domHandShapesSameDifferent) {
		this.domHandShapesSameDifferent = domHandShapesSameDifferent;
		this.defaultClassificationValues = false;
	}

	public GestureStartEndHandshapesType getNondomHandShapesSameDifferent() {
		return nondomHandShapesSameDifferent;
	}

	public void setNondomHandShapesSameDifferent(
			GestureStartEndHandshapesType nondomHandShapesSameDifferent) {
		this.nondomHandShapesSameDifferent = nondomHandShapesSameDifferent;
		this.defaultClassificationValues = false;
	}

	public boolean isMarkedNumberOfHands() {
		return markedNumberOfHands;
	}

	public void setMarkedNumberOfHands(boolean markedNumberOfHands) {
		this.markedNumberOfHands = markedNumberOfHands;
		this.defaultClassificationValues = false;
	}
}
