package edu.bu.signstream.grepresentation.fields.gestureField.dialog;


import edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations.GestureLocationInfo;

import javax.swing.*;

public class LocationEncloserPanel extends JPanel {

    private final LocationPanel domLocation;
    private final LocationPanel ndomLocation;

    public LocationEncloserPanel(GestureInfoPanel gip) {
        domLocation = new LocationPanel(gip);
        ndomLocation = new LocationPanel(gip);
        initUI();
    }

    private void initUI() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        domLocation.createDepHandAssociations(ndomLocation.getLocDispPanels(), "NonDom");
        add(domLocation);

        ndomLocation.createDepHandAssociations(domLocation.getLocDispPanels(), "Dom");
        add(ndomLocation);

        setPanelRegionText(domLocation, "Dom Hand");
        setPanelRegionText(ndomLocation, "NonDom Hand");
    }

    public void resetHands() {
        domLocation.resetHand();
        ndomLocation.resetHand();
    }

    public void resetDialogs() {
        domLocation.resetDialogs();
        ndomLocation.resetDialogs();
    }

    public GestureLocationInfo getDomLocationInfo() {
        return domLocation.getCurrentLocationInfo();
    }

    public GestureLocationInfo getNDomLocationInfo() {
        return ndomLocation.getCurrentLocationInfo();
    }

    public void setDomLocationInfo(GestureLocationInfo currentLocationInfo) {
        domLocation.setCurrentLocationInfo(currentLocationInfo);
    }

    public void setNDomLocationInfo(GestureLocationInfo currentLocationInfo) {
        ndomLocation.setCurrentLocationInfo(currentLocationInfo);
    }

    private void setPanelRegionText(LocationPanel location, String text) {
        location.setVisible(true);
        location.setBorder(BorderFactory.createTitledBorder(text));
    }

    public void clearDOMRegion(){
        domLocation.clearRegions();
    }

    public void clearNDOMRegion(){
        ndomLocation.clearRegions();
    }

    public void enableOnlyDOM() {
        domLocation.enableButtons();
        ndomLocation.disableButtons();
        domLocation.enableOrDisableOtherLocationCopyButton(false);

    }
    public void enableOnlyNDOM() {
        ndomLocation.enableButtons();
        domLocation.disableButtons();
        ndomLocation.enableOrDisableOtherLocationCopyButton(false);
    }

    public void enableDOMandNDOM() {
        domLocation.enableButtons();
        domLocation.enableOrDisableOtherLocationCopyButton(true);
        ndomLocation.enableButtons();
        ndomLocation.enableOrDisableOtherLocationCopyButton(true);
    }
}
