package edu.bu.signstream.grepresentation.fields.gestureField.orientationField;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.FingerTipImage;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.PalmImage;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.OrientationImage;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.SelectedGestureHandshapesPanel;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureOrientationFieldWrapper extends FieldWrapper implements OrientationPalletClient, GeneralOrientationPanelClient {

    private final GestureOrientationField field;

    private final JDialog d = null;

    private int mousePressedAltAt;

    public GestureOrientationFieldWrapper(
            GestureOrientationField field,
            SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
        super(field, segmentPanel, timeRange);
        this.field = field;
    }

    public GestureOrientationField getGestureOrientationField() {
        return this.field;
    }

    @Override
    public void removeAll() {
    }


    public EventsEntity getEntityWithRefID(String refEntityId) {
        return this.field.getEntityWithRefID(refEntityId);
    }


    @Override
    public void highlight(boolean highlight, String refID) {
        this.field.highlight(highlight, refID);
        this.highlightDependentWrappers(highlight, refID);
    }

    private void highlightDependentWrappers(boolean highlight, String refID) {
        for (FieldWrapper wrapper : this.dependentWrappers) {
            wrapper.highlight(highlight, refID);
        }
    }

    @Override
    public FieldWrapper getDependentFieldWrapper(String fieldID) {
        for (FieldWrapper wrp : this.dependentWrappers) {
            String fID = wrp.getField().getFieldID();
            if (fieldID.equalsIgnoreCase(fID))
                return wrp;
        }
        return null;
    }

    @Override
    public void mouseClickedAlt(Point point) {
        this.mousePressedAltAt = point.x;
    }

    @Override
    public Event getSelectedEvent() {
        return this.field.getSelectedEvent();
    }

    @Override
    public void unselectAll() {
        this.field.unselect();
        for (FieldWrapper wrapper : this.dependentWrappers) {
            wrapper.unselectAll();
        }
    }

    @Override
    public boolean contains(int Y) {
        return this.field.contains(Y);
    }

    // MouseListener
    @Override
    public void mouseClicked(Point point, JPanel panel) {
    }

    @Override
    public void mouseRightClicked(Point point) {
    }

    @Override
    public void mouseLeftClicked(Point point) {
    }

    @Override
    public void mouseDblLeftClicked(Point point, JPanel panel) {
        // 2016-11-07 removing all the below to disallow re-selection of the hs images from the main window - must be done via Morph-Phon window.
        // With only one mechanism to change it, we can better control only "sanctioned" entries
    }

    @Override
    public void setItem(Object obj, boolean bothHands) {
        this.addItem(obj, this.field);

        if (bothHands) {
            String type = this.getType();
            FieldWrapper wrp;

            if (type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SHAPE)) {
                wrp =
                        this.segmentPanel.getSegment(this.getField().getSegmentID()).getFieldWrapper(
                                Constants.TYPE_DOM_HAND_SHAPE);
            } else {
                wrp =
                        this.segmentPanel.getSegment(this.getField().getSegmentID()).getFieldWrapper(
                                Constants.TYPE_NDOM_HAND_SHAPE);
            }

            GestureOrientationField hsFld = (GestureOrientationField) wrp.getField();
            this.addItem(obj, hsFld);
        }

        this.d.dispose();
        SS3Singleton.getSignStreamApplication().repaint();
    }

    @Override
    public void setItem(Object obj) {
        this.addItem(obj, this.field);
        this.d.dispose();
        SS3Singleton.getSignStreamApplication().repaint();
    }

    private void addItem(Object obj, GestureOrientationField hsField) {
        SS3FieldValue value = (SS3FieldValue) obj;
        String valueID = value.getId();

        SS3CodingScheme dbScheme = this.segmentPanel.getSS3Database().getSS3CodingScheme();
        GestureOrientationsImageManager hsManager =
                SS3Singleton.getGestureOrientationsImageManager();
        System.out.println(value.getIid());
        SS3FieldTypeImg ftImg = dbScheme.getImage(value.getIid());

        Image lImage = hsManager.getImage(ftImg.getPath());
        Image sImage = hsManager.getImage(ftImg.getPath());

        GestureOrientationEvent event;
        Event ev = hsField.getSelectedEvent();

        SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

        if (ev != null) {
            event = (GestureOrientationEvent) ev;
        } else {
            SS3Item item = new SS3Item();
            event = new GestureOrientationEvent(item, this.segmentPanel);
            event.setStartTimeCoordinate(this.mousePressedAltAt);

            EventsEntity entity = hsField.getEntityAt(this.mousePressedAltAt);

            GestureOrientationEntity hsEntity = (GestureOrientationEntity) entity;
            hsEntity.addGestureOrientationEvent(event);
        }

        event.setID(valueID);
        event.setLargeImage(lImage);
        event.setSmallImage(sImage);
        event.setSS3FieldTypeImg(ftImg);
    }

    @Override
    public void mouseDblRightClicked(Point point, JPanel panel) {
    }

    // MouseMotionListener
    @Override
    public void mouseMoved(Point point, JPanel panel) {
//      if (!isEntitySelected()) {
//         EventsEntity entity = field.getEntityAt(point.x);
//
//         if (entity != null) {
//            OrientationEntity_XYZ hsEntity = (OrientationEntity_XYZ) entity;
//            OrientationEvent_XYZ hsEvent = (OrientationEvent_XYZ) field.getEventAt(point.x);
//
//            if (hsEvent == null) {
//               EventsIterator iterator = hsEntity.getEventsIterator();
//
//               OrientationEvent_XYZ pHsEvent =
//                  (OrientationEvent_XYZ) iterator.previous(point.x);
//               OrientationEvent_XYZ nHsEvent =
//                  (OrientationEvent_XYZ) iterator.next(point.x);
//
//               if (pHsEvent == null) {
//                  hsEvent = nHsEvent;
//               } else if (nHsEvent == null) {
//                  hsEvent = pHsEvent;
//               } else {
//                  double x1 = point.x - pHsEvent.getStartTimeCoordinate();
//                  double x2 = nHsEvent.getStartTimeCoordinate() - point.x;
//                  hsEvent = x1 <= x2 ? pHsEvent : nHsEvent;
//               }
//
//               hsEvent.setMagnifiedImage(true);
//               panel.repaint();
//               panel.setToolTipText(hsEvent.getToolTipText());
//            }
//
//            hsEvent.setMagnifiedImage(true);
//            panel.repaint();
//            panel.setToolTipText(hsEvent.getToolTipText());
//         } else {
//            panel.setToolTipText("");
//         }
//
//         panel.setCursor(getCursor(point));
//      }
    }

    @Override
    public void mousePressedAlt(Point point, JPanel panel) {
    }

    public void mouseReleasedAlt(Point point) {

    }

    @Override
    public void mouseReleased(Point point, JPanel panel) {
    }

    @Override
    public void addEventsEntity(EventsEntity entity) {
        this.field.addGestureOrientationEntity(entity);
    }

    @Override
    public void delete(int eventAt) {
        EventsEntity entity = this.field.getEntityAt(eventAt);

        if (entity != null)
            this.field.deleteEntity(entity);
    }

    private Cursor getCursor(Point point) {

        Event event = this.field.getEventAt(point.x);
        if (event != null && event.isSelected())
            return new Cursor(Cursor.E_RESIZE_CURSOR); // .HAND_CURSOR;
        return new Cursor(Cursor.DEFAULT_CURSOR);
    }

    @Override
    public void addOrientation(NewOrientation hs) {
        GestureOrientationEntity hsEntity = this.field.getEntityWithNewSubEntity();

        if (hsEntity != null) {
            NewGestureOrientationEntity newEntity = hsEntity.getNewGestureOrientationEntity();

            GestureOrientationSubEntity subEntity =
                    new GestureOrientationSubEntity(this.segmentPanel, this.field);
            subEntity.updateParentEntity();

            SelectedGestureHandshapesPanel panel = hs.getPanel();
            ArrayList<FingerTipImage> hShapes = panel.getFingerTipOrientationsImages();
            ArrayList<PalmImage> pShapes = panel.getPalmOrientationsImages();

            if (hShapes.size() == 1) {
                FingerTipImage hsImg = hShapes.get(0);
                FingerTipImage clonedImg = (FingerTipImage) hsImg.clone();
                hShapes.add(clonedImg);
            }

            int startTimeCoord = newEntity.getStartTimeCoord();
            int duration = newEntity.getEndTimeCoord() - startTimeCoord;

            int step = duration / (hShapes.size() - 1);

            for (int i = 0; i < hShapes.size(); i++) {
                OrientationImage image = hShapes.get(i);
                String id = image.getSS3FieldValue().getId();
                SS3Item item = new SS3Item();
                item.setId(id);

                GestureOrientationEvent hsEvent = new GestureOrientationEvent(item, this.segmentPanel);
                int coordinate = startTimeCoord + i * step;

                hsEvent.setStartTimeCoordinate(coordinate);
                hsEvent.setEndTimeCoordinate(coordinate);
                subEntity.addOrientation(hsEvent);
            }

            if (pShapes.size() == 1) {
                PalmImage psImg = pShapes.get(0);
                PalmImage clonedImg = (PalmImage) psImg.clone();
                pShapes.add(clonedImg);
            }

            step = duration / (pShapes.size() - 1);

            for (int i = 0; i < pShapes.size(); i++) {
                OrientationImage image = pShapes.get(i);
                String id = image.getSS3FieldValue().getId();
                SS3Item item = new SS3Item();
                item.setId(id);

                GestureOrientationEvent psEvent = new GestureOrientationEvent(item, this.segmentPanel);
                int coordinate = startTimeCoord + i * step;

                psEvent.setStartTimeCoordinate(coordinate);
                psEvent.setEndTimeCoordinate(coordinate);
                subEntity.addOrientation(psEvent);
            }

            hsEntity.setNewGestureOrientationEntity(null);
            hsEntity.addGestureOrientationSubEntity(subEntity);

            SS3Singleton.getSignStreamApplication().repaint();
        }
    }

    @Override
    public void cancelNewOrientation() {
        GestureOrientationEntity hsEntity = this.field.getEntityWithNewSubEntity();
        hsEntity.setNewGestureOrientationEntity(null);
        SS3Singleton.getSignStreamApplication().repaint();
    }

    @Override
    public void mouseHoldReleased(Point point, JPanel panel) {
    }
}

class DeleteOrientationEntityListener implements ActionListener {
    private EventsEntity entity = null;
    private GestureOrientationField field = null;
    private JPanel panel = null;

    public DeleteOrientationEntityListener(EventsEntity entity, GestureOrientationField field, JPanel panel) {
        this.entity = entity;
        this.field = field;
        this.panel = panel;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        final JPopupMenu popup = new JPopupMenu();

        JLabel msgLabel = new JLabel("Are you sure you want to delete the object?");
        UserInterfaceUtil.setLabelLNF(msgLabel);

        final JButton okBtn = new JButton("Proceed");
        final JButton cancelBtn = new JButton("Cancel");

        JPanel msgPanel = new JPanel();
        GridBagConstraints gbc = new GridBagConstraints();

        msgPanel.setLayout(new GridBagLayout());
        gbc.anchor = GridBagConstraints.WEST;

        gbc.gridwidth = 3;
        gbc.gridx = 0;
        gbc.gridy = 0;
        msgPanel.add(msgLabel, gbc);

        gbc.gridy = 1;
        gbc.gridx = 1;
        msgPanel.add(new JLabel(" "), gbc);

        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy = 3;
        gbc.gridx = 0;
        msgPanel.add(new JLabel(" "), gbc);
        gbc.gridx = 1;
        msgPanel.add(okBtn, gbc);
        gbc.gridx = 2;
        msgPanel.add(cancelBtn, gbc);

        cancelBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                popup.setVisible(false);
            }
        });

        okBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                DeleteOrientationEntityListener.this.field.deleteEntity(DeleteOrientationEntityListener.this.entity);

                DeleteOrientationEntityListener.this.field.deleteEmptyEntities();

                SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

                DeleteOrientationEntityListener.this.panel.repaint();
                popup.setVisible(false);
            }
        });

        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                okBtn.requestFocus();
            }
        });

        int width = msgPanel.getPreferredSize().width + 30;
        int height = msgPanel.getPreferredSize().height + 20;

        popup.add(msgPanel);
        popup.setPopupSize(new Dimension(width, height));
        popup.setLocation((int) this.entity.getStartTimeCoordinate(), 200);
        popup.setVisible(true);

        SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
    }
}

