package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations;

import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.LocationPanel;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

/**
 * This class is responsible for showing a small thumbnail of a selected region
 * on the {@link LocationPanel} along with buttons to copy this panel's region info
 * to its siblings and a button that opens a dialog to select the region for this panel.
 */
public class LocationDisplayPanel extends JPanel {

    private RegionSelectDialog dialog;
    private RegionDisplayPanel regionDisplayPanel;
    private JButton selectRegionButton = new JButton("Select Region");
    private JButton clearButton = new JButton("Clear");

    private ArrayList<JButton> copyButtons = new ArrayList<JButton>();

    private ArrayList<LocationDisplayPanel> siblingPanels;
    private LocationType locationType;

    public LocationDisplayPanel(LocationType locationType, AssociatedFieldsPanel afp) {
        this.locationType = locationType;
        regionDisplayPanel = new RegionDisplayPanel(afp);
        setRegion(null);
    }

    public void init() {
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;

        setBorder(new TitledBorder(locationType.getName() + " Location"));
        selectRegionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (dialog == null) {
                    dialog = new RegionSelectDialog(LocationDisplayPanel.this, locationType.getName());
                } else if (regionDisplayPanel.getRegion().equals(Region.NONE)) {
                    dialog.reset();
                }
                dialog.setVisible(true);
            }
        });
        add(selectRegionButton, c);
        c.gridy++;
        add(regionDisplayPanel, c);
        for (final LocationDisplayPanel siblingPanel : siblingPanels) {
            JButton button = new JButton("Copy to " + siblingPanel.getLocationType().getName());
            button.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    siblingPanel.copyFrom(LocationDisplayPanel.this);
                }
            });
            copyButtons.add(button);

            c.gridy++;
            add(button, c);
        }

        clearButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                setRegion(null);
            }
        });

        c.gridy++;
        add(clearButton, c);
    }

    public void setSiblingPanels(ArrayList<LocationDisplayPanel> siblings) {
        this.siblingPanels = new ArrayList<LocationDisplayPanel>(siblings);
    }

    public LocationType getLocationType() {
        return locationType;
    }

    void copyFrom(LocationDisplayPanel panel) {
        setRegion(panel.regionDisplayPanel.getRegion());
    }

    public Region getRegion() {
        return regionDisplayPanel.getRegion();
    }

    void selectedRegion(Region result) {
        Region r = result.getParent();
        if (r != null) {
            setRegion(result);
        }
    }

    public void setRegion(Region sr) {
        boolean enable = sr != null && !sr.equals(Region.NONE);
        clearButton.setEnabled(enable);
        for (JButton copyButton : copyButtons) {
            copyButton.setEnabled(enable);
        }
        regionDisplayPanel.setRegion(sr);
    }

    public void resetDialog() {
        if (dialog != null) {
            dialog.reset();
        }
    }
}
