package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

public class Region {

    public static final Region NONE = new Region("-1", "None Selected", "None");

    private String id;
    private String name;
    private String abbreviation;
    private boolean selectable = true;

    private String imageID;
    private Region parent;
    private HashMap<Region, ArrayList<Polygon>> polygonMap = new HashMap<Region, ArrayList<Polygon>>();
    private HashMap<Region, Integer> subregionOrders = new HashMap<Region, Integer>();
    private ArrayList<Region> subregions = new ArrayList<Region>();

    public Region(String id, String name, String abbreviation) {
        this.id = id;
        this.name = name;
        this.abbreviation = abbreviation;
    }

    public String getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getAbbreviation() {
        return abbreviation;
    }

    public boolean isSelectable() {
        return selectable;
    }

    public void setSelectable(boolean selectable) {
        this.selectable = selectable;
    }

    public boolean isParent() {
        return !subregions.isEmpty();
    }

    public String getImageID() {
        return imageID;
    }

    public void setImageID(String imageID) {
        this.imageID = imageID;
    }

    public ArrayList<Polygon> getPolygons(Region subregion) {
        return polygonMap.get(subregion);
    }

    public Region getParent() {
        return parent;
    }

    public void setParent(Region parent) {
        this.parent = parent;
    }

    public ArrayList<Region> getSubregions() {
        return new ArrayList<Region>(subregions);
    }

    public void addSubregion(final Region subregion, final int order, ArrayList<Polygon> polygons) {
        subregions.add(subregion);
        subregionOrders.put(subregion, order);
        Collections.sort(subregions, new Comparator<Region>() {
            @Override
            public int compare(Region o1, Region o2) {
                return subregionOrders.get(o1) - subregionOrders.get(o2);
            }
        });
        polygonMap.put(subregion, polygons);
    }

    @Override
    public String toString() {
        return abbreviation + " (" + name + ")";
    }

    @Override
    public boolean equals(Object obj) {
        return (obj instanceof Region) && ((Region)obj).id.equals(id);
    }

    @Override
    public int hashCode() {
        return Integer.parseInt(id);
    }
}
