package edu.bu.signstream.grepresentation.fields.glossField.dialog.dai;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.Version;
import edu.bu.signstream.common.util.https.HttpRequest;
import edu.bu.signstream.common.util.https.responses.BasicResponse;
import edu.bu.signstream.common.util.https.responses.ChoiceResponse;
import edu.bu.signstream.common.util.https.responses.DAIResponse;
import edu.bu.signstream.common.util.https.requestBuilder.RequestBuilder;
import edu.bu.signstream.common.util.https.requestBuilder.RequestType;
import edu.bu.signstream.common.util.https.responses.InvalidJSONResponse;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;
import edu.bu.signstream.media.Video;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.conn.HttpHostConnectException;
import org.apache.http.entity.ContentType;

import javax.net.ssl.SSLException;
import javax.net.ssl.SSLHandshakeException;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.swing.*;
import javax.swing.Timer;
import javax.swing.event.HyperlinkEvent;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.function.Predicate;
import java.util.logging.Level;

/*
If you are modifying anything to do with DAISearch, these classes may be of interest to you:
- DAISearch.java
- BasicResponse.java
- ChoiceResponse.java extends BasicResponse.java
- DAIResponse.java extends BasicResponse.java
- VideoSplicer.java
- HttpRequest.java

Some of the comments in these files are out of date because while I was working on this, A LOT of requirements changed and I
had to make major refactors. I didn't have time to update the comments, some of the code is admittedly messy, but
hopefully it's a little more readable and obvious than other parts of the code.
 */

public class DAISearch {
    private final Time startTime; // The gloss start time
    private final Time endTime; // The gloss end time
    private final JComboBox<String> videoComboBox; // The combobox that contains the list of video files
    private final AssociatedFieldsPanel parentComponent; // The parent component, in this case, the Morph-Phon Info dialog (AKA, the Associated Fields dialog)
    private final JDialog associatedFieldsParentDialog; // This is the parent of the parentComponent.
    private final String DAI_PROCESSING_ENDPOINT; // This is a constant that contains the URL of the DAI processing endpoint... it can be overridden in info.txt
    private String DAI_CHOICE_ENDPOINT; // This is a constant that contains the URL of the DAI choice endpoint... it can be overridden in info.txt
    private String DAI_BROWSER_LINK; // This is a constant that contains the URL of the DAI browser link... it can be overridden in info.txt
    private final String DAI_ERROR_MESSAGE_URL; // This is a constant that contains the URL of the DAI error message link... it can be overridden in info.txt
    private final int SUCCESS; // This is a constant that contains the success code returned from the DAI choice endpoint... it can be overridden in info.txt
    private final int CHOICE_NOT_MADE; // This is a constant that contains the choice not made code returned from the DAI choice endpoint... it can be overridden in info.txt
    private final long DAI_PAUSE_TIME; // This is a constant that contains the pause time between the DAI search and the DAI choice endpoint... it can be overridden in info.txt
    private final long DAI_SEARCH_TIMEOUT; // This is a constant that contains the timeout for the DAI search... it can be overridden in info.txt
    private final long DAI_POLLING_RATE; // This is a constant that contains the polling rate for the DAI search... it can be overridden in info.txt
    private final long MINIMUM_DIALOG_OPEN_TIME; // This is a constant that contains the minimum time the DAI search dialog must be open... it can be overridden in info.txt

    private static String defaultVideoFile;

    private final String ERROR_MESSAGE_FOOTER; // This is a constant that contains the footer of the error message... it can be overridden in info.txt

    private final Segment currentSegment;
    private final HashMap<String, String> defaultDAIVideoNames;


    /**
     * @param startTime The start time of the DAI search
     * @param endTime  The end time of the DAI search
     * @param parentComponent The parent component, in this case, the Morph-Phon Info dialog (AKA, the Associated Fields dialog)
     * @param associatedFieldsParentDialog The parent of parentComponent
     */
    public DAISearch(Time startTime, Time endTime, AssociatedFieldsPanel parentComponent, JDialog associatedFieldsParentDialog, Segment currentSegment,
                     HashMap<String, String> defaultDAIVideoNames) {
        this.startTime = startTime;
        this.endTime = endTime;
        this.videoComboBox = new JComboBox<>();
        this.parentComponent = parentComponent;
        this.associatedFieldsParentDialog = associatedFieldsParentDialog;
        this.DAI_PROCESSING_ENDPOINT = getValueFromInfoTxt("dai_processing_endpoint", null);
        this.DAI_CHOICE_ENDPOINT = getValueFromInfoTxt("dai_choice_endpoint", null);
        this.DAI_BROWSER_LINK = getValueFromInfoTxt("dai_browser_link", null);
        this.DAI_ERROR_MESSAGE_URL = getValueFromInfoTxt("dai_error_message_url", null);
        this.ERROR_MESSAGE_FOOTER = String.format("DAI Search: <html><a href=%s>%s</a></html>.", this.DAI_ERROR_MESSAGE_URL, this.DAI_ERROR_MESSAGE_URL);
        this.SUCCESS = Integer.parseInt(getValueFromInfoTxt("dai_success", "0"));
        this.CHOICE_NOT_MADE = Integer.parseInt(getValueFromInfoTxt("dai_choice_not_made", "1"));
        this.DAI_PAUSE_TIME = Long.parseLong(getValueFromInfoTxt("dai_pause", "5000"));
        this.DAI_SEARCH_TIMEOUT = Long.parseLong(getValueFromInfoTxt("dai_polling_timeout", "60000"));
        this.DAI_POLLING_RATE = Long.parseLong(getValueFromInfoTxt("dai_polling_rate", "3000"));
        this.MINIMUM_DIALOG_OPEN_TIME = Long.parseLong(getValueFromInfoTxt("minimum_dialog_open_time", String.valueOf(Constants.MIN_DAI_LOADING_TIME)));
        this.currentSegment = currentSegment;
        this.defaultDAIVideoNames = defaultDAIVideoNames;
    }

    /**
     * @return Whether or not there is a valid DAI URL to POST to
     */
    public static boolean isDAIEnabled() {
        return getValueFromInfoTxt("dai_processing_endpoint", null) != null;
    }

    /**
     * This method will read the properties in the info.txt file and return the possible corresponding value
     * @param key The key of the value to get from the info.txt file
     * @param defaultValue The default value to return if the key is not found
     * @return The value of the key in the info.txt file
     */
    private static String getValueFromInfoTxt(String key, String defaultValue) {
        try (InputStream inputStream = new FileInputStream(SSResources.infoPath)) {
            Properties properties = new Properties();
            properties.load(inputStream);

            // If we are given a default value, then we respect it, else we just return null for a null default value if that value does not have a corresponding key
            return defaultValue != null ? properties.getProperty(key, defaultValue).strip() : properties.getProperty(key, null);
        } catch (IOException e) {
            SS3SignStreamApplication.logger.log(Level.SEVERE, "Error loading config.properties file");
            throw new RuntimeException("Error loading config.properties file");
        }
    }

    /**
     * @param errorMessage The error message to display to the end user
     * @param lineBreaks Whether to add line breaks to the error message, variable arguments, so you can pass in a boolean to determine if you want line breaks or not. By default, line breaks are added.
     * @return A JEditorPane with the footer message and a link below it. If the link is clicked, the user will be taken to the manual DAI search page.
     */
    private JEditorPane createEditorPane(String errorMessage, Boolean... lineBreaks) { // Variable arguments, so you can pass in a boolean to determine if you want line breaks or not
        JEditorPane editorPane = new JEditorPane();
        editorPane.setFont(UIManager.getFont("Label.font"));
        editorPane.setContentType("text/html");


        if (lineBreaks.length == 0 || lineBreaks[0]) {
            editorPane.setText("<html>" + errorMessage + (this.DAI_ERROR_MESSAGE_URL != null ? "<br><br>" + this.ERROR_MESSAGE_FOOTER : "") + "</html>");
        } else {
            editorPane.setText("<html>" + errorMessage + (this.DAI_ERROR_MESSAGE_URL != null ? this.ERROR_MESSAGE_FOOTER : "") + "</html>");
        }
        editorPane.setEditable(false);
        editorPane.setOpaque(false);

        return this.addHyperLinkListener(editorPane);
    }

    /**
     * This method will allow the hyperlink to be clicked and opened up using the default browser on the users system.
     * @param editorPane The editor pane to add the hyperlink listener to
     * @return The editor pane with the hyperlink listener added
     */
    private JEditorPane addHyperLinkListener(JEditorPane editorPane) {
        editorPane.addHyperlinkListener(e -> {
            if (HyperlinkEvent.EventType.ACTIVATED.equals(e.getEventType())) {
                Desktop desktop = Desktop.isDesktopSupported() ? Desktop.getDesktop() : null;
                if (desktop != null && desktop.isSupported(Desktop.Action.BROWSE)) {
                    try {
                        desktop.browse(new URI(e.getURL().toString()));
                    } catch (Exception ex) {
                        SS3SignStreamApplication.logger.log(Level.SEVERE, "Error opening link", ex);
                    }
                }
            }
        });

        return editorPane;
    }


    /**
     * This method does several things:
     * 1. Checks the current version of the SignBank
     * 2. Checks the duration of the splice
     * 3. Checks if the video files exist
     * 4. If the video files exist, it adds them to the videoComboBox for the end user to select a video they want to splice to use for the DAI search
     * If any of these checks fail, an error message is displayed to the end user and the method returns
     * If the method succeeds and the user selects OK to the videoComboBox, the initializeDAISearch method is called
     */
    public void prepareAndStartDAISearch() {
        if (this.DAI_PROCESSING_ENDPOINT == null) {
            this.parentComponent.getDaiVideoSearch().setEnabled(false);
            this.parentComponent.revalidate();
            this.parentComponent.repaint();
            return;
        }

        ArrayList<File> videoFiles = this.getVideoFiles();

        if (!(this.checkCurrentSignBankVersion() && this.checkSpliceDuration(this.startTime, this.endTime))) {
            return;
        }

        if (this.checkIfVideoFilesExist(videoFiles)) { // Make sure the video wasn't deleted between when they were selected and now
            for (File fileName : videoFiles) {
                this.videoComboBox.addItem(fileName.getName());
            }

            if (videoFiles.size() > 1) {
                JLabel videoLabel = new JLabel("<html>Please select a video to use for the DAI search. You will get best results<br>when the video is front-facing and includes a clear view of the signer's hands.</html>");
                JPanel panel = new JPanel(new BorderLayout());
                panel.add(videoLabel, BorderLayout.NORTH);
                panel.add(this.videoComboBox, BorderLayout.CENTER);

                this.videoComboBox.setSelectedItem(this.defaultDAIVideoNames.get(this.currentSegment.getID()));

                int result = JOptionPane.showConfirmDialog(this.parentComponent, panel, "Select Video", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);

                if (result == JOptionPane.CANCEL_OPTION || result == JOptionPane.CLOSED_OPTION) {
                    return;
                }
            } else if (videoFiles.size() == 1 && !SS3Singleton.getDAICameraAnglePrompt()) {
                SS3Singleton.setDAICameraAnglePrompt();
                String title = "Single Video Selected";
                String message = "There is only one video loaded into SignStream. You will get best results when the video is front-facing and includes a clear view of the signer's hands.";
                int optionType = JOptionPane.OK_CANCEL_OPTION;
                int messageType = JOptionPane.INFORMATION_MESSAGE;

                int results = JOptionPane.showConfirmDialog(this.parentComponent, this.createEditorPane(divideString(50, message, "<br>")), title, optionType, messageType);

                if (results == JOptionPane.CANCEL_OPTION || results == JOptionPane.CLOSED_OPTION) {
                    return;
                }
            }

            this.defaultDAIVideoNames.put(this.currentSegment.getID(), (String) this.videoComboBox.getSelectedItem());
            this.initializeDAISearch(videoFiles, this.videoComboBox, this.startTime, this.endTime);
        }

    }

    /**
     * @return Gets the video files currently loaded into the SignStream application; treats the video files as a list of File objects
     */
    private ArrayList<File> getVideoFiles() {
        ArrayList<File> videoFiles = new ArrayList<>();

        for (Video video : SS3Singleton.getMediaPlayer().videos()) {
            videoFiles.add(new File(video.uri()));
        }

        return videoFiles;
    }

    /**
     * @return Checks the current sign bank version, true if the latest sign bank version is installed, an error message is displayed to the end user and the method returns false
     */
    private boolean checkCurrentSignBankVersion() {
        SignBankCollection collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
        String currentVersion = collection.getSignBankSchemaVersion(Util.SIGN_BANK_DEFAULT_GLOBAL_FILE);


        if (currentVersion == null) {
            String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_INVALID_SIGNBANK_VERSION);
            String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_INVALID_SIGNBANK_VERSION_TITLE);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.WARNING_MESSAGE;
            String[] options = {"OK"};


            JOptionPane.showOptionDialog(this.parentComponent, errorMessage, errorTitle, optionType, messageType, null, options, options[0]);

            return false;
        }

        return true;
    }

    /**
     * @param startTime The start time of the DAI search
     * @param endTime The end time of the DAI search
     * @return Checks if the duration the user has request to splice is greater than 7 seconds, if so, an error message is displayed to the end user and the method returns false
     */
    private boolean checkSpliceDuration(Time startTime, Time endTime) {
        Duration duration = endTime.subtract(startTime);

        if (duration.greaterThan(new Duration(Constants.DAI_SEARCH_MAXIMUM_ALLOWED_VIDEO_TIME, TimeUnit.SECOND))) {
            String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_VIDEO_TOO_LONG);
            String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_VIDEO_TOO_LONG_TITLE);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);

            return false;
        }

        return true;
    }

    /**
     * @param videoFiles ArrayList of all the video files
     * @param selectedVideoFileName The string name of the video file the user has selected from the videoComboBox
     * @return The file object of the video file the user has selected from the videoComboBox
     */
    private File getVideoFileSelected(ArrayList<File> videoFiles, String selectedVideoFileName) {
        for (File videoFile : videoFiles) {
            if (videoFile.getName().equals(selectedVideoFileName)) {
                return videoFile;
            }
        }

        return null;
    }


    /**
     * Displays an error message if the video splicer failed
     */
    private void videoSplicerFailure() {
        String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_VIDEO_FRAGMENT_CREATION_FAILED);
        String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_VIDEO_FRAGMENT_CREATION_FAILED_TITLE);
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String[] options = {"OK"};

        JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * At one point in time, when I was developing this, to get a working loading bar, I had to change the look and feel of the application to the cross-platform look and feel.
     * We ultimately decided not to do this, this was a MacOS limitation. Even though this method is not used, I'm leaving it here for future reference in case we want to change the look.
     * Alternatively, you can make your own animated loading bar using a custom JPanel, but I didn't have time to do that.
     */
    private void resetLookAndFeel() {
        try { // Try to set the look and feel back to the system look and feel. The change in look and feel was only for the progress bars.
            /*
             * On MacOS, the look and feel for indeterminate progress bars is not supported, and so it looks like the application is frozen.
             * To get around this, we set the look and feel to the cross platform look and feel, which is supported on all platforms and is more generally supported.
             */
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException | UnsupportedLookAndFeelException | IllegalAccessException |
                 InstantiationException e) {
            SS3SignStreamApplication.logger.log(Level.SEVERE, "Error setting look and feel", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * @param videoFiles ArrayList of all the video files
     * @param videoComboBox The combobox that contains the list of video files
     * @param startTime The start time of the DAI search
     * @param endTime The end time of the DAI search
     * @param sessionUUIDVideo The session UUID of the video file with the .mp4 extension at the end
     * @param daiSearchPerformingWaitingPage The dynamic loading waiting page for the DAI search
     * @param dialogStartTime The start time of the DAI search
     * @return A SwingWorker that will do the video splicing and the DAI search
     */
    private SwingWorker<DAIResponse, Void> initializePOSTWorker(ArrayList<File> videoFiles, JComboBox<String> videoComboBox,
                                                                Time startTime, Time endTime, String sessionUUIDVideo, DynamicDAIWaitingPage daiSearchPerformingWaitingPage, long dialogStartTime) {
        return new SwingWorker<>() {
            @Override
            protected DAIResponse doInBackground() {
                VideoSplicer videoSplicer = new VideoSplicer(startTime, endTime, DAISearch.this.getVideoFileSelected(videoFiles, (String) videoComboBox.getSelectedItem()), sessionUUIDVideo);
                boolean readyToSplice = videoSplicer.initializeSplicer();
                boolean spliceSuccess;

                if (readyToSplice) {
                    spliceSuccess = videoSplicer.spliceVideo();
                } else {
                    return null;
                }

                if (this.isCancelled() || !spliceSuccess) { // If video splicing was cancelled or failed, delete the spliced video file and return null
                    deleteSplicedVideoFile(sessionUUIDVideo);

                    this.hideAndDisposeWaitingPage();

                    if (!spliceSuccess && !this.isCancelled()) {
                        this.handleSpliceFailure();
                        return null;
                    } else if (this.isCancelled()) {
                        this.logSevereError("Video splicing cancelled");
                        return null;
                    }
                }

                SS3SignStreamApplication.logger.log(Level.INFO, "Video splicing completed successfully");
                return DAISearch.this.postVideoToDAI(sessionUUIDVideo, daiSearchPerformingWaitingPage, this); // Do the post request to DAI and return the response.
            }

            // Helper functions
            private void logSevereError(String errorMessage) {
                SS3SignStreamApplication.logger.log(Level.SEVERE, errorMessage);
            }

            private void hideAndDisposeWaitingPage() {
                daiSearchPerformingWaitingPage.setVisible(false);
                daiSearchPerformingWaitingPage.dispose();
            }

            private void handleSpliceFailure() {
                DAISearch.this.videoSplicerFailure();
                this.logSevereError("Video splicing failed");
            }

            @Override
            protected void done() {
                // Determine if the DAI search was cancelled or not
                if (this.isCancelled()) {
                    SS3SignStreamApplication.logger.log(Level.INFO, "DAI search cancelled");
                    return;
                }

                deleteSplicedVideoFile(sessionUUIDVideo);


                DAIResponse daiResponse = null;
                try {
                    if (this.isCancelled()) {
                        SS3SignStreamApplication.logger.log(Level.INFO, "DAI search cancelled");
                        return;
                    }

                    daiResponse = this.get(); // Get a DAI Response from the POST request to DAI... Check method above doInBackGround for more info
                } catch (InterruptedException | ExecutionException | CancellationException e) {
                    SS3SignStreamApplication.logger.log(Level.SEVERE, "Error getting DAI response", e);
                }

                if (daiResponse == null) {
                    daiSearchPerformingWaitingPage.dispose();
                }

                if (daiSearchPerformingWaitingPage.isVisible() || daiSearchPerformingWaitingPage.isActive()) { // Check this to avoid a race condition where the user closes the dialog before the video splicing is done.
                    long elapsedTime = System.currentTimeMillis() - dialogStartTime;
                    long remainingTime = DAISearch.this.MINIMUM_DIALOG_OPEN_TIME - elapsedTime;

                    // If less than three seconds have passed
                    if (remainingTime > 0) {
                        // Create a Timer that will execute the code after the remaining time
                        Timer timer = new Timer((int) remainingTime, e -> {
                            daiSearchPerformingWaitingPage.setVisible(false);
                            daiSearchPerformingWaitingPage.dispose();
                        });

                        timer.setRepeats(false);

                        if (this.isCancelled()) {
                            SS3SignStreamApplication.logger.log(Level.INFO, "DAI search cancelled");
                            return;
                        }

                        timer.start();
                    } else {
                        daiSearchPerformingWaitingPage.setVisible(false);
                        daiSearchPerformingWaitingPage.dispose();
                    }
                }
            }
        };
    }



    /**
     * This method checks to see what video the user wants to splice, and then calls the VideoSplicer to splice the video and get it ready for the DAI Search.
     * @param videoFiles  The list of video files from the combobox, only one video file can be selected. We pass in the File object type because we need to get the URI of the video file, not the string. The combobox and the arraylists must contain the same videos.
     * @param videoComboBox The combobox that contains the list of video files. Same videos as the arraylist of videoFiles.
     * @param startTime  The start time of the video to be spliced.
     * @param endTime   The end time of the video to be spliced.
     */
    private void initializeDAISearch(ArrayList<File> videoFiles, JComboBox<String> videoComboBox, Time startTime, Time endTime) {
        String sessionUUID = UUID.randomUUID().toString(); // Every video splice needs a random and unique UUID as the video name. This serves as the session ID for the entire DAI search and the video name.
        String sessionUUIDVideo = sessionUUID + ".mp4"; // The video name is the session UUID with the .mp4 extension.
        DynamicDAIWaitingPage daiSearchPerformingWaitingPage = this.initializeDynamicWaitingPageForDaiSearch(); // Initialize the dynamic loading waiting page for the DAI search. This is the page that will be displayed while the DAI search is being performed.
        long dialogStartTime = System.currentTimeMillis(); // Get the start time of the DAI search. This is used to calculate the total time of the DAI search.

        SwingWorker<DAIResponse, Void> worker = this.initializePOSTWorker(videoFiles, videoComboBox, startTime, endTime, sessionUUIDVideo, daiSearchPerformingWaitingPage, dialogStartTime);

        daiSearchPerformingWaitingPage.getCancelButton().addActionListener(e -> {
            worker.cancel(true);
            deleteSplicedVideoFile(sessionUUIDVideo);
            daiSearchPerformingWaitingPage.setVisible(false);
            daiSearchPerformingWaitingPage.dispose();
        });

        // Determine if daiSearchPerformingWaitingPage had the X button pressed to close it
        daiSearchPerformingWaitingPage.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                worker.cancel(true);
                deleteSplicedVideoFile(sessionUUIDVideo);
                daiSearchPerformingWaitingPage.setVisible(false);
                daiSearchPerformingWaitingPage.dispose();
            }
        });

        worker.execute();
        daiSearchPerformingWaitingPage.setVisible(true); // Show the loading bar to the user.

        if (worker.isCancelled()) {
            return;
        }

        try {
            DAIResponse response = worker.get();

            if (response != null) {
                this.getChoiceFromAPI(response);
            } else {
                deleteSplicedVideoFile(sessionUUIDVideo);
            }

        } catch (InterruptedException | ExecutionException | CancellationException e) {
            SS3SignStreamApplication.logger.log(Level.SEVERE, "Error getting DAI response", e);
            deleteSplicedVideoFile(sessionUUIDVideo);
        }
    }

    /**
     * This method will delete the spliced video file if it exists
     * @param sessionUUID The session UUID of the video file with the .mp4 extension at the end
     */
    private static void deleteSplicedVideoFile(String sessionUUID) {
        File splicedVideoFile = new File(SSResources.ffmpegOutputDirectoryPath + sessionUUID);

        if (splicedVideoFile.exists()) {
            splicedVideoFile.delete();
        }
    }

    /**
     * This method will setup the progress bar panel for the DAI search.
     */
    private DynamicDAIWaitingPage initializeDynamicWaitingPageForDaiSearch() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

        JLabel label = new JLabel(
                "<html>"
                        + "<h2 style='color: black; text-align: center;'><b>Searching DAI...</b></h2><br>"
                        + "<p style='font-size: 14px; font-family: Arial, sans-serif; color: darkgray; text-align: justify;'>"
                        + "This may take some time. Please be patient. Press the Cancel button to cancel the search."
                        + "</p>"
                        + "</html>"
        );

        label.setHorizontalAlignment(JLabel.CENTER);
        label.setFont(new Font("Serif", Font.PLAIN, 16));
        label.setBorder(BorderFactory.createEmptyBorder(0, 0, 20, 0)); // This creates an empty border below the JLabel

        panel.add(label, BorderLayout.NORTH);
        LoadingBarPanel loadingBarPanel = new LoadingBarPanel(); // This is using a GIF, do not try to do it natively in Java Swing
        // MacOS does not like indeterminate progress bars, the Aqua look and feel does not support it. You can get it to work by setting
        // the default look and feel to something that is supported, like the cross platform look and feel, but we decided against that for now.
        panel.add(loadingBarPanel, BorderLayout.CENTER);

        JButton cancelButton = new JButton("Cancel");

        panel.add(cancelButton, BorderLayout.SOUTH);

        DynamicDAIWaitingPage dynamicDAIWaitingPage = new DynamicDAIWaitingPage();
        dynamicDAIWaitingPage.setTitle("DAI Video Search");
        // dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        // DO nothing on close
        dynamicDAIWaitingPage.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        dynamicDAIWaitingPage.add(panel);

        dynamicDAIWaitingPage.pack();
        dynamicDAIWaitingPage.setLocationRelativeTo(null);
        dynamicDAIWaitingPage.setModal(true);
        dynamicDAIWaitingPage.setResizable(true);
        dynamicDAIWaitingPage.setVisible(false);

        dynamicDAIWaitingPage.setCancelButton(cancelButton);

        return dynamicDAIWaitingPage;
    }

    /**
     * @param uri The URI to open in the browser
     */
    private void openDesktopBrowser(URI uri) {
        try {
            Desktop.getDesktop().browse(uri);
        } catch (IOException e) {
            String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_FAILED_TO_OPEN_WEB_BROWSER);
            String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_FAILED_TO_OPEN_WEB_BROWSER_TITLE);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);

            SS3SignStreamApplication.logger.log(Level.SEVERE, "Failed to open web browser", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * Occurs in the event where the DAI does not send a properly formatted URI to open the browser
     */
    private void malformedResponseURIDialog() {
        String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_MALFORMED_RESPONSE_URI);
        String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_MALFORMED_RESPONSE_URI_TITLE);
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String[] options = {"OK"};

        JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * @param worker The worker that was pinging the DAI search endpoint waiting for a response
     * @return The DAI response from the DAI search endpoint, if not null, then it's a valid ChoiceResponse
     */
    private BasicResponse handleChoiceResponse(SwingWorker<BasicResponse, Void> worker) {
        BasicResponse daiResponse;
        try {
            daiResponse = worker.get();
            if (daiResponse == null) {
                this.handleInvalidRequest(this.DAI_CHOICE_ENDPOINT, "DAI Choice API", null, this.associatedFieldsParentDialog);
                SS3SignStreamApplication.logger.log(Level.SEVERE, "Failed to get DAI Choice Response: Bad URL");
                return null;
            } else if (!this.processHTTPStatusCode(daiResponse.getHttpStatusCode(), this.associatedFieldsParentDialog, this.DAI_CHOICE_ENDPOINT, worker)) {
                return null;
            }
        } catch (InterruptedException | ExecutionException e) {
            SS3SignStreamApplication.logger.log(Level.SEVERE, "Failed to get DAI Choice Response", e);

            String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_GENERAL_CHOICE_FAILURE);
            String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_GENERAL_CHOICE_FAILURE_TITLE);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);

            throw new RuntimeException(e);
        }

        return daiResponse;
    }

    /**
     * This method will open the browser with the DAI search results and get the user's choice they selected from the web session.
     * @param response The response from the DAI Search
     */
    private void getChoiceFromAPI(DAIResponse response) throws InterruptedException {
        try {
            if (this.DAI_BROWSER_LINK == null) {
                this.DAI_BROWSER_LINK = response.getResultsLink();
            } else {
                this.DAI_BROWSER_LINK = this.DAI_BROWSER_LINK + response.getSession(); // Concatenate the session ID to the end of the DAI browser link
            }
            this.openDesktopBrowser(new URI(this.DAI_BROWSER_LINK));
        } catch (URISyntaxException e) {
            this.malformedResponseURIDialog();
            SS3SignStreamApplication.logger.log(Level.SEVERE, "URI Format Exception", e);
            throw new RuntimeException(e);
        }

        if (this.DAI_CHOICE_ENDPOINT == null) {
            this.DAI_CHOICE_ENDPOINT = response.getChoiceAPILink();
        }

        RequestBuilder requestBuilder = new RequestBuilder(this.DAI_CHOICE_ENDPOINT, RequestType.GET);
        HttpRequest request = new HttpRequest(requestBuilder);
        JDialog waitingForUserChoiceDialog = this.initializeWaitingPageForUserChoice();

        SwingWorker<BasicResponse, Void> worker = this.getChoiceResponseWorker(request, waitingForUserChoiceDialog);
        waitingForUserChoiceDialog.setVisible(true);

        JOptionPane waitingForUserChoiceDialogOption = (JOptionPane) waitingForUserChoiceDialog.getRootPane().getContentPane().getComponent(0);

        // GREG TODO: this should be changed to a smarter way, but for now, mind this exact string "Close Window" and if you change it,
        // also change it where it is used in initializeWaitingPageForUserChoice
        if ( waitingForUserChoiceDialogOption.getValue() == null || waitingForUserChoiceDialogOption.getValue().equals("Close Window")) { // If the user clicks cancel, cancel the worker and return or if the user clicks the X button on the dialog
            worker.cancel(true);
            // wait for the worker to finish
            while (!worker.isDone()) {
                Thread.sleep(250);
            }
            waitingForUserChoiceDialog.dispose();
            return;
        }

        BasicResponse daiResponse = this.handleChoiceResponse(worker);

        if (daiResponse == null) {
            return;
        }

        try {
            ChoiceResponse choiceResponse = new ChoiceResponse(daiResponse.getHttpStatusCode(), daiResponse.getContentLength(), daiResponse.getContentType(), daiResponse.getContentJSON()); // The users choice
            this.processChoiceResponse(choiceResponse);
        } catch (JsonProcessingException | NullPointerException e) {
            this.handleBadDAIServerResponse(this.associatedFieldsParentDialog, daiResponse.getContentJSON(), e);
        }
    }

    /**
     * @param request The GET request to the DAI Choice API
     * @param waitingForUserChoiceDialog The dialog that shows up while the user is selecting a choice from the DAI Search results
     * @return The SwingWorker that will ping the DAI Choice API waiting for a response and return the response
     */
    private SwingWorker<BasicResponse, Void> getChoiceResponseWorker(HttpRequest request, JDialog waitingForUserChoiceDialog) {
        Predicate<BasicResponse> requestTerminator = basicResponse -> { // This is the condition we want the http pinging request to terminate on.
            try {
                ChoiceResponse choiceResponse = new ChoiceResponse(basicResponse.getHttpStatusCode(), basicResponse.getContentLength(), basicResponse.getContentType(), basicResponse.getContentJSON());
                if (choiceResponse.getChoiceStatusCode() != this.CHOICE_NOT_MADE) {
                    return true;
                }
            } catch (JsonProcessingException | NullPointerException e) {
                return false;
            }

            return false;
        };

        return this.createChoiceResponseWorker(request, waitingForUserChoiceDialog, requestTerminator);
    }

    /**
     * @param request The GET request to the DAI Choice API
     * @param waitingForUserChoiceDialog The dialog that shows up while the user is selecting a choice from the DAI Search results
     * @param requestTerminator The condition we want the http pinging request to terminate on.
     * @return The SwingWorker that will ping the DAI Choice API waiting for a response and return the response
     */
    private SwingWorker<BasicResponse, Void> createChoiceResponseWorker(HttpRequest request, JDialog waitingForUserChoiceDialog, Predicate<BasicResponse> requestTerminator) {
        SwingWorker<BasicResponse, Void> worker = new SwingWorker<>() {
            @Override
            protected BasicResponse doInBackground() throws InterruptedException {
                Thread.sleep(DAISearch.this.DAI_PAUSE_TIME);

                BasicResponse response;

                try {
                    if (this.isCancelled()) {
                        return null;
                    }
                    // GREG HERE
                    // The HTTPRequest will wait for a response based on these parameters and terminate based on the requestTerminator.
                    response = request.waitForResponse(DAISearch.this.DAI_SEARCH_TIMEOUT, DAISearch.this.DAI_POLLING_RATE, requestTerminator);
                } catch (IOException e) {
                    return null;
                }

                return response;
            }

            @Override
            protected void done() {
                waitingForUserChoiceDialog.dispose();
            }
        };

        worker.execute();
        return worker;
    }

    /**
     * We don't use "createEditorPane" because we don't want the error message. The methods that call this method
     * will display non-error messages. So, calling this method is more appropriate for those cases.
     * @param htmlText The text to be displayed in the JEditorPane
     * @return A JEditorPane that will display the text in htmlText
     */
    private JEditorPane createTextPane(String htmlText) {
        JTextPane textPane = new JTextPane();
        textPane.setEditable(false);
        textPane.setContentType("text/html");
        textPane.setOpaque(false);
        textPane.setText(htmlText);

        return this.addHyperLinkListener(textPane);
    }

    /**
     * @return The waiting page that will be displayed while the user is selecting a choice from the DAI Search results
     */
    private JDialog initializeWaitingPageForUserChoice() {
        JDialog dialog;

        String informationMessage = String.format("Please select the Sign you want to use from the DAI Search results. " + // TODO: Hasnain, should this be in the errors?
                "If you do not select a Sign within %d seconds, the DAI search " +
                "will be cancelled and you will have to try again.<br><br>", this.DAI_SEARCH_TIMEOUT / 1000);

        informationMessage = divideString(100, informationMessage, "<br>");
        informationMessage = String.format("%sIf a browser window does not automatically open, you can select your choice here:<br><a href=\"%s\">%s</a>", informationMessage, this.DAI_BROWSER_LINK, this.DAI_BROWSER_LINK);

        String informationTitle = "Awaiting User Selection from DAI";
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.INFORMATION_MESSAGE;
        // GREG TODO: this should be changed to a smarter way, but for now, mind this exact string "Close Window" and if you change it,
        // also change it where it is used in getChoiceFromAPI
        String[] options = {"Close Window"};

        JOptionPane optionPane = new JOptionPane(this.createTextPane(informationMessage), messageType, optionType, null, options, options[0]);

        dialog = optionPane.createDialog(this.parentComponent, informationTitle);
        dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

        dialog.pack();

        return dialog;
    }

    /**
     * This method will interpret the response from the DAI Choice API and display the appropriate error message to the end user, or process the selection.
     * @param choiceResponse The response from the DAI Choice API
     */
    private void processChoiceResponse(ChoiceResponse choiceResponse) {
        if (choiceResponse.getChoiceStatusCode() != this.SUCCESS) {
            String errorMessage;
            String errorTitle;

            if (choiceResponse.getChoiceStatusCode() == this.CHOICE_NOT_MADE) {
                errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_TIMEOUT);
                errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_TIMEOUT_TITLE);

                int optionType = JOptionPane.DEFAULT_OPTION;
                int messageType = JOptionPane.ERROR_MESSAGE;
                String[] options = {"OK"};
                
                this.parentComponent.getParentDialog().setAlwaysOnTop(true);
                this.parentComponent.requestFocus();
                this.parentComponent.getParentDialog().toFront();
                
                // just putting 1000 to make it so long that it doesn't matter about the line breaks, already done with the message itself
                JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(1000, errorMessage, "<br>"), true), errorTitle, optionType, messageType, null, options, options[0]);

                this.parentComponent.getParentDialog().setAlwaysOnTop(false);
                
                return;
            }

            if (choiceResponse.getMessage().isBlank() || choiceResponse.getMessage().isEmpty() || choiceResponse.getMessage() == null) {
                errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_GENERAL_CHOICE_FAILURE);
                errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_GENERAL_CHOICE_FAILURE_TITLE);
            } else {
               errorMessage = String.format("DAI Search Failed.\nDAI Status Code: %d\nReason Given: %s", choiceResponse.getChoiceStatusCode(), choiceResponse.getMessage());
               errorTitle = "DAI Search Failed";
            }

            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(this.parentComponent,  errorMessage, errorTitle, optionType, messageType, null, options, options[0]);

            
            return;
        } else if (choiceResponse.getSchemaVersion() == null || choiceResponse.getDataVersion() == null || !this.compareSignBankVersionsToCurrentVersion(choiceResponse.getSchemaVersion(), choiceResponse.getDataVersion())) {
            String statement = divideString(50, SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_CANNOT_POPULATE_SIGN), "\n");
            String reason = divideString(50, SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_REASON_SIGN_BANK_UPDATE_NEEDED), "\n");
            String solution = divideString(50, SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_SOLUTION_UPDATE_SIGN_BANK), "\n");

            String errorMessage = String.format("%s\n\n%s\n\n%s", statement, reason, solution);
            String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_MISSING_SCHEMA_OR_DATA_VERSION_TITLE);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(this.parentComponent, errorMessage, errorTitle, optionType, messageType, null, options, options[0]);

            return;
        }


        this.processSelection(choiceResponse);
    }

    /**
     * @param schemaVersion The schema version object of the DAI Choice API response
     * @param dataVersion The data version object of the DAI Choice API response
     * @return True if the schema and data versions of the DAI Choice API response match the current schema and data versions of the Sign Bank, false otherwise
     */
    private boolean compareSignBankVersionsToCurrentVersion(Version schemaVersion, Version dataVersion) {
        Version currentSchemaVersion = new Version(SS3Singleton.getSignStreamApplication().getSignBankCollection().getSignBankSchemaVersion(Util.SIGN_BANK_DEFAULT_GLOBAL_FILE));
        Version currentDataVersion = new Version(SS3Singleton.getSignStreamApplication().getSignBankCollection().getSignBankDataVersion(Util.SIGN_BANK_DEFAULT_GLOBAL_FILE));


        return schemaVersion.compareTo(currentSchemaVersion) == 0 && dataVersion.compareTo(currentDataVersion) == 0;
    }

    /**
     * @param signText The sign text that was searched for in the DAI Search
     */
    private void signNotFound(String signText) {
        String errorMessage = String.format("SignStream could not insert the sign: <b>%s</b>.<br><br>You can enter the information manually for this sign or " +
                "you can try searching the DAI <html><a href=%s>%s</a>.</html>", signText, this.DAI_ERROR_MESSAGE_URL, this.DAI_ERROR_MESSAGE_URL);
        String errorTitle = "Sign Bank Search Failure";
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String[] options = new String[]{"OK"};


        JEditorPane editorPane = new JEditorPane("text/html", divideString(75, errorMessage, "<br>"));
        editorPane.setFont(UIManager.getFont("Label.font"));
        editorPane.setText(errorMessage);
        editorPane.setEditable(false);
        editorPane.setOpaque(false);

        JOptionPane.showOptionDialog(this.parentComponent, editorPane, errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * SignStream does not support compound signs. If the user selects a compound sign from the DAI Search results, this method will be called.
     * @param signText The sign text that was searched for in the DAI Search
     */
    private void compoundSignSelectedErrorMessage(String signText) {
        String statement = divideString(50, "You selected the sign: ", "<br>") + String.format("%s.<br> ", signText) +
                divideString(50, "It cannot be successfully entered directly into SignStream.", "<br>");
        String reason = divideString(50, "The DAI returned a compound as the result.", "<br>");
        String solution = divideString(50, "Solution: You will need to enter the gloss for each part " +
                "individually and then combine them to form a compound in the utterance window. You may be able to find " +
                "the compound parts separately in the Sign Bank (by clicking on Search SignBank) to facilitate data entry; " +
                "however, if the sign exists in the SignBank only as part of a compound, the gloss will appear with an " +
                "asterisk before or after the text of the gloss, and you will not be able to automatically enter the " +
                "information from that sign.You will then need to enter the information manually.", "<br>");


        String errorMessage = String.format("%s<br><br>%s<br><br>%s", statement, reason, solution);
        String errorTitle = "Error - Compound Sign Returned";

        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String[] options = new String[]{"OK"};

        JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(errorMessage), errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * This method will process the users DAI Search selection and populate the AssociatedFieldsPanel with the data from the Sign Bank based on the selection.
     * @param choiceResponse The response from the DAI Choice API
     */
    public void processSelection(ChoiceResponse choiceResponse) {
        String signText = choiceResponse.getSignText();

        if (signText == null) {
            this.nullSignTextErrorMessage();
            return;
        } else if (signText.isEmpty() || signText.isBlank()) {
            this.emptySignTextErrorMessage();
            return;
        } else if (SignBankVariant.isCompound(signText)) {
            this.compoundSignSelectedErrorMessage(signText);
            return;
        }

        SignBankOccurrence occurrence = SS3Singleton.getSignStreamApplication().getSignBankCollection().lookupSpecificVariantMatchingText(signText); // This will take the response from the DAI server and search through the local sign-bank for the responded sign data, attempting to populate it into SignSTream

        if (occurrence == null) {
            this.signNotFound(signText);
            return;
        }

        String successMessage = String.format("You selected the sign: %s\nSignStream will now populate the fields in the Morph-Phon Window with the data " +
                "associated with %s.", signText, signText);
        String successTitle = "DAI Search Complete";
        int optionType = JOptionPane.YES_NO_OPTION;
        int messageType = JOptionPane.INFORMATION_MESSAGE;
        String[] options = {"OK", "Cancel"};

        int selection = JOptionPane.showOptionDialog(this.parentComponent, successMessage, successTitle, optionType, messageType, null, options, options[0]);

        if (selection == JOptionPane.NO_OPTION || selection == JOptionPane.CLOSED_OPTION) {
            return;
        }


        this.parentComponent.populateFromSignBankOccurrence(occurrence);
    }

    /**
     * The error message to be displayed to the user if an empty sign text is returned from the DAI Search
     */
    private void emptySignTextErrorMessage() {
        String errorMessage = "DAI Search Failed: A selected Sign was missing from the chosen search result.";
        String errorTitle = "Error - Empty Sign Text Returned";
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.WARNING_MESSAGE;
        String[] options = new String[]{"OK"};

        JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(errorMessage, true), errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * The error message to be displayed to the user if a null sign text is returned from the DAI Search
     */
    private void nullSignTextErrorMessage() {
        String errorMessage = "The DAI search failed.  You did not find the sign you were looking for<br>"
        		+ "among the choices offered.  You can try using the \"Search Sign Bank\" button<br>"
        		+ "in the Morph-Phon Window, or you can try other ways of searching the DAI.";
        String errorTitle = "Error - Null Sign Text Returned";
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.WARNING_MESSAGE;
        String[] options = new String[]{"OK"};

        JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(errorMessage, true), errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * This method will post the video file to the DAI Search Server and return the response from the DAI Search Server after doing some basic response checking on the response to ensure it is a valid DAI Response
     * @param sessionUUIDVideo The session UUID of the video splicer process and current DAI search
     * @return The response from the DAI Search after sending the POST request with the video file
     */
    private DAIResponse postVideoToDAI(String sessionUUIDVideo, JDialog waitingPage, SwingWorker<DAIResponse, Void> worker) {
        // Create an HTTPPost request
        File videoFile = new File(SSResources.ffmpegOutputDirectoryPath + "/" + sessionUUIDVideo);

        RequestBuilder requestBuilder = new RequestBuilder(this.DAI_PROCESSING_ENDPOINT, "DAI_VIDEO_POST", videoFile, ContentType.create("video/mp4"), RequestType.POST);

        HttpRequest httpPoster = new HttpRequest(requestBuilder);
        SS3SignStreamApplication.logger.log(Level.INFO, "Created HTTP POST Request To " + requestBuilder.getUrl());

        BasicResponse basicResponse;

        if (worker.isCancelled()) {
            return null;
        }

        try {
            basicResponse = httpPoster.executeHttpRequest(); // Send the HTTP POST request to the DAI Search Server... This is where the video gets "sent" to the DAI Search Server

            if (worker.isCancelled()) {
                return null;
            }
        } catch (IOException e) {
            if (e instanceof ClientProtocolException || e instanceof org.apache.hc.client5.http.ClientProtocolException || e instanceof SSLException ||
                    e instanceof UnknownHostException || e instanceof ConnectException || e instanceof SocketTimeoutException) { // Of the various HTTP and SSL and Connection Exceptions we check for
                if (!worker.isCancelled()) {
                    this.handleInvalidRequest(this.DAI_PROCESSING_ENDPOINT, "DAI Video POST", e, waitingPage);
                }
                return null;
            }

            return null;
        } catch (CancellationException e) {
            return null;
        }


        DAIResponse daiResponse;

        if (worker.isCancelled()) { // We need to make these repeated checks to ensure that the worker was not cancelled while we were waiting for the response to avoid race conditions as Java Swing doesn't really allow for elegant cancellation of SwingWorkers
            SS3SignStreamApplication.logger.log(Level.INFO, "DAI Video POST Request Cancelled");
            return null;
        }

        boolean httpStatusCodeSuccess = this.processHTTPStatusCode(basicResponse.getHttpStatusCode(), waitingPage, this.DAI_PROCESSING_ENDPOINT, worker); // Check the HTTP Status Code to ensure that we got 200, or handle the error if we didn't

        if (httpStatusCodeSuccess) {
            try {
                daiResponse = new DAIResponse(basicResponse.getHttpStatusCode(), basicResponse.getContentLength(), basicResponse.getContentType(), basicResponse.getContentJSON());
            } catch (JsonProcessingException | NullPointerException e) {
                this.handleBadDAIServerResponse(waitingPage, basicResponse.getContentJSON(), e);
                return null;
            }
        } else {
            return null;
        }

        boolean daiResponseSuccess = this.processDAIResponseCode(waitingPage, daiResponse.getDAIStatusCode(), daiResponse.getMessage());

        if (daiResponseSuccess) {
            return daiResponse;
        } else {
            return null;
        }
    }

    /**
     * This method will process HTTP and SSL and invalid request exceptions, mainly any errors that might occur while sending a request to the DAI Search Server.
     * For example, SSLExceptions, invalid connections, expired certifications, insecure pages, etc.
     * @param url The URL that was used to make the request
     * @param urlType The type of URL that was used to make the request ("DAI Choice API" or "DAI Video POST")
     * @param e The exception that was thrown while processing the request
     * @param waitingPage The waiting page that was displayed while the user was selecting a choice from the DAI Search results
     */
    private void handleInvalidRequest(String url, String urlType, IOException e, JDialog waitingPage) {
        String errorMessage;

        if (e instanceof SSLHandshakeException) {
            errorMessage = "Search failed. SignStream was unable to connect to the website because of an expired SSL certificate.";
        } else if (e instanceof SSLPeerUnverifiedException) {
            errorMessage = String.format("Search failed. SignStream was unable to connect to the website because of an the %s URL is unverified and possibly dangerous.", urlType);
        } else if (e == null) {
            errorMessage = String.format("The %s response was invalid. That is all we know at this time.", urlType);
        } else if (e instanceof HttpHostConnectException) {
            errorMessage = String.format("Search failed. SignStream was unable to connect to the DAI Server [ %s ].", url);
        } else if (e instanceof UnknownHostException) {
            errorMessage = String.format("Search failed. SignStream was unable to connect to the DAI Server. There was a domain name server error on %s.", url);
        } else {
            errorMessage = String.format("The %s URL caused an exception. This is an unexpected error. Error: %s", urlType, e.getCause().getMessage());
            SS3SignStreamApplication.logger.log(Level.SEVERE, "An unexpected exception occurred while processing the " + urlType + " URL.", e);
        }

        String errorTitle = "Invalid URL";
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String[] options = {"OK"};

        JOptionPane.showOptionDialog(waitingPage, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);
    }

    /**
     * @param x The number of characters to place a delimiter after
     * @param contentString The string to split
     * @param delimiter The delimiter to place after every x characters
     * @return A string that is split into parts with the delimiter after every x characters. This method will cut off words in the middle and put them on the next line.
     * Unlike the divideString method, this method will cut off words in the middle and put them on the next line.
     */
    public static String splitStringXParts(int x, String contentString, String delimiter) {
        // Place a newline character after every x characters
        StringBuilder stringBuilder = new StringBuilder(contentString);

        int insertPosition = x;
        while (insertPosition < stringBuilder.length()) {
            stringBuilder.insert(insertPosition, delimiter);
            insertPosition += x + 1;  // Adjust the position
        }

        return stringBuilder.toString();
    }

    /**
     * @param parts The number of parts to divide the string into
     * @param str The string to divide
     * @param delimeter The delimeter to place between each part
     * @return A string that is divided into parts with the delimeter between each part. This method will respect word boundaries and will not cut off words in the middle.
     * Thus, it will wait until the end of the word to place the delimeter by looking for the next space. THIS METHOD IS NOT THE SAME AS splitStringXParts!!!
     */
    public static String divideString(int parts, String str, String delimeter) {
        if (parts <= 0 || str == null || (str.isEmpty() || str.isBlank())) {
            return str;
        }

        StringBuilder sb = new StringBuilder();
        int currentCount = 0;

        for (String word : str.split(" ")) {
            if (currentCount + word.length() > parts) {
                sb.append(delimeter);
                currentCount = 0;
            }
            sb.append(word).append(" ");
            currentCount += word.length() + 1; // adding one for space
        }

        return sb.toString().strip();
    }

    /**
     * If the DAI server returned a bad response when either posting a video or getting a choice, this method will be called. This method will be called before converting a BasicResponse into anything else.
     * @param waitingPage The waiting page that will be displayed while the user is selecting a choice from the DAI Search results
     * @param contentJSON The content JSON that was returned from the DAI server
     * @param e The exception that was thrown while parsing the content JSON
     */
    private void handleBadDAIServerResponse(JDialog waitingPage, String contentJSON, Exception e) {
        String errorMessage = String.format("Search Failed. The data that was returned was invalid, and SignStream was unable to process it.<br><br>%s", splitStringXParts(50, contentJSON, "<br>"));
        String errorTitle = "Invalid Response from DAI Server";
        int optionType = JOptionPane.DEFAULT_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String[] options = {"OK"};

        JOptionPane.showOptionDialog(waitingPage, this.createEditorPane(errorMessage), errorTitle, optionType, messageType, null, options, options[0]);

        SS3SignStreamApplication.logger.log(Level.SEVERE, "Invalid response from DAI server", e);

        ObjectMapper mapper = new ObjectMapper();
        try {
            InvalidJSONResponse invalidJSONResponse = mapper.readValue(contentJSON, InvalidJSONResponse.class);
            SS3SignStreamApplication.logger.log(Level.SEVERE, String.format("Content JSON Returned:<br>%s", invalidJSONResponse.getData()));
        } catch (JsonProcessingException ex) {
            SS3SignStreamApplication.logger.log(Level.SEVERE, String.format("Bad Response From DAI: Could not parse JSON response into invalid response.<br>Content JSON: %s<br>", contentJSON));
        }
    }

    /**
     * This method will process the response from the DAI server of the POST request and ensure that the POST request was successful, or display an error message otherwise.
     * @param statusCode The HTTP status code of the POST request to the DAI server
     * @return True if the DAI search should continue, false otherwise
     */
    private boolean processHTTPStatusCode(int statusCode, JDialog waitingPage, String url, SwingWorker<? extends BasicResponse, Void> worker) {
        String responseType = "";

        // First we process HTTP error codes
        if (statusCode >= 100 && statusCode < 200) {
            responseType = "informational";
        } else if (statusCode >= 300 && statusCode < 400) {
            responseType = "redirection";
        } else if (statusCode >= 400 && statusCode < 500) {
            responseType = "client error";
        } else if (statusCode >= 500 && statusCode < 600) {
            responseType = "server error";
        } else if (statusCode >= 600) {
            responseType = "unknown";
        }

        if ((responseType.isEmpty() || responseType.isBlank()) && statusCode != 200) {
            responseType = "critical error"; // This should never happen in any circumstance
        }

        if (statusCode >= 400 && statusCode < 500) { // Special handling of 400 errors
            String errorMessage = String.format("Search failed. SignStream was unable to connect to the DAI Server: [ %s ]. The target page was not found.", url);
            String errorTitle = String.format("DAI Video Search Failure - (%d)", statusCode);

            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.WARNING_MESSAGE;
            String[] options = {"OK"};

            if (worker.isCancelled()) {
                SS3SignStreamApplication.logger.log(Level.INFO, "DAI Video POST Request Cancelled");
                return false;
            }

            JOptionPane.showOptionDialog(waitingPage, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);

            return false;
        }

        if (!responseType.isEmpty() || !responseType.isBlank()) { // If we did encounter an error, responseType was updated with an error message. If it was empty, then that would indicate success.
            String errorMessage = String.format("DAI Video Search Failed. DAI Search returned a %s response (%d). Likely, the DAI server is down, or this is a bug.", responseType, statusCode);
            String errorTitle = String.format("DAI Video Search Failure - (%d)", statusCode);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.WARNING_MESSAGE;
            String[] options = {"OK"};

            if (worker.isCancelled()) {
                SS3SignStreamApplication.logger.log(Level.INFO, "DAI Video POST Request Cancelled");
                return false;
            }

            JOptionPane.showOptionDialog(waitingPage, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);

            return false;
        }

        return true;
    }

    /**
     * @param waitingPage The waiting page that will be displayed while the user is selecting a choice from the DAI Search results
     * @param daiResponseCode The DAI response code
     * @param daiResponseMessage The DAI response message
     * @return True if the DAI response code is successful, false otherwise
     */
    private boolean processDAIResponseCode(JDialog waitingPage, int daiResponseCode, String daiResponseMessage) {
        if (daiResponseCode != this.SUCCESS) {
            String errorMessage = String.format("We're sorry, the DAI Search Failed<br>DAI Status Code Provided: %d. <br>DAI Response Provided: %s", daiResponseCode, daiResponseMessage == null ? "" : daiResponseMessage);
            String errorTitle = String.format("DAI Error: %s", daiResponseCode);

            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(waitingPage, this.createEditorPane(errorMessage), errorTitle, optionType, messageType, null, options, options[0]);

            return false;
        }

        return true;
    }

    /**
     * @param directory The directory to delete
     */
    private void deleteDirectory(File directory) {
        if (directory.isDirectory()) {
            File[] files = directory.listFiles();
            if (files != null) {  // null if security restricted
                for (File file : files) {
                    if (file.isDirectory()) {
                        this.deleteDirectory(file);  // recursive call
                    } else if (directory.exists()){
                        if (!file.delete()) {
                            SS3SignStreamApplication.logger.log(Level.WARNING, String.format("Failed to delete %s", file));
                        }
                    }
                }
            }
        }

        if (!directory.delete()) {
            SS3SignStreamApplication.logger.log(Level.WARNING, String.format("Failed to delete %s", directory));
        }
    }

    /**
     * @param videoFiles The list of video files to check
     * @return True if the video files exist, false otherwise
     */
    private boolean checkIfVideoFilesExist(ArrayList<File> videoFiles) {
        if (videoFiles.isEmpty()) {
            String errorMessage = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_ZERO_VIDEOS_LOADED);
            String errorTitle = SS3Singleton.getErrorMessages().getErrorMessageString(ErrorMessages.DAI_SEARCH_ZERO_VIDEOS_LOADED_TITLE);
            int optionType = JOptionPane.DEFAULT_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String[] options = {"OK"};

            JOptionPane.showOptionDialog(this.parentComponent, this.createEditorPane(divideString(50, errorMessage, "<br>")), errorTitle, optionType, messageType, null, options, options[0]);
            return false;
        }

        return true;
    }

}

/**
 * This is the DAI Waiting Page that is displayed to the user while the DAI Search is processing
 */
class DynamicDAIWaitingPage extends JDialog {

    private JButton cancelButton;

    public JButton getCancelButton() {
        return this.cancelButton;
    }

    public JButton setCancelButton(JButton cancelButton) {
        return this.cancelButton = cancelButton;
    }
}
