package edu.bu.signstream.grepresentation.fields.palettes;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSPaletteItemGroup;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GenericStartEndHandShapeTypes;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.FieldValueWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.PalettePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class GeneralHandShapePalettePanel <T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends PalettePanel implements MouseListener {
    private final HashMap<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> folders;
    private ArrayList<JLabel> imageLabels;
    private final String currentPaletteID;
    private ArrayList<SS3FieldValue> values;
    private ArrayList<HandShapeImage> selectedHshImages;
    private ArrayList<FieldValueWrapper> panelValues;
    private int itemsInAllFolders;
    private SS3CodingScheme scheme;
    private String defaultTemplateID;
    private JPanel handShapePanel;
    private JScrollPane handShapeScrollPane;
    private JButton clearButton;
    private ParentPaletteInformationHelper<K> parentHandShapePanel; //
    private T selectedItemsPanel; // Selected items in this case is anything the user can select -- ex: handshapes, orientations, etc
    private JPanel searchPanel;
    private ArrayList<TriangleLabel> triangleLabels; // Only if there are folders will this value.

    private GridBagConstraints itemConstraints; // Constraints for each item in the panel
    private GridBagConstraints centerPanelConstraints; // Constraints for the handshape panel and search panel

    private HashMap<String, SS3FieldValue> selectedHandshapesMap;

    public GeneralHandShapePalettePanel(HandShapesPaletteFormBean formBean, SS3CodingScheme scheme,
                                        ParentPaletteInformationHelper<K> parentPaletteInformationPanel, T selectionPanel, String templateID, String currentPaletteID) {
        super();
        super.setLayout(new GridBagLayout());
        super.setBorder(new TitledBorder(formBean.getTitle()));
        this.values = formBean.getValues();
        this.selectedHshImages = formBean.getSelectedHshImages();
        this.defaultTemplateID = templateID;
        this.currentPaletteID = currentPaletteID;
        this.scheme = scheme;
        this.selectedHandshapesMap = new HashMap<>();
        this.parentHandShapePanel = parentPaletteInformationPanel;
        this.selectedItemsPanel = selectionPanel;
        this.handShapePanel = new JPanel(new GridBagLayout());
        this.handShapeScrollPane = new JScrollPane(this.handShapePanel);
        this.triangleLabels = null;
        this.clearButton = new JButton("Clear");

        for (SS3FieldValue value : this.values) {
            this.selectedHandshapesMap.put(value.getId(), value);
        }

        this.imageLabels = new ArrayList<>();
        this.folders = this.populatePanelValues();
        this.populateHandshapeImages();
    }

    public JLabel getImage(SS3FieldValue value) {
        HandShapeImage handShapeImage = new HandShapeImage(value, this.scheme);

        Image image = handShapeImage.getImage();
        JLabel imageLabel = new JLabel();
        Font imageLabelFont = new Font(imageLabel.getFont().getName(), Font.PLAIN, 10);
        imageLabel.setFont(imageLabelFont);
        imageLabel.setBorder(BorderFactory.createLineBorder(UIPalette.getBackground()));
        imageLabel.setOpaque(true);
        imageLabel.setIcon(new ImageIcon(image));
        imageLabel.setText(value.getLabel());

        imageLabel.addMouseListener(new MouseListener() { // TODO: Since we need to override the mousePressed method, we need to implement the MouseListener interface
            @Override
            public void mouseClicked(MouseEvent e) {
            }

            /**
             * Invoked when a mouse button has been pressed on a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mousePressed(MouseEvent e) {
                GeneralHandShapePalettePanel.this.selectedHshImages.add(handShapeImage);
                GeneralHandShapePalettePanel.this.selectedItemsPanel.setItem(value); // Set the item adds the item in the selected items panel
            }

            /**
             * Invoked when a mouse button has been released on a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseReleased(MouseEvent e) {
            }

            /**
             * Invoked when the mouse enters a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseEntered(MouseEvent e) {
            }

            /**
             * Invoked when the mouse exits a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseExited(MouseEvent e) {
            }
        });

        return imageLabel;
    }

    private void revalidateJComponent(JComponent component) {
        component.revalidate();
        component.repaint();
    }

    private void populateWithoutFolders() {
        this.handShapePanel.removeAll();

        ArrayList<FieldValueWrapper> values = this.folders.get((CSPaletteItemGroup) (this.folders.keySet().toArray()[0]));

        for (int x = 0; x < values.size(); x++) {
            FieldValueWrapper value = values.get(x);
            JLabel imageLabel = this.createImageLabel(value, false);

            this.itemConstraints.weightx = 1.0;
            this.itemConstraints.weighty = 0.0;
            this.itemConstraints.fill = GridBagConstraints.HORIZONTAL;
            this.itemConstraints.anchor = GridBagConstraints.NORTH;

            this.itemConstraints.gridx = x / ((int) Math.ceil(this.itemsInAllFolders / 3.0));
            this.itemConstraints.gridy = x % ((int) Math.ceil(this.itemsInAllFolders / 3.0));
            this.handShapePanel.add(imageLabel, this.itemConstraints);
        }

        GridBagConstraints dummyConstraints = new GridBagConstraints();
        dummyConstraints.weighty = 1.0;  // takes up the remaining vertical space
        dummyConstraints.gridx = 0;
        dummyConstraints.gridy = GridBagConstraints.RELATIVE;
        dummyConstraints.gridwidth = GridBagConstraints.REMAINDER;  // span across all grid columns
        dummyConstraints.fill = GridBagConstraints.BOTH;

        JPanel dummyPanel = new JPanel();
        dummyPanel.setOpaque(false);  // make it transparent
        this.handShapePanel.add(dummyPanel, dummyConstraints);

        this.handShapeScrollPane.setMinimumSize(this.handShapePanel.getPreferredSize());
        this.revalidateJComponent(this.handShapeScrollPane);
        this.revalidateJComponent(this.handShapePanel);
    }

    private JLabel createImageLabel(FieldValueWrapper value, boolean boxLayout) {
        JLabel imageLabel = this.getImage(value.getSS3FieldValue());
        imageLabel.setBackground(value.getBackground());
        imageLabel.setPreferredSize(new Dimension(120, imageLabel.getPreferredSize().height));

        if (boxLayout) {
            imageLabel.setMaximumSize(new Dimension(Integer.MAX_VALUE, imageLabel.getPreferredSize().height));
        }

        this.imageLabels.add(imageLabel);

        return imageLabel;
    }

    private void setupConstraints(int totalIterations, int totalItems) {
        this.itemConstraints.fill = GridBagConstraints.HORIZONTAL;
        this.itemConstraints.anchor = GridBagConstraints.NORTH;
        
        int newGridx = totalIterations / ((int) Math.ceil(totalItems / 3.0));
        int newGridy = totalIterations % ((int) Math.ceil(totalItems / 3.0));
        this.itemConstraints.gridx = newGridx;
        this.itemConstraints.gridy = newGridy;
        
        this.itemConstraints.weightx = 1.0;
        this.itemConstraints.weighty = 0.0;
    }
    
    // making this function to 'dry run' through the display, to get the total count of visible things with everything expanded as it should be
    // need the total count for the display calculations to be correct
    private int getCountVisibleItems() {
        this.handShapePanel.removeAll();
        int totalIterations = 0;
        int totalVisibleItems = this.folders.size();

        this.triangleLabels = new ArrayList<>();

        Comparator<CSPaletteItemGroup> itemGroupComparator = Comparator.comparingInt(CSPaletteItemGroup::getOrder);
        Map<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> sortedFolders = new TreeMap<>(itemGroupComparator);
        sortedFolders.putAll(this.folders);

        for (Map.Entry<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> parent : sortedFolders.entrySet()) {
            ArrayList<FieldValueWrapper> values = parent.getValue();
            CSPaletteItemGroup parentItemGroup = parent.getKey();
            boolean incrementedTotalVisibleItemsFlag = false;

            for (FieldValueWrapper value : values) {

                if (value.isExpanded()) {
                    if (value.isFolder()) {
                        continue;
                    }

                    if (!incrementedTotalVisibleItemsFlag) {
                        totalVisibleItems += values.size() - 1;
                        incrementedTotalVisibleItemsFlag = true;
                    }
                }
            }
        }
        
        return totalVisibleItems;
    }

    private void populateWithFolders() {
        this.handShapePanel.removeAll();
        int totalIterations = 0;
//        int totalVisibleItems = this.folders.size();	// use below function instead
        int totalVisibleItems = getCountVisibleItems();

        this.triangleLabels = new ArrayList<>();

        Comparator<CSPaletteItemGroup> itemGroupComparator = Comparator.comparingInt(CSPaletteItemGroup::getOrder);
        Map<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> sortedFolders = new TreeMap<>(itemGroupComparator);
        sortedFolders.putAll(this.folders);

        for (Map.Entry<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> parent : sortedFolders.entrySet()) {
            ArrayList<FieldValueWrapper> values = parent.getValue();
            CSPaletteItemGroup parentItemGroup = parent.getKey();
            boolean incrementedTotalVisibleItemsFlag = false;

            for (FieldValueWrapper value : values) {
                JPanel parentLabelPanel = new JPanel();
                parentLabelPanel.setLayout(new BoxLayout(parentLabelPanel, BoxLayout.X_AXIS));
                parentLabelPanel.setBackground(UIPalette.getBackground());
                parentLabelPanel.setOpaque(true);

                if (value.isFolder() && parentItemGroup.getItems().size() > 1) { // This means there is more than one item in the folder
                    TriangleLabel parentLabel = new TriangleLabel(value, values);

                    this.triangleLabels.add(parentLabel);

                    Dimension preferredSize = parentLabel.getPreferredSize();
                    parentLabel.setMaximumSize(preferredSize);
                    parentLabelPanel.add(parentLabel);

                    parentLabel.addMouseListener(this);

                    JLabel imageLabel = this.createImageLabel(value, true);
                    Color SS3Blue = new Color(123, 166, 172);
                    Color white = Color.WHITE;
                    imageLabel.setBackground(totalIterations % 2 == 0 ? white : SS3Blue);
                    parentLabelPanel.add(imageLabel);

                    this.setupConstraints(totalIterations, totalVisibleItems);

                    this.handShapePanel.add(parentLabelPanel, this.itemConstraints);

                    totalIterations++;
                } else if (value.isFolder() && parentItemGroup.getItems().size() == 1) { // This means there is only one item in the folder
                    JLabel imageLabel = this.createImageLabel(value, true);
                    Color SS3Blue = new Color(123, 166, 172);
                    Color white = Color.WHITE;
                    imageLabel.setBackground(totalIterations % 2 == 0 ? white : SS3Blue);

                    Dimension triangleLabelSize = new TriangleLabel(value, values).getPreferredSize();

                    // Create an empty placeholder label and add it to the panel
                    JLabel placeholderLabel = new JLabel();
                    placeholderLabel.setPreferredSize(triangleLabelSize);
                    parentLabelPanel.add(placeholderLabel);
                    parentLabelPanel.add(imageLabel);

                    // Setup and apply the constraints
                    this.setupConstraints(totalIterations, totalVisibleItems);

                    this.handShapePanel.add(parentLabelPanel, this.itemConstraints);
                    totalIterations++;
                }

                if (value.isExpanded()) {
                    if (value.isFolder()) {
                        continue;
                    }

                    if (!incrementedTotalVisibleItemsFlag) {
//                        totalVisibleItems += values.size() - 1;	// see function called at top of this one to get total count beforehand
                        incrementedTotalVisibleItemsFlag = true;
                    }

                    JLabel imageLabel = this.createImageLabel(value, true);
                    Color SS3Blue = new Color(123, 166, 172);
                    Color white = Color.WHITE;
                    imageLabel.setBackground(totalIterations % 2 == 0 ? white : SS3Blue);
                    JLabel placeHolderLabel = new JLabel();
                    Dimension triangleLabelSize = new TriangleLabel(value, values).getPreferredSize();
                    placeHolderLabel.setPreferredSize(triangleLabelSize);
                    parentLabelPanel.add(placeHolderLabel);
                    parentLabelPanel.add(imageLabel);

                    this.setupConstraints(totalIterations, totalVisibleItems);
                    this.handShapePanel.add(parentLabelPanel, this.itemConstraints);
                    totalIterations++;
                }
            }
        }

        GridBagConstraints dummyConstraints = new GridBagConstraints();
        dummyConstraints.weighty = 1.0;  // takes up the remaining vertical space
        dummyConstraints.gridx = 0;
        dummyConstraints.gridy = GridBagConstraints.RELATIVE;
        dummyConstraints.gridwidth = GridBagConstraints.REMAINDER;  // span across all grid columns
        dummyConstraints.fill = GridBagConstraints.BOTH;

        JPanel dummyPanel = new JPanel();
        dummyPanel.setOpaque(false);  // make it transparent
        this.handShapePanel.add(dummyPanel, dummyConstraints);

        this.handShapeScrollPane.setMinimumSize(this.handShapePanel.getPreferredSize());
        this.revalidateJComponent(this.handShapeScrollPane);
        this.revalidateJComponent(this.handShapePanel);
    }

    private JPanel initializeSearchPanel() {
        JPanel searchPanel = new JPanel(new GridBagLayout());
        GridBagConstraints constraints = new GridBagConstraints();
//        JTextField searchField = new JTextField();
//        searchField.setEnabled(false);
        // Show message when hovering over the search field
//        searchField.setText("This feature is currently unavailable.");
//        searchField.setToolTipText("This feature will be released in the next version of SignStream!");
//        JButton searchButton = new JButton("Search");
//        searchButton.setEnabled(false);
//        searchButton.setToolTipText("This feature will be released in the next version of SignStream!");
        JButton expandAllTriangles = new JButton("Expand All Triangles");

        expandAllTriangles.addActionListener(e -> {
            for (TriangleLabel triangleLabel : this.triangleLabels) {
                triangleLabel.setExpanded(true);
            }

            this.updateHandShapePanel();
            this.revalidateJComponent(this.handShapePanel);
        });

        JButton collapseAllTriangles = new JButton("Collapse All Triangles");

        collapseAllTriangles.addActionListener(e -> {
            for (TriangleLabel triangleLabel : this.triangleLabels) {
                triangleLabel.setExpanded(false);
            }

            this.updateHandShapePanel();
            this.revalidateJComponent(this.handShapePanel);
        });

        // Create a new panel to hold the searchField and searchButton
        JPanel searchPanelTop = new JPanel();
        searchPanelTop.setLayout(new BoxLayout(searchPanelTop, BoxLayout.X_AXIS));
//        searchPanelTop.add(searchField);
//        searchPanelTop.add(searchButton);

        // Set up the search field and button
//        searchField.setPreferredSize(new Dimension(100, 25));
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.weightx = 2.0;
        constraints.weighty = 0;
        constraints.gridwidth = 2;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        searchPanel.add(searchPanelTop, constraints);

        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.weightx = 1.0;
        searchPanel.add(expandAllTriangles, constraints);

        constraints.gridx = 1;
        constraints.gridy = 1;
        searchPanel.add(collapseAllTriangles, constraints);

        return searchPanel;
    }


    private void updateHandShapePanel() {
        this.handShapePanel.removeAll();

        if (this.folders.size() == 1) { // This means there is only one item group, so no folders
            this.populateWithoutFolders();
            this.centerPanelConstraints.gridx = 0;
            this.centerPanelConstraints.gridy = 0;
            this.centerPanelConstraints.gridheight = 2;
        } else if (this.folders.size() > 1) {
            this.searchPanel = this.initializeSearchPanel();

            // Setting up the search panel to be at the top
            this.centerPanelConstraints.gridx = 0;
            this.centerPanelConstraints.gridy = 0;
            this.centerPanelConstraints.weightx = 1.0;
            this.centerPanelConstraints.weighty = 0.0;
            this.centerPanelConstraints.anchor = GridBagConstraints.NORTH; // Anchor to the top

            super.add(this.searchPanel, this.centerPanelConstraints);

            // Setting up the handshape panel to be just below the search panel
            this.centerPanelConstraints.gridx = 0;
            this.centerPanelConstraints.gridy = 1;
            this.centerPanelConstraints.weightx = 1.0;
            this.centerPanelConstraints.weighty = 1.0;
            this.centerPanelConstraints.fill = GridBagConstraints.BOTH;
            this.centerPanelConstraints.anchor = GridBagConstraints.NORTH; // Anchor to the top
            this.centerPanelConstraints.gridheight = 2;

            this.populateWithFolders();
        }
    }

    private void populateHandshapeImages() {
        this.handShapePanel.removeAll();
        this.centerPanelConstraints = new GridBagConstraints();

        this.itemConstraints = new GridBagConstraints();
        this.handShapePanel.setBackground(UIPalette.getBackground());
        this.itemConstraints.insets = new Insets(0, 0, 0, 0);

        this.centerPanelConstraints.fill = GridBagConstraints.BOTH;
        this.centerPanelConstraints.weightx = 1.0;
        this.centerPanelConstraints.weighty = 1.0;

        this.centerPanelConstraints.anchor = GridBagConstraints.NORTH;


        this.updateHandShapePanel();

        super.add(this.handShapeScrollPane, this.centerPanelConstraints);
    }


    private HashMap<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> populatePanelValues() {
        Color SS3Blue = new Color(123, 166, 172);
        Color white = Color.WHITE;

        CSHandShapePalette handShapePalettes = this.scheme.getHandShapeTemplates().getHandShapeTemplate(this.defaultTemplateID).getPalettesAsHashMap().get(this.currentPaletteID);
        HashMap<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> folders = new HashMap<>();

        int totalIterations = 0; // This is used to calculate the color since it should be alternating

        for (CSPaletteItemGroup group : handShapePalettes.getGroups()) {
            ArrayList<CSHandShapeItem> items = group.getItems();
            items.sort(Comparator.comparingInt(CSHandShapeItem::getOrder));
            folders.put(group, new ArrayList<>());

            for (CSHandShapeItem item : items) {
                Color itemColor = totalIterations % 2 == 0 ? white : SS3Blue;
                boolean isFolder = handShapePalettes.getGroups().size() > 1 && item.getOrder() == 1;
                FieldValueWrapper value = super.createFieldValueWrapper(item.getId(), item.getImageID(), item.getDescription(), false, isFolder, itemColor);

                folders.get(group).add(value);

                totalIterations++;
            }
        }

        this.itemsInAllFolders = totalIterations;

        return folders;
    }

    // Getters
    public ArrayList<SS3FieldValue> getValues() {
        return this.values;
    }

    // Setters
    public void setValues(ArrayList<SS3FieldValue> values) {
        this.values = values;
    }

    public ArrayList<HandShapeImage> getSelectedHshImages() {
        return this.selectedHshImages;
    }

    public void setSelectedHshImages(ArrayList<HandShapeImage> selectedHshImages) {
        this.selectedHshImages = selectedHshImages;
    }

    public ArrayList<FieldValueWrapper> getPanelValues() {
        return this.panelValues;
    }

    public void setPanelValues(ArrayList<FieldValueWrapper> panelValues) {
        this.panelValues = panelValues;
    }

    public SS3CodingScheme getScheme() {
        return this.scheme;
    }

    public void setScheme(SS3CodingScheme scheme) {
        this.scheme = scheme;
    }

    public HashMap<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> getFolders() {
        return this.folders;
    }

    public ArrayList<JLabel> getImageLabels() {
        return this.imageLabels;
    }

    public String getDefaultTemplateID() {
        return this.defaultTemplateID;
    }

    public void setDefaultTemplateID(String defaultTemplateID) {
        this.defaultTemplateID = defaultTemplateID;
    }

    public String getCurrentPaletteID() {
        return this.currentPaletteID;
    }

    public JPanel getHandShapePanel() {
        return this.handShapePanel;
    }

    public void setHandShapePanel(JPanel handShapePanel) {
        this.handShapePanel = handShapePanel;
    }

    protected JScrollPane getHandShapeScrollPane() {
        return this.handShapeScrollPane;
    }

    public void setHandShapeScrollPane(JScrollPane handShapeScrollPane) {
        this.handShapeScrollPane = handShapeScrollPane;
    }

    public GridBagConstraints getItemConstraints() {
        return this.itemConstraints;
    }

    public void setItemConstraints(GridBagConstraints itemConstraints) {
        this.itemConstraints = itemConstraints;
    }

    public JButton getClearButton() {
        return this.clearButton;
    }

    public void setClearButton(JButton clearButton) {
        this.clearButton = clearButton;
    }

    public ParentPaletteInformationHelper getParentHandShapePanel() {
        return this.parentHandShapePanel;
    }

    public void setParentHandShapePanel(ParentPaletteInformationHelper<K> parentHandShapePanel) {
        this.parentHandShapePanel = parentHandShapePanel;
    }

    public T getSelectionPanel() {
        return this.selectedItemsPanel;
    }

    public void setSelectedItemsPanel(T selectedItemsPanel) {
        this.selectedItemsPanel = selectedItemsPanel;
    }

    public HashMap<String, SS3FieldValue> getSelectedHandshapesMap() {
        return this.selectedHandshapesMap;
    }

    public void setSelectedHandshapesMap(HashMap<String, SS3FieldValue> selectedHandshapesMap) {
        this.selectedHandshapesMap = selectedHandshapesMap;
    }

    public GridBagConstraints getCenterPanelConstraints() {
        return this.centerPanelConstraints;
    }

    /**
     * Invoked when the mouse button has been clicked (pressed
     * and released) on a component.
     *
     * @param e the event to be processed
     */
    @Override
    public void mouseClicked(MouseEvent e) {
    }

    /**
     * Invoked when a mouse button has been pressed on a component.
     *
     * @param e the event to be processed
     */
    @Override
    public void mousePressed(MouseEvent e) {
        if (e.getSource() instanceof TriangleLabel) {
            TriangleLabel label = (TriangleLabel) e.getSource();
            label.setExpanded(!label.isExpanded());
            this.updateHandShapePanel();
            this.revalidateJComponent(this.handShapePanel);
            this.revalidateJComponent(label);
        }
    }

    /**
     * Invoked when a mouse button has been released on a component.
     *
     * @param e the event to be processed
     */
    @Override
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Invoked when the mouse enters a component.
     *
     * @param e the event to be processed
     */
    @Override
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Invoked when the mouse exits a component.
     *
     * @param e the event to be processed
     */
    @Override
    public void mouseExited(MouseEvent e) {
    }
}
