package edu.bu.signstream.grepresentation.fields.palettes;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteMenu;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteReference;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.PaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Objects;

public class GestureOrientationPalettePanel<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends JPanel {
    private final HandShapesPaletteFormBean formBean;
    private final SS3CodingScheme scheme;
    private final ParentPaletteInformationHelper<K> parentPaletteInformationPanel;
    private final T selectionPanel;
    private final PaletteInformationHelper paletteInformationHelper;
    private ArrayList<PaletteReference> fingerTipPaletteReferences;
    private ArrayList<PaletteReference> palmPaletteReferences;

    private String fingerTipPaletteID;
    private String palmPaletteID;
    private final String defaultMenuID;

    private final String fingerTipPaletteGroupID;
    private final String palmPaletteGroupID;

    private final JPanel fingerTipOrientationComboBoxPanel;
    private final JPanel palmOrientationComboBoxPanel;
    private GeneralHandShapePalettePanel<T, K> fingerTipOrientationPanel;
    private GeneralHandShapePalettePanel<T, K> palmOrientationPanel;
    private JPanel orientationContainerPanel;


    public GestureOrientationPalettePanel(HandShapesPaletteFormBean formBean, SS3CodingScheme scheme, ParentPaletteInformationHelper<K> parentPaletteInformationPanel,
                                          T selectionPanel, PaletteInformationHelper paletteInformationHelper, String defaultMenuID,
                                          String fingerTipPaletteGroupID, String palmPaletteGroupID) {
        super(new GridBagLayout());

        this.formBean = formBean;
        this.scheme = scheme;
        this.parentPaletteInformationPanel = parentPaletteInformationPanel;
        this.selectionPanel = selectionPanel;
        this.paletteInformationHelper = paletteInformationHelper;

        this.defaultMenuID = defaultMenuID;

        this.fingerTipPaletteGroupID = fingerTipPaletteGroupID;
        this.palmPaletteGroupID = palmPaletteGroupID;

        JPanel containerPanel = new JPanel();
        containerPanel.setLayout(new BoxLayout(containerPanel, BoxLayout.X_AXIS));

        this.fingerTipOrientationComboBoxPanel = new JPanel();
        this.palmOrientationComboBoxPanel = new JPanel();

        JComboBox<String> fingerTipPaletteCombobox = new JComboBox<>();
        this.fingerTipOrientationComboBoxPanel.add(fingerTipPaletteCombobox);

        JComboBox<String> palmPaletteCombobox = new JComboBox<>();
        this.palmOrientationComboBoxPanel.add(palmPaletteCombobox);

        // TODO: Add on change listeners to the combo boxes to update the palette panels

        boolean comboBoxesSuccess = this.initOrientationComboBoxes(fingerTipPaletteCombobox, palmPaletteCombobox);

        if (!comboBoxesSuccess) {
            return;
        }

        this.fingerTipPaletteID = this.getDefaultPaletteID(this.fingerTipPaletteReferences);
        this.palmPaletteID = this.getDefaultPaletteID(this.palmPaletteReferences);

        this.fingerTipOrientationPanel = new GestureOrientationImagesPanel<>(this.formBean, this.scheme, this.parentPaletteInformationPanel, this.selectionPanel,
                this.paletteInformationHelper.getCurrentTemplateID(), this.fingerTipPaletteID);
        this.palmOrientationPanel = new GestureOrientationImagesPanel<>(this.formBean, this.scheme, this.parentPaletteInformationPanel, this.selectionPanel,
                this.paletteInformationHelper.getCurrentTemplateID(), this.palmPaletteID);

        String defaultFingerTipTitle = this.getDefaultStringPaletteTitle(this.fingerTipPaletteReferences);
        String defaultPalmTitle = this.getDefaultStringPaletteTitle(this.palmPaletteReferences);

        JScrollPane fingerOrientationScrollPane = new JScrollPane(this.fingerTipOrientationPanel.getHandShapePanel(), ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        JScrollPane palmOrientationScrollPane = new JScrollPane(this.palmOrientationPanel.getHandShapePanel(), ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        Border fingerTipOrientationEtched = BorderFactory.createEtchedBorder();
        Border fingerTipOrientationTitle = BorderFactory.createTitledBorder(fingerTipOrientationEtched, defaultFingerTipTitle);
        fingerOrientationScrollPane.setBorder(fingerTipOrientationTitle);

        Border palmOrientationEtched = BorderFactory.createEtchedBorder();
        Border palmOrientationTitle = BorderFactory.createTitledBorder(palmOrientationEtched, defaultPalmTitle);
        palmOrientationScrollPane.setBorder(palmOrientationTitle);

        this.orientationContainerPanel = new JPanel();
        this.orientationContainerPanel.setLayout(new GridBagLayout());
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weighty = 0;  // No extra vertical space for the combo box
        gridBagConstraints.weightx = 1;  // Take all extra horizontal space for the combo box
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;  // Resize only horizontally
        this.orientationContainerPanel.add(fingerTipPaletteCombobox, gridBagConstraints);

// For the fingerOrientationScrollPane:
        gridBagConstraints.gridy = GridBagConstraints.RELATIVE;
        gridBagConstraints.weighty = 0.5;  // Take all extra vertical space for the panel
        gridBagConstraints.fill = GridBagConstraints.BOTH;  // Resize in both directions
        this.orientationContainerPanel.add(fingerOrientationScrollPane, gridBagConstraints);

// For the palmOrientationComboBoxPanel:
        gridBagConstraints.gridy = GridBagConstraints.RELATIVE;
        gridBagConstraints.weighty = 0;  // No extra vertical space for the combo box
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;  // Resize only horizontally
        this.orientationContainerPanel.add(palmPaletteCombobox, gridBagConstraints);

// For the palmOrientationScrollPane:
        gridBagConstraints.gridy = GridBagConstraints.RELATIVE;
        gridBagConstraints.weighty = 0.5;  // Take all extra vertical space for the panel
        gridBagConstraints.fill = GridBagConstraints.BOTH;  // Resize in both directions
        this.orientationContainerPanel.add(palmOrientationScrollPane, gridBagConstraints);

        gridBagConstraints.gridy = GridBagConstraints.RELATIVE;
        gridBagConstraints.gridx = 0;
        super.add(this.orientationContainerPanel, gridBagConstraints);
    }

    private boolean initOrientationComboBoxes(JComboBox<String> fingerTipPaletteCombobox, JComboBox<String> palmPaletteCombobox) {
        PaletteMenu paletteMenu = this.getPaletteMenu();
        if (paletteMenu == null) {
            return false;
        }

        PaletteGroup palmPaletteGroup = this.getPaletteGroup(paletteMenu, this.palmPaletteGroupID);
        PaletteGroup fingerTipPaletteGroup = this.getPaletteGroup(paletteMenu, this.fingerTipPaletteGroupID);

        if (palmPaletteGroup == null || fingerTipPaletteGroup == null) {
            return false;
        }

        this.palmPaletteReferences = palmPaletteGroup.getPaletteReferences();
        this.fingerTipPaletteReferences = fingerTipPaletteGroup.getPaletteReferences();


        if (this.fingerTipPaletteReferences.isEmpty() || this.palmPaletteReferences.isEmpty()) {
            return false;
        }

        for (PaletteReference paletteReference : this.fingerTipPaletteReferences) {
            fingerTipPaletteCombobox.addItem(Objects.requireNonNull(this.getHandShapePaletteFromID(paletteReference.getPaletteId())).getName());
        }

        for (PaletteReference paletteReference : this.palmPaletteReferences) {
            palmPaletteCombobox.addItem(Objects.requireNonNull(this.getHandShapePaletteFromID(paletteReference.getPaletteId())).getName());
        }

        return true;
    }

    private PaletteMenu getPaletteMenu() {
        for (PaletteMenu paletteMenu : this.scheme.getPaletteMenus()) {
            if (paletteMenu.getId().equals(this.defaultMenuID)) {
                return paletteMenu;
            }
        }

        return null;
    }

    private String getDefaultPaletteID(ArrayList<PaletteReference> references) {
        for (PaletteReference reference : references) {
            if (this.parentPaletteInformationPanel.isDomHandSelected() && reference.getIsDomDefault()) {
                return reference.getPaletteId();
            } else if (this.parentPaletteInformationPanel.isNondomHandSelected() && reference.getIsNDomDefault()) {
                return reference.getPaletteId();
            }
        }

        return null;
    }

    private String getDefaultStringPaletteTitle(ArrayList<PaletteReference> references) {
        for (PaletteReference reference : references) {
            if (this.parentPaletteInformationPanel.isDomHandSelected() && reference.getIsDomDefault()) {
                return Objects.requireNonNull(this.getHandShapePaletteFromID(reference.getPaletteId())).getName();
            } else if (this.parentPaletteInformationPanel.isNondomHandSelected() && reference.getIsNDomDefault()) {
                return Objects.requireNonNull(this.getHandShapePaletteFromID(reference.getPaletteId())).getName();
            }
        }

        return null;
    }

    private PaletteGroup getPaletteGroup(PaletteMenu paletteMenu, String paletteGroupID) {
        for (PaletteGroup paletteGroup : paletteMenu.getPaletteGroups()) {
            if (paletteGroup.getId().equals(paletteGroupID)) {
                return paletteGroup;
            }
        }

        return null;
    }

    private CSHandShapePalette getHandShapePaletteFromID(String id) {
        for (CSHandShapePalette csHandShapePalette : this.scheme.getHandShapeTemplates().
                getHandShapeTemplate(this.paletteInformationHelper.getCurrentTemplateID()).getPalettes()) {
            if (id.equals(csHandShapePalette.getId())) {
                return csHandShapePalette;
            }
        }

        return null;
    }

    public HandShapesPaletteFormBean getFormBean() {
        return this.formBean;
    }

    public SS3CodingScheme getScheme() {
        return this.scheme;
    }

    public ParentPaletteInformationHelper<K> getParentPaletteInformationPanel() {
        return this.parentPaletteInformationPanel;
    }

    public T getSelectionPanel() {
        return this.selectionPanel;
    }

    public PaletteInformationHelper getPaletteInformationHelper() {
        return this.paletteInformationHelper;
    }

    public ArrayList<PaletteReference> getFingerTipPaletteReferences() {
        return this.fingerTipPaletteReferences;
    }

    public ArrayList<PaletteReference> getPalmPaletteReferences() {
        return this.palmPaletteReferences;
    }

    public String getFingerTipPaletteID() {
        return this.fingerTipPaletteID;
    }

    public String getPalmPaletteID() {
        return this.palmPaletteID;
    }

    public String getDefaultMenuID() {
        return this.defaultMenuID;
    }

    public String getFingerTipPaletteGroupID() {
        return this.fingerTipPaletteGroupID;
    }

    public String getPalmPaletteGroupID() {
        return this.palmPaletteGroupID;
    }

    public JPanel getFingerTipOrientationComboBoxPanel() {
        return this.fingerTipOrientationComboBoxPanel;
    }

    public JPanel getPalmOrientationComboBoxPanel() {
        return this.palmOrientationComboBoxPanel;
    }

    public GeneralHandShapePalettePanel<T, K> getFingerTipOrientationPanel() {
        return this.fingerTipOrientationPanel;
    }

    public GeneralHandShapePalettePanel<T, K> getPalmOrientationPanel() {
        return this.palmOrientationPanel;
    }

    public JPanel getOrientationContainerPanel() {
        return this.orientationContainerPanel;
    }

}
