package edu.bu.signstream.grepresentation.fields.palettes;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSPaletteItemGroup;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GenericStartEndHandShapeTypes;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.FieldValueWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;

import javax.swing.*;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;

public class LettersPalette <T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends GeneralHandShapePalettePanel<T, K> {
    private JTextField lettersTextField;
    private HashMap<CSPaletteItemGroup, ArrayList<FieldValueWrapper>> folders;
    private HashMap<Character, FieldValueWrapper> letterFieldValueMap;

    public LettersPalette(HandShapesPaletteFormBean formBean, SS3CodingScheme scheme, ParentPaletteInformationHelper<K> parentPanel, T selectionPanel, String templateID, String currentPaletteID) {
        super(formBean, scheme, parentPanel, selectionPanel, templateID, currentPaletteID);
        this.folders = super.getFolders();
        this.letterFieldValueMap = this.convertArrayListToHashMap();
        this.addLettersTextField();
        this.synchronizeHandshapeWithTextField(super.getSelectedHshImages());
    }

    private HashMap<Character, FieldValueWrapper> convertArrayListToHashMap() {
        HashMap<Character, FieldValueWrapper> map = new HashMap<>();
        for (CSPaletteItemGroup group : this.folders.keySet()) {
            for (FieldValueWrapper wrapper : this.folders.get(group)) {
                String letter = wrapper.getSS3FieldValue().getLetter();

                // Check if letter has a slash in it
                if (letter.contains("/")) {
                    String[] letters = letter.split("/");
                    for (String l : letters) {
                        map.put(l.strip().charAt(0), wrapper);
                    }
                } else {
                    map.put(letter.strip().charAt(0), wrapper);
                }
            }
        }

        return map;
    }

    private SS3FieldValue getSS3ValueFromText(HashMap<Character, FieldValueWrapper> map, Character letter) {
        if (map.containsKey(letter)) {
            return map.get(letter).getSS3FieldValue();
        }

        return null;
    }


    public void synchronizeHandshapeWithTextField(ArrayList<HandShapeImage> handshapes) {
        StringBuilder sb = new StringBuilder();

        ((AbstractDocument) this.lettersTextField.getDocument()).setDocumentFilter(null);

        for(HandShapeImage handshape : handshapes) {
            SS3FieldValue value = handshape.getSS3FieldValue();

            if (value.getLabel().contains("/")) {
                sb.append(String.format("<%s>", value.getLabel()));
            } else {
                sb.append(value.getLabel());
            }
        }

        this.lettersTextField.setText(sb.toString());

        ((AbstractDocument) this.lettersTextField.getDocument()).setDocumentFilter(new NonNumericFilter());
    }

    private void addLettersTextField() {
        this.lettersTextField = new JTextField();
        this.lettersTextField.setPreferredSize(new Dimension(super.getPreferredSize().width, 30));
        this.lettersTextField.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        this.lettersTextField.setFont(new Font("Times New Roman", Font.PLAIN, 20));
        this.lettersTextField.addKeyListener(new LetterTypedListener<>(this.letterFieldValueMap, super.getSelectionPanel(), super.getSelectedHshImages()));
/*        this.lettersTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                if (lettersTextField.getText().equals("Enter letters here")) {
                    lettersTextField.setText("");
                    lettersTextField.setForeground(Color.BLACK);
                }
            }

            public void focusLost(java.awt.event.FocusEvent evt) {
                if (lettersTextField.getText().isEmpty()) {
                    lettersTextField.setForeground(Color.GRAY);
                    lettersTextField.setText("Enter letters here");
                }
            }
        });*/

        ((AbstractDocument) this.lettersTextField.getDocument()).setDocumentFilter(new NonNumericFilter());

        GridBagConstraints constraints = super.getCenterPanelConstraints();
        constraints.gridx = 0;
        constraints.gridy = GridBagConstraints.RELATIVE;
        constraints.weightx = 1.0;
        constraints.weighty = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;

        super.add(this.lettersTextField, constraints);
        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();

    }

    @Override
    public JLabel getImage(SS3FieldValue value) {
        HandShapeImage handShapeImage = new HandShapeImage(value, super.getScheme());

        Image image = handShapeImage.getImage();
        JLabel imageLabel = new JLabel();
        Font imageLabelFont = new Font(imageLabel.getFont().getName(), Font.PLAIN, 10);
        imageLabel.setFont(imageLabelFont);
        imageLabel.setBorder(BorderFactory.createLineBorder(UIPalette.getBackground()));
        imageLabel.setOpaque(true);
        imageLabel.setIcon(new ImageIcon(image));
        imageLabel.setText(value.getLabel());

        imageLabel.addMouseListener(new MouseListener() { // TODO: Since we need to override the mousePressed method, we need to implement the MouseListener interface
            @Override
            public void mouseClicked(MouseEvent e) {
            }

            /**
             * Invoked when a mouse button has been pressed on a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mousePressed(MouseEvent e) {
                LettersPalette.super.getSelectedHshImages().add(handShapeImage);
                LettersPalette.super.getSelectionPanel().setItem(value);

                ((AbstractDocument) LettersPalette.this.lettersTextField.getDocument()).setDocumentFilter(null);

                if (value.getLabel().contains("/")) {
                    LettersPalette.this.lettersTextField.setText(LettersPalette.this.lettersTextField.getText() + String.format("<%s>", value.getLabel()));
                } else {
                    LettersPalette.this.lettersTextField.setText(LettersPalette.this.lettersTextField.getText() + value.getLabel());
                }

                ((AbstractDocument) LettersPalette.this.lettersTextField.getDocument()).setDocumentFilter(new NonNumericFilter());
            }

            /**
             * Invoked when a mouse button has been released on a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseReleased(MouseEvent e) {
            }

            /**
             * Invoked when the mouse enters a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseEntered(MouseEvent e) {
            }

            /**
             * Invoked when the mouse exits a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseExited(MouseEvent e) {
            }
        });

        return imageLabel;
    }

    static class LetterTypedListener<T extends SelectedPanel> extends KeyAdapter {
        private boolean deleteExtraChar = false;
        private final T selectionPanel;
        private final ArrayList<HandShapeImage> handShapeImages;
        private final HashMap<Character, FieldValueWrapper> letters;

        public LetterTypedListener(HashMap<Character, FieldValueWrapper> letters, T selectionPanel, ArrayList<HandShapeImage> handShapeImages) {
            this.letters = letters;
            this.selectionPanel = selectionPanel;
            this.handShapeImages = handShapeImages;
        }

        @Override
        public void keyPressed(KeyEvent e) {
            JTextField textField = (JTextField) e.getSource();

            if (Character.isLetter(e.getKeyChar())) {
                // Convert the input to upper case
                char letter = Character.toUpperCase(e.getKeyChar());
                SS3FieldValue fieldValue = this.letters.get(letter).getSS3FieldValue();

                if (fieldValue == null) {
                    return;
                }

                int caretPosition = textField.getCaretPosition();
                StringBuilder currentText = new StringBuilder(textField.getText().substring(0, caretPosition).toUpperCase(Locale.ROOT));

                if (fieldValue.getLabel().contains("/")) {
                    // Remove the filter from the text field
                    ((AbstractDocument) textField.getDocument()).setDocumentFilter(null);

                    currentText.append(String.format("<%s>", fieldValue.getLabel()));
                    textField.setText(currentText.toString());
                    // Add the filter back to the text field
                    ((AbstractDocument) textField.getDocument()).setDocumentFilter(new NonNumericFilter());
                    this.deleteExtraChar = true;
                } else {
                    this.deleteExtraChar = false;
                }
                e.consume();
            } else if (e.getKeyCode() == KeyEvent.VK_LEFT || e.getKeyCode() == KeyEvent.VK_RIGHT) {
                e.consume();  // ignore the event
            } else if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
                try {
                    int caretPosition = textField.getCaretPosition();
                    String textFieldText = textField.getText().toUpperCase(Locale.ROOT);
                    char characterToBeRemoved = textFieldText.charAt(caretPosition - 1);

                    if (characterToBeRemoved == '>') {
                        // Remove the '>' character and the four characters before it too
                        String newText = textFieldText.substring(0, caretPosition - 5) + textFieldText.substring(caretPosition);
                        // Remove document filter
                        ((AbstractDocument) textField.getDocument()).setDocumentFilter(null);
                        textField.setText(newText);
                        ((AbstractDocument) textField.getDocument()).setDocumentFilter(new NonNumericFilter());
                        e.consume();
                    }
                } catch (StringIndexOutOfBoundsException ex) {
                    // Do nothing
                }
                // Else if the key pressed is a letter, then we do nothing
            }
        }

        @Override
        public void keyTyped(KeyEvent e) {
            if (this.deleteExtraChar) {
                e.setKeyChar('\0');
            }
        }


        @Override
        public void keyReleased(KeyEvent e) {
            JTextField tf = (JTextField) e.getSource();

            this.handShapeImages.clear();
            this.selectionPanel.getHandshapes().clear();
            this.selectionPanel.synchronizeWithFormBean();
            this.selectionPanel.revalidate();
            this.selectionPanel.repaint();

            char[] c = tf.getText().toUpperCase().toCharArray();
            for (int i = 0; i < c.length; i++) {
                char cc = c[i];
                if (Character.isLetter(cc)) {
                    String s = String.valueOf(cc);
                    SS3FieldValue obj = this.letters.get(s.charAt(0)).getSS3FieldValue();
                    if (obj != null) {
                        this.selectionPanel.setItem(obj);
                    }
                } else if (c[i] == '<') {
                    SS3FieldValue obj;
                    try {
                        obj = this.letters.get(c[i + 1]).getSS3FieldValue();
                    } catch (ArrayIndexOutOfBoundsException e1) {
                        continue;
                    }

                    if (obj != null) {
                        this.selectionPanel.setItem(obj);
                    }

                    i += 4;
                }
            }
        }
    }

    static class NonNumericFilter extends DocumentFilter {

        @Override
        public void insertString(DocumentFilter.FilterBypass fb, int offset,
                                 String text, javax.swing.text.AttributeSet attr)
                throws BadLocationException {

            fb.insertString(offset, text.toUpperCase(Locale.ROOT).replaceAll("\\d", ""), attr); // remove numbers
        }

        @Override
        public void replace(DocumentFilter.FilterBypass fb, int offset, int length,
                            String text, javax.swing.text.AttributeSet attr)
                throws BadLocationException {

            if (text.matches("^[a-zA-Z]*$")) {
                fb.replace(offset, length, text.toUpperCase(Locale.ROOT).replaceAll("\\d", ""), attr); // remove numbers
            }
        }
    }
}
