package edu.bu.signstream.grepresentation.fields.palettes.doublePalettes;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeTemplate;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GenericStartEndHandShapeTypes;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.PaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.PaletteUIFactory;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;

import javax.swing.*;
import java.awt.*;
import java.util.Objects;

public class GeneralDoublePaletteDialog<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends JDialog {
    private final HandShapesPaletteFormBean rightHandFormBean;
    private final HandShapesPaletteFormBean leftHandFormBean;
    private final HandShapeFormBean bean;
    private final ParentPaletteInformationHelper<K> parentPaletteInformationPanel;
    private final SS3CodingScheme scheme;
    private final PaletteInformationHelper paletteInformationHelper;
    private T rightHandSelectionPanel;
    private T leftHandSelectionPanel;
    private JPanel rightHandSelectionClearPanel;
    private JPanel leftHandSelectionClearPanel;
    private JComboBox<String> rightHandComboBox;
    private JComboBox<String> leftHandComboBox;
    private final String defaultPaletteMenuID;
    private final String selectedPaletteID;

    private final JPanel bottomPanel;

    private final JButton enterButton;
    private final JButton cancelButton;

    public GeneralDoublePaletteDialog(HandShapesPaletteFormBean rightHandFormBean, HandShapesPaletteFormBean leftHandFormBean,
                                      HandShapeFormBean bean, ParentPaletteInformationHelper<K> parentInformationPanel, T rightHandSelectionPanel, T leftHandSelectionPanel,
                                      SS3CodingScheme scheme, JDialog parentDialog, String defaultPaletteMenuID, String selectedPaletteID) {
        super(parentDialog, "Double Palette", true);
        super.setLayout(new BorderLayout());
        super.setLocationRelativeTo(parentDialog);
        super.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

        this.rightHandFormBean = rightHandFormBean;
        this.leftHandFormBean = leftHandFormBean;
        this.bean = bean;
        this.parentPaletteInformationPanel = parentInformationPanel;
        this.rightHandSelectionPanel = rightHandSelectionPanel;
        this.leftHandSelectionPanel = leftHandSelectionPanel;
        this.scheme = scheme;
        this.defaultPaletteMenuID = defaultPaletteMenuID;
        this.selectedPaletteID = selectedPaletteID;
        this.paletteInformationHelper = new PaletteInformationHelper(scheme, defaultPaletteMenuID);

        JPanel topPanel = this.initializeTopPanel();

        this.enterButton = new JButton("Enter");
        this.cancelButton = new JButton("Cancel");

        this.bottomPanel = this.initializeBottomPanel();

        super.add(topPanel, BorderLayout.NORTH);
        // Child implements the center panel
        super.add(this.bottomPanel, BorderLayout.SOUTH);
        super.pack();
    }

    protected JPanel createSelectionPanel(T selectionPanel) {
        JPanel selectionClearPanel = new JPanel(new GridBagLayout());
        // Black outline border
        selectionClearPanel.setBorder(BorderFactory.createEtchedBorder());
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        Dimension d = new Dimension(super.getPreferredSize().width / 2, selectionPanel.getPreferredSize().height); // This is default and should be overriden by the child class; else the selectionPanels will look weird. Once the center palette is initialized, this size should be updated.
        selectionPanel.setPreferredSize(d);

        JScrollPane selectionScrollPane = new JScrollPane(selectionPanel);
        selectionScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        selectionScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        gridBagConstraints.insets = new Insets(10, 0, 0, 0);
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;

        selectionClearPanel.add(selectionScrollPane, gridBagConstraints);

        JButton clearButton = new JButton("Clear");
        clearButton.addActionListener(e -> {
            selectionPanel.removeAll();
            selectionPanel.revalidate();
            selectionPanel.repaint();
        });


        // Add the clearButtonPanel to the main panel below the JScrollPane
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 0; // Try setting this to a non-zero value, if the button needs more vertical space.
        gridBagConstraints.gridy = 1; // Place the clear button below the JScrollPane
        selectionClearPanel.add(clearButton, gridBagConstraints);

        return selectionClearPanel;
    }

    private JPanel initializeBottomPanel() {
        JPanel bottomPanel = new JPanel(new GridBagLayout());
        bottomPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10)); // Temporary

        GridBagConstraints gridBagConstraints = new GridBagConstraints();


        this.rightHandSelectionClearPanel = this.createSelectionPanel(this.rightHandSelectionPanel);
        this.leftHandSelectionClearPanel = this.createSelectionPanel(this.leftHandSelectionPanel);

        // Add rightHandSelectionPanel to the very left.
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        bottomPanel.add(this.rightHandSelectionClearPanel, gridBagConstraints);

        // Add leftHandSelectionPanel to the very right.
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        bottomPanel.add(this.leftHandSelectionClearPanel, gridBagConstraints);

        // Add buttonPanel to the bottom center.
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
        buttonPanel.add(this.cancelButton);
        buttonPanel.add(this.enterButton);

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;  // This will make the buttonPanel span across both columns.
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.SOUTH;
        bottomPanel.add(buttonPanel, gridBagConstraints);

        return bottomPanel;
    }





    private JPanel initializeTopPanel() {
        JPanel topPanel = new JPanel();
        topPanel.setLayout(new GridBagLayout());
        topPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;

        JLabel paletteLabel = new JLabel("Palette:");
        topPanel.add(paletteLabel, gridBagConstraints);

        gridBagConstraints.gridx = 1;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER; // use all remaining cells in the current row
        JComboBox<String> paletteComboBox = PaletteUIFactory.createComboBox(this.paletteInformationHelper.getTemplateData());
        topPanel.add(paletteComboBox, gridBagConstraints);

        JLabel rightHandLabel = new JLabel(this.rightHandFormBean.getTitle());
        String paletteGroupID = Objects.equals(this.defaultPaletteMenuID, Util.DEFAULT_GLOSS_HANDSHAPES_MENU_ID) ? Util.DEFAULT_GLOSS_HANDSHAPE_PALETTE_GROUP_ID : Util.DEFAULT_GESTURE_HANDSHAPE_PALETTE_GROUP_ID;

        this.rightHandComboBox = PaletteUIFactory.createComboBox(
                this.paletteInformationHelper.getPaletteItemGroupOptions(this.paletteInformationHelper.getCurrentTemplateID(),
                paletteGroupID, this.rightHandFormBean.isRightHandSelected(), this.leftHandFormBean.isRightHandSelected())
        );

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.gridwidth = 1;
        topPanel.add(rightHandLabel, gridBagConstraints);

        gridBagConstraints.gridx = 1;
        topPanel.add(this.rightHandComboBox, gridBagConstraints);

        JLabel leftHandLabel = new JLabel(this.leftHandFormBean.getTitle());
        this.leftHandComboBox = PaletteUIFactory.createComboBox(
                this.paletteInformationHelper.getPaletteItemGroupOptions(this.paletteInformationHelper.getCurrentTemplateID(),
                paletteGroupID, this.leftHandFormBean.isRightHandSelected(), this.rightHandFormBean.isRightHandSelected())
        );

        boolean rightHandIsDominant = this.parentPaletteInformationPanel.isDominantHandRight();

        String selectedItemName = this.paletteInformationHelper.getPaletteNameFromID(this.selectedPaletteID);
        if (rightHandIsDominant && selectedItemName != null) {
            this.rightHandComboBox.setSelectedItem(selectedItemName);
        } else if (selectedItemName != null) {
            this.leftHandComboBox.setSelectedItem(selectedItemName);
        }

        this.paletteInformationHelper.setCurrentDomPaletteID(this.selectedPaletteID);

        // Create a wrapper panel for left hand label and combobox, and anchor it to the right
        gridBagConstraints.gridx = 2;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        JPanel leftHandPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        leftHandPanel.add(leftHandLabel);
        leftHandPanel.add(this.leftHandComboBox);
        topPanel.add(leftHandPanel, gridBagConstraints);

        return topPanel;
    }

    protected String getPaletteID(String currentItem) {
        CSHandShapeTemplate defaultTemplate = this.scheme.getHandShapeTemplates().getTemplates().get(this.paletteInformationHelper.getCurrentTemplateID()); // Get the default template based on the template ID
        String palettePrefix = defaultTemplate.getLabel(); // For example: "ASL" for "American Sign Language" ~ Straight from the defCodingScheme.xml

        for (CSHandShapePalette palette : this.paletteInformationHelper.getHandShapePalettes()) {
            String paletteName = String.format("%s - %s", palettePrefix, palette.getName());
            if (currentItem.equals(paletteName)) {
                return palette.getId();
            }
        }

        return null;
    }


    public HandShapesPaletteFormBean getRightHandFormBean() {
        return this.rightHandFormBean;
    }

    // Setter and Getter methods for leftHandFormBean
    public HandShapesPaletteFormBean getLeftHandFormBean() {
        return this.leftHandFormBean;
    }

    // Setter and Getter methods for bean
    public HandShapeFormBean getBean() {
        return this.bean;
    }

    // Setter and Getter methods for parentPaletteInformationPanel
    public ParentPaletteInformationHelper<K> getParentPaletteInformationPanel() {
        return this.parentPaletteInformationPanel;
    }

    // Setter and Getter methods for scheme
    public SS3CodingScheme getScheme() {
        return this.scheme;
    }

    // Setter and Getter methods for paletteInformationHelper
    public PaletteInformationHelper getPaletteInformationHelper() {
        return this.paletteInformationHelper;
    }

    // Setter and Getter methods for rightHandSelectionPanel
    public T getDomHandSelectionPanel() {
        return this.rightHandSelectionPanel;
    }

    public void setRightHandSelectionPanel(T rightHandSelectionPanel) {
        this.rightHandSelectionPanel = rightHandSelectionPanel;
    }

    // Setter and Getter methods for leftHandSelectionPanel
    public T getNonDomHandSelectionPanel() {
        return this.leftHandSelectionPanel;
    }

    public void setLeftHandSelectionPanel(T leftHandSelectionPanel) {
        this.leftHandSelectionPanel = leftHandSelectionPanel;
    }

    // Setter and Getter methods for rightHandComboBox
    public JComboBox<String> getRightHandComboBox() {
        return this.rightHandComboBox;
    }

    // Setter and Getter methods for leftHandComboBox
    public JComboBox<String> getLeftHandComboBox() {
        return this.leftHandComboBox;
    }

    // Setter and Getter methods for defaultPaletteMenuID
    public String getDefaultPaletteMenuID() {
        return this.defaultPaletteMenuID;
    }

    // Setter and Getter methods for selectedPaletteID
    public String getSelectedPaletteID() {
        return this.selectedPaletteID;
    }

    // Setter and Getter methods for enterButton
    public JButton getEnterButton() {
        return this.enterButton;
    }

    // Setter and Getter methods for cancelButton
    public JButton getCancelButton() {
        return this.cancelButton;
    }

    protected JPanel getBottomPanel() {
        return this.bottomPanel;
    }

    protected JPanel getRightHandSelectionClearPanel() {
        return this.rightHandSelectionClearPanel;
    }

    protected JPanel getLeftHandSelectionClearPanel() {
        return this.leftHandSelectionClearPanel;
    }
}
