package edu.bu.signstream.grepresentation.fields.presentation;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.NewChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.DeletePresentationEntityActionListener;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.popup.TextEntryPopupGenerator;

public class PresentationFieldWrapper extends FieldWrapper {
	private int counter = 1;
	protected PresentationField prField = null;

	private ArrayList<SegmentFieldWrapper> segmentsFieldWrapper = new ArrayList<SegmentFieldWrapper>();

	public PresentationFieldWrapper(PresentationField field, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(field, segmentPanel, timeRange);
		// super.setType(Constants.PresentationFieldWrapper); <------- This type is not used at the moment... It might
		// be used later to solve some unrelated issues with gesture setTime... There is a nullPointer there if you get the dependent
		// wrappers, hence this setType
		this.prField = field;
	}

	public ArrayList<SegmentFieldWrapper> getSegmentsFieldWrappers() {
		return segmentsFieldWrapper;
	}
	
	public void removeAllEntities() {
		prField.removeAllEntities();
	}

	public void addSegmentFieldWrapper(SegmentFieldWrapper wrapper) {
		segmentsFieldWrapper.add(wrapper);
	}
	
	public SignStreamSegmentPanel getSegmentPanel(){
		return segmentPanel;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		prField.addEventsEntity(entity);
	}

	// MouseListener
	@Override
	public void mouseClicked(Point point, JPanel panel) {
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
	    var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()));

		Event sEvent = field.getEventAt(point.x);

		if (!isEntitySelected() && sEvent == null && time.lessThan(SS3Singleton.getMediaPlayer().end())) {
			NewEvent event = new NewEvent(time, segmentPanel);
			if (prField.getNextEntity(event.getStartTimeInfo()) != null) {
				// If this entity has an event in front of it, set a reference to it
				event.setNextEntity(prField.getNextEntity(event.getStartTimeInfo()));
			} else {
				// If this entity has no event in front of it, create a temporary "boundary" event at
				// the end of the movie duration to prevent this new event from going out of bounds.
				PresentationEvent endBound = new PresentationEvent(segmentPanel);
				endBound.setStartTimeInfo(
				    SS3Singleton.getVideoTimeRange().time(Math.round(segmentPanel.getMultipleMovieController().getMovieDuration()), TimeUnit.TIMESTAMP));
				event.setNextEntity(new PresentationEventsEntity(endBound, segmentPanel, field));
			}
			
			if (prField.getPreviousEntity(event.getStartTimeInfo()) != null) {
				// If this entity has an event behind it, set a reference to it
				event.setPreviousEntity(prField.getPreviousEntity(event.getStartTimeInfo()));
			} else {
				// If this entity has no event behind it, create a temporary "boundary" event at the
				// beginning of the movie to prevent this new event from going out of bounds.
				PresentationEvent startBound = new PresentationEvent(segmentPanel);
				startBound.setEndTimeInfo(SS3Singleton.getMediaPlayer().start());
				event.setPreviousEntity(new PresentationEventsEntity(startBound, segmentPanel, field));
			}
			prField.setNewEvent(event);
		}
	}

	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}
	
	public void mouseCtrlAltClicked(Point point) {

		final SegmentGraphicRepresentation repPanel = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation();

		EventsEntity entity = prField.getSelectedEntity();

		final JPopupMenu popup = new JPopupMenu();
		JMenuItem delete = new JMenuItem("Delete");
		UserInterfaceUtil.setMenuItemLNF(delete);
		delete.addActionListener(new DeletePresentationEntityActionListener(
				entity, prField, this, repPanel));
		popup.add(delete);

		JMenuItem cancel = new JMenuItem("Cancel");
		UserInterfaceUtil.setMenuItemLNF(cancel);
		popup.add(cancel);

		popup.show(repPanel, point.x, point.y);
		popup.setPreferredSize(popup.getPreferredSize());
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}

	/*
	 * Show selected utterance/segment, etc
	 */
	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {

		Event event = field.getEventAt(point.x);

		SS3SignStreamApplication.logger.log(Level.INFO, "Selected event " + event);
		
		if (event == null)
			return;
		
		PresentationField pField = (PresentationField) field;
		
		String defaultSegmentTierLabel = null;
		if (pField.getTemporalPartition() != null) {
			defaultSegmentTierLabel = pField.getTemporalPartition().getDefaultSegment().getLabel();
		}

		if (defaultSegmentTierLabel != null) {
			for (SegmentFieldWrapper segmentFieldWrapper: segmentsFieldWrapper) {
				String segmentTierLabel = segmentFieldWrapper.getField().getName();
				if (defaultSegmentTierLabel.equals(segmentTierLabel)) {
					// re-route to load just the default segment tier of the clicked temporal partition
					segmentFieldWrapper.mouseDblLeftClicked(point, panel);
				}
			}
		}
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		if (panel == null)
			return;
		panel.setCursor(new Cursor(getCursorType(point)));

		Event event = prField.getEventAt(point.x);
		if (event != null) {
			panel.setToolTipText(event.getToolTipText());
		} else {
			panel.setToolTipText("");
		}
	}

	/**
	 * Deals with event logic when a user ctrl+clicks or alt+clicks and drags the mouse; or when 
	 * they regular click and drag the mouse.
	 * @param point
	 * @param panel
	 */
	@Override
	public void mouseDragged(Point point, JPanel panel) {
	    var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.NEAREST)
            .clamp(SS3Singleton.getMediaPlayer().interval());

        var player = SS3Singleton.getMediaPlayer();
        if (time.lessThan(player.start())) {
            time = player.start();
        } else if (time.greaterThan(player.end())) {
            time = player.end();
        }

            // Returns null if user drags normally, and not null if user is creating a new event
		NewChainedEvent newEvent = prField.getNewEvent();
		if (newEvent != null) {
			// Returns a temporary entity representing the end of the movie if there is no event
			// after this one.
			PresentationEventsEntity nextPresEntity = 
					(PresentationEventsEntity) newEvent.getNextEntity();
			// Returns a temporary entity representing the start of the movie if there is no event
			// before this one.
			PresentationEventsEntity prevPresEntity = 
					(PresentationEventsEntity) newEvent.getPreviousEntity();
			
			// If there is an event after this one, limit this event's end time to that event's 
			// start time. Otherwise, limit it to the end of the movie.
			Time max = nextPresEntity.getStartTimeInfo()
                .in(Util.getCurrentRange(getField()), Rounding.FLOOR);
			// If there is an event before this one, limit this event's start time to that event's
			// end time. Otherwise, limit it to zero to prevent a negative event start time
			Time min = prevPresEntity.getEndTimeInfo()
                .in(Util.getCurrentRange(getField()), Rounding.CEIL);

			if (time.greaterThan(max)) {
				newEvent.setEndTimeInfo(max);
				player.seek(max);
			} else if (time.lessThan(min)) {
				newEvent.setEndTimeInfo(min);
				player.seek(min);
			} else {
                newEvent.setEndTimeInfo(time);
                player.seek(time);
			}
		} else {
			if (panel.getCursor().getType() != Cursor.E_RESIZE_CURSOR)
				return;

			var sEntity = prField.getSelectedEntity();
			var sEvent = prField.getSelectedEvent();

			if (MOVE_DIRECTION == MOVE_START_POINT) {

			    var bar = SS3Singleton.getMediaToolBar();
				
		    	  SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				double seStartCoord = sEvent.getStartTimeCoordinate();

                if (seStartCoord < point.x
						&& point.x >= sEvent.getEndTimeCoordinate()) {

					var maxPossibleStartTime = sEvent.getEndTimeInfo();
                    MediaToolbarCommons.setPresentationEventStartTime(maxPossibleStartTime, sEvent, sEntity);
                    bar.update();

					return;
				} else if (seStartCoord > point.x) {
					ArrayList<?> entities = prField.getEntities();
					for (int i = 0; i < entities.size(); i++) {
						PresentationEventsEntity ent = (PresentationEventsEntity) entities
								.get(i);
						if (!ent.isSelected() && !ent.getEvent().isSelected()) {
							if (point.x <= ent.getEndTimeCoordinate()
									&& ent.getEndTimeCoordinate() <= seStartCoord) {

                                MediaToolbarCommons.setPresentationEventStartTime(ent.getEndTimeInfo().in(sEvent.getStartTimeInfo().range()), sEvent, sEntity);
                                SS3Singleton.getMediaPlayer().seek(ent.getEndTimeInfo());
                                bar.update();
								return;
							}
						}
					}
				}

                MediaToolbarCommons.setPresentationEventStartTime(time.in(sEvent.getEndTimeInfo().range()), sEvent, sEntity);
				SS3Singleton.getMediaPlayer().seek(time);
				bar.update();
			}

			if (MOVE_DIRECTION == MOVE_END_POINT) {

                var bar = SS3Singleton.getMediaToolBar();

		    	SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				
				double seEndCoord = sEvent.getEndTimeCoordinate();

				if (seEndCoord > point.x
						&& point.x <= sEvent.getStartTimeCoordinate()) {

					var minPossibleEndTime = sEvent.getStartTimeInfo();
                    MediaToolbarCommons.setPresentationEventEndTime(minPossibleEndTime, sEvent, sEntity);
					bar.update();

					String[] msg = new String[4];
					msg[0] = "Warning: The event's start time ";
					msg[1] = "cannot exceed  event's end time. ";
					msg[2] = "You must manually adjust that ";
					msg[3] = "end time and then try again. ";
					cantAutomaticallyAdjustObjectsTime(msg);
					return;
				} else if (seEndCoord < point.x) {
					ArrayList<?> entities = prField.getEntities();
					for (int i = 0; i < entities.size(); i++) {
						PresentationEventsEntity ent = (PresentationEventsEntity) entities.get(i);
						if (!ent.isSelected() && !ent.getEvent().isSelected()) {
							if (point.x >= ent.getStartTimeCoordinate()
									&& ent.getStartTimeCoordinate() >= seEndCoord) {

								MediaToolbarCommons.setPresentationEventEndTime(ent.getStartTimeInfo().in(sEvent.getEndTimeInfo().range()), sEvent, sEntity);
								SS3Singleton.getMediaPlayer().seek(ent.getStartTimeInfo());
								bar.update();
								return;
							}
						}
					}
				}

                if (MediaToolbarCommons.setPresentationEventEndTime(time.in(sEvent.getEndTimeInfo().range()), sEvent, sEntity)) {
                    SS3Singleton.getMediaPlayer().seek(time);
                    bar.update();
                }
			}
		}
	}

	@Override
	public void mouseReleased(Point point, JPanel panel) {
		NewEvent newEvent = prField.getNewEvent();
		if (newEvent != null) {
			
	  	    SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

			ArrayList<?> entities = prField.getEntities();

			var start = newEvent.getStartTimeInfo();
			var end = newEvent.getEndTimeInfo();

            // If event was created by dragging backwards, switch the start and end coordinates
			if (start.greaterThan(end)) {
			    var temp = start;
			    start = end;
			    end = temp;
            }

			for (int i = 0; i < entities.size(); i++) {
				PresentationEventsEntity ent = (PresentationEventsEntity) entities
						.get(i);
				ent.unhighlight();
				ent.unselect();
				ent.getEvent().unhighlight();
				ent.getEvent().unselect();
			}

			SS3GlossWindowItem item = new SS3GlossWindowItem("", "");
			item.setId(Util.getLongUniqueNumber());

			PresentationEvent event = new PresentationEvent(item, segmentPanel);
			
			event.setStartTimeInfo(start);
			event.setEndTimeInfo(end);
			
			item.setEvent(event);

			event.select();
			
			// Deals with utterances overlapping or being too small
			PresentationEventsEntity nextPresEntity = 
					(PresentationEventsEntity) newEvent.getNextEntity();
			PresentationEventsEntity prevPresEntity = 
					(PresentationEventsEntity) newEvent.getPreviousEntity();
			
			if (event.getStartTimeInfo().lessThan(prevPresEntity.getEndTimeInfo())) {
				event.setStartTimeInfo(prevPresEntity.getEndTimeInfo());
			}

			if(event.getEndTimeInfo().greaterThan(nextPresEntity.getStartTimeInfo())){
			    event.setEndTimeInfo(nextPresEntity.getStartTimeInfo());
//				event.getEndTimeInfo().setUnitValue(nextPresEntity.getStartTimeInfo().getUnitValueAs(convertUnit));
			}


			boolean isUnique = false;
			counter = prField.getFieldSize() + 1;

			String label = "";
			while (!isUnique) {
				label = "U" + counter;
				isUnique = prField.isEventLabelUnique(label);
				counter++;
			}

			event.setText(label);
			event.setStartTimeInfo(event.getStartTimeInfo());
			event.setEndTimeInfo(event.getEndTimeInfo());

			PresentationEventsEntity entity = new PresentationEventsEntity(
					event, segmentPanel, prField);

			prField.addEventsEntity(entity);

			item.setStartTimeInfo(event.getStartTimeInfo());
			item.setEndTimeInfo(event.getEndTimeInfo());
			item.setLabel(event.getText());
			item.setId(event.getID());

			prField.getSS3GlossWindowSegment().addSS3GlossWindowItem(item);

			for (int j = 0; j < segmentsFieldWrapper.size(); j++) {
				SegmentFieldWrapper wrapper = segmentsFieldWrapper.get(j);

				PresentationEventsEntity clonedEntity = entity.clone();
				entity.addDependentEntity(clonedEntity);
				wrapper.addEventsEntity(clonedEntity);

				SS3GlossWindowItem pItem = new SS3GlossWindowItem(wrapper.getPartitionLabel(), wrapper.getSegmentLabel());
				pItem.setId(Util.getLongUniqueNumber());
				pItem.setStartTimeInfo(item.getStartTimeInfo());
				pItem.setEndTimeInfo(item.getEndTimeInfo());
				pItem.setLabel(event.getText());
				//
				PresentationEvent pEvent = clonedEntity.getEvent();
				pEvent.setSS3GlossWindowItem(pItem);
				pItem.setEvent(pEvent);
				item.addSS3GlossWindowItem(pItem);

				PresentationField dependentPresentationField = (PresentationField) wrapper
						.getField();
				dependentPresentationField.getSS3GlossWindowSegment()
						.addSS3GlossWindowItem(pItem);
			}

			entity.select();

			MediaToolBar tb = segmentPanel.getToolBarPanels().getMediaToolBar();
			tb.selectEvent(event, entity, prField);

			prField.setNewEvent(null);
		}

 		segmentPanel.repaint();
	}

	private int getCursorType(Point point) {
		int cursorType = Cursor.DEFAULT_CURSOR;
		Event event = prField.getSelectedEvent();

		if (event != null) {
			double startCoord = event.getStartTimeCoordinate();
			double endCoord = event.getEndTimeCoordinate();

			if (startCoord - 3 < point.x && point.x < startCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_START_POINT;
			}

			if (endCoord - 3 < point.x && point.x < endCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_END_POINT;
			}
		}
		return cursorType;
	}

	@Override
	public void unselectAll() {
		field.unselect();
	}

	@Override
	public double getStartTime() {
		return field.getStartTime();
	}

	@Override
	public double getEndTime() {
		return field.getEndTime();
	}

	@Override
	public int getViewLineNumber() {
		if (viewLineNumber >= 0)
			return viewLineNumber;
		return Integer.parseInt(field.getFieldID());
	}

	@Override
	public Event getSelectedEvent() {
		return field.getSelectedEvent();
	}

	private void cantAutomaticallyAdjustObjectsTime(String[] msg) {
		final JPopupMenu popup = new JPopupMenu();

		JLabel msgLabel1 = new JLabel(msg[0]);
		JLabel msgLabel2 = new JLabel(msg[1]);
		JLabel msgLabel3 = new JLabel(msg[2]);
		JLabel msgLabel4 = new JLabel(msg[3]);

		UserInterfaceUtil.setLabelLNF(msgLabel1);
		UserInterfaceUtil.setLabelLNF(msgLabel2);
		UserInterfaceUtil.setLabelLNF(msgLabel3);
		UserInterfaceUtil.setLabelLNF(msgLabel4);

		final JButton okBtn = new JButton("OK");

		JPanel msgPanel = new JPanel();
		GridBagConstraints gbc = new GridBagConstraints();

		msgPanel.setLayout(new GridBagLayout());
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridwidth = 3;
		gbc.gridx = 0;
		gbc.gridy = 0;
		msgPanel.add(msgLabel1, gbc);

		gbc.gridy = 1;
		gbc.gridx = 0;
		msgPanel.add(msgLabel2, gbc);

		gbc.gridy = 2;
		gbc.gridx = 0;
		msgPanel.add(msgLabel3, gbc);

		gbc.gridy = 3;
		gbc.gridx = 0;
		msgPanel.add(msgLabel4, gbc);

		gbc.gridy = 4;
		gbc.gridx = 0;
		msgPanel.add(new JLabel(" "), gbc);

		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridy = 5;
		gbc.gridx = 0;
		msgPanel.add(new JLabel(" "), gbc);
		gbc.gridx = 2;
		msgPanel.add(okBtn, gbc);

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				popup.setVisible(false);
			}
		});

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				okBtn.requestFocus();
			}
		});

		Dimension d = new Dimension();
		int width = msgPanel.getPreferredSize().width + 30;
		int height = msgPanel.getPreferredSize().height + 20;
		d.setSize(width, height);

		CoordinateTimeConverter convertor = segmentPanel.getZoomer()
				.getCoordinateTimeConvertor();

        var x = SS3Singleton.getMediaPlayer().time().as(TimeUnit.COORDINATE, Rounding.NEAREST);

		int y = field.getY() + height / 2 + 50;

		popup.add(msgPanel);
		popup.setPopupSize(d);
		popup.show(field.getSignStreamSegmentPanel().getDatabaseView(), (int) x, y);
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}

	@Override
	public void mouseHoldReleased(Point point, JPanel panel) {

		Event event = field.getSelectedEvent();

		if (event == null)
			return;

		final PresentationEvent pEvent = (PresentationEvent) event;

		PresentationEventsEntity selectedEntity = (PresentationEventsEntity) field
				.getSelectedEntity();
		ArrayList<PresentationEventsEntity> entities = selectedEntity
				.getDependentEntities();

        Point loc = new Point((int)event.getStartTimeCoordinate(), field.getY() - 11);
        
		TextEntryPopupGenerator.getInstance("", panel, loc, label -> {
			ArrayList<PresentationEventsEntity> fieldEntities = field.getEntities();
			for (PresentationEventsEntity fieldEntity: fieldEntities) {
				PresentationEvent existingEvent = fieldEntity.getEvent();
				String existingText = existingEvent.getText();
				if (label.equals(existingText) && ! existingEvent.equals(pEvent)) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.DUPLICATE_NAME_UTTERANCE);
					return;
				}
			}
			
			pEvent.setText(label);

			for (PresentationEventsEntity entity : entities) {
				entity.getEvent().setText(label);
			}

			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
			
			segmentPanel.repaint();
        }, () -> {
            segmentPanel.repaint();
        });
	}
}
