package edu.bu.signstream.grepresentation.fields.signTypeField;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;

public class SignTypeField extends Field {

    private ArrayList<SignTypeEntity> entities = new ArrayList<SignTypeEntity>();

    public SignTypeField(SS3Track ss3Track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
        super(segmentPanel, parentField);
        this.ss3Track = ss3Track;
        createTrackEntity(ss3Track);
    }

    private void createTrackEntity(SS3Track track) {
        ArrayList<SignTypeEntity> stEntities = new ArrayList<SignTypeEntity>();
        ArrayList<SS3Entity> ent = track.getEntities();
        for (int j = 0; j < ent.size(); j++) {
            SS3Entity entity = ent.get(j);

            SS3Item item = entity.getItemsAsList().get(0);
            SignTypeEvent event = new SignTypeEvent(segmentPanel, item);

            SignTypeEntity stEntity = new SignTypeEntity(event, segmentPanel, this);

            stEntity.setID(entity.getID());
            stEntity.setRefEntityId(entity.getRefEntityId());
            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
            	stEntity.updateParentEntity();
            } else {
            	stEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
            }

            stEntities.add(stEntity);
        }
        this.entities = stEntities;
    }

    @Override
    public ArrayList<SignTypeEntity> getEntities() {
        return entities;
    }

    @Override
    public EventsEntity getEntityWithRefID(String refEntityID) {
        for (SignTypeEntity entity : entities) {
            if (entity.getRefEntityId().equals(refEntityID)) {
                return entity;
            }
        }
        return null;
    }

    @Override
    public EventsEntity getEntity(String entityID) {
        for (SignTypeEntity entity : entities) {
            if (entity.getID().equals(entityID)) {
                return entity;
            }
        }
        return null;
    }

    @Override
    public void highlight(boolean highlight, String refID) {

    }

    @Override
    public Event getEventAt(double x) {
        return null;
    }

    @Override
    public EventsEntity getEntityAt(double x) {
        return null;
    }

    @Override
    public void deleteEntity(EventsEntity entity) {
        entities.remove(entity);
    }

    @Override
    public int getFieldSize() {
        return entities.size();
    }

    @Override
    public void deleteEmptyEntities() {}

    @Override
    public boolean isIntervalEmpty(Interval interval) {
        return false;
    }

    @Override
    public void addEventsEntity(EventsEntity entity) {
        entities.add((SignTypeEntity) entity);
    }

    @Override
    public double getStartTimeCoordinate() {
        return entities.get(0).getStartTimeCoordinate();
    }

    @Override
    public double getEndTimeCoordinate() {
        return entities.get(entities.size() - 1).getEndTimeCoordinate();
    }

    @Override
    public void unselect() {}

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        super.paint(g2d, Y, font);
        for (SignTypeEntity entity : entities) {
            entity.paint(g2d, Y, font);
        }
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public EventsEntity getSelectedEntity() {
        return null;
    }
}
