package edu.bu.signstream.grepresentation.view.signbank;

import java.util.HashMap;

public class SignBankVariant {
	
	private String variantID = null;
	private String variantLabel = null;
	private String defaultOccurrenceID = null;
	private String occurrenceCount = null;
	private HashMap<String, SignBankOccurrence> occurrencesList = null;
	
	private SignBankMainGloss parentMainGloss = null;
	
	// Used to identify if an entry has the form "*text (text)" with the "*" in front of the word
	private static String preCompoundRegex = "[*].*\\s[(].*[)]";
	// Used to identify if an entry has the form "text* (text)" with the "*" after the word
	private static String postCompoundRegex = ".*[*]\\s[(].*[)]";
	// Used to remove the " (text)" from a compound
	private static String compoundParenthesesRegex = "\\s[(].*[)]";
	
	// NOTE 2024-07-16: Per Carol, the above are not to be returned from the DAI, only the below
	
	// Used to identify if an entry has the form "text+text"
	private static String plusSignCompoundRegex = "^[^+]+\\+[^+]+$";
	
	public SignBankVariant(SignBankMainGloss parentMainGloss, 
			String variantID, String varaintLabel, String defaultOccurrenceID, String occurrenceCount) {
		this.parentMainGloss = parentMainGloss;
		this.variantID = variantID;
		this.variantLabel = varaintLabel;
		this.defaultOccurrenceID = defaultOccurrenceID;
		this.occurrenceCount = occurrenceCount;
		this.occurrencesList = new HashMap<String, SignBankOccurrence>();
	}
	
	public SignBankMainGloss getParentMainGloss() {
		return parentMainGloss;
	}
	
	public String getVariantLabel() {
		return variantLabel;
	}
	
	public String getVariantID() {
		return variantID;
	}
	
	public SignBankOccurrence getOccurrenceByID(String occurrenceID) {
		return occurrencesList.get(occurrenceID);
	}
	
	public SignBankOccurrence getDefaultOccurrence() {
		return occurrencesList.get(defaultOccurrenceID);
	}
	
	public String getDefaultOccurenceID() {
		return defaultOccurrenceID;
	}
	
	public void setDefaultOccurrenceID(String defaultOccurrenceID) {
		this.defaultOccurrenceID = defaultOccurrenceID;
	}
	
	public String getOccurrenceCount() {
		return occurrenceCount;
	}

	/**
	 * Gets the list of SignBankOccurrence objects
	 * @return A HashMap from Occurrence ID -> Occurrence
	 */
	public HashMap<String, SignBankOccurrence> getOccurrencesList() {
		return occurrencesList;
	}
	
	@Override
	public String toString() {
		
		String ret = "SignBankVariant[";
		ret += "Variant ID=" + variantID;
		ret += ", Variant Label=" + variantLabel;
		ret += ", Default Occurrence ID=" + defaultOccurrenceID;
		for (SignBankOccurrence occurrence: occurrencesList.values()) {
			ret += "\n" + occurrence.toString();
		}
		ret += "\n/SignBankVariant]";
		
		return ret;
	}
	/**
	 * Checks if a label is part of a compound.
	 * @param entryLabel - The gloss label to be checked
	 * @return
	 */
	public static boolean isCompound(String entryLabel) {
		return entryLabel.matches(plusSignCompoundRegex);
	}
	
	/**
	 * Checks if a label is part of a compound and formats it accordingly
	 * @param entryLabel - The gloss label to be checked
	 * @return The variant label with the "*" and trailing parentheses removed; or just the
	 * 			original label if it was not part of a compound
	 */
	public static String getVariantLabel(String entryLabel){
		if (isCompound(entryLabel))
			return entryLabel.replaceAll("[*]", "").replaceAll(compoundParenthesesRegex, "");
		else
			return entryLabel;
	}

}
