package edu.bu.signstream.navigation;

import java.awt.Dialog;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.util.*;
import java.util.logging.Level;

import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.xml.parsers.DocumentBuilderFactory;

import edu.bu.signstream.codingScheme.ui.importLegacyCodingScheme.CodingSchemeUpdateDialog;
import edu.bu.signstream.common.codingSchema.CodingSchema;
import edu.bu.signstream.common.db.SS2toSS3Adaptor;
import edu.bu.signstream.common.db.XMLWriter;
import edu.bu.signstream.common.db.parser.Parser;
import edu.bu.signstream.common.db.parser.SS2Parser;
import edu.bu.signstream.common.db.ss3.SS3Reader;
import edu.bu.signstream.common.db.parser.XMLParser;
import edu.bu.signstream.common.db.ss3.SS3Validator;
import edu.bu.signstream.common.db.ss3.SS3ValidatorConverterReturns;
import edu.bu.signstream.common.db.ss3.SS3_1Converter;
import edu.bu.signstream.common.db.ss3.SS3_1Validator;
import edu.bu.signstream.common.db.ss3.SS3_2Converter;
import edu.bu.signstream.common.db.ss3.SS3_2Validator;
import edu.bu.signstream.common.db.ss3.Tuple;
import edu.bu.signstream.common.db.ss3.Validator;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

import org.w3c.dom.Document;

public class DatabaseDelegate {
	   private DefaultMutableTreeNode topNode = new DefaultMutableTreeNode("  ");

	   private SignStreamSegmentPanel ssPanel;

	   public DatabaseDelegate(SignStreamSegmentPanel ssPanel) {
	      this.ssPanel = ssPanel;
	   }

	   public DefaultMutableTreeNode getMutableTreeNode() {
	      return topNode;
	   }

	   public void openDb(File file, File zipFile) {
	      DefaultMutableTreeNode node = createNode(file, zipFile);
	      topNode.removeAllChildren();	// if we are switching databases, remove the old db name node
	      topNode.add(node);
	      
	      DefaultMutableTreeNode duplicateTopNode = new DefaultMutableTreeNode("  ");
	      DefaultMutableTreeNode duplicateChildNode = (DefaultMutableTreeNode) node.clone();
	      
	      for (Enumeration e = node.children(); e.hasMoreElements() ;) {
              DefaultMutableTreeNode entNode = (DefaultMutableTreeNode)e.nextElement();
              DefaultMutableTreeNode duplicateEntNode = (DefaultMutableTreeNode) entNode.clone();
              duplicateChildNode.add(duplicateEntNode);
	      }
          
	      duplicateTopNode.add(duplicateChildNode);

	      // find db which was created in createNode
	      for (SS3Database db: SS3Singleton.getSS3DatabaseCollection().getDatabases().values()) {
	    	  String str1 = db.getReadFromXMLFile().toString();
	    	  int index = str1.indexOf("/");
	    	  String str2 = str1.substring(0, index);

	    	  if (file.getPath().equals(str2)) {
	    		  db.setDbNavigationPanelNode(duplicateTopNode);
	    	  }
	      }
	   }

	   public void addNewDatabase(String fileName) {
		      
	      DefaultMutableTreeNode fileNode = new DefaultMutableTreeNode(fileName);
	      topNode.add(fileNode);
	   }

	   private void addFile(
			   DefaultMutableTreeNode node, 
			   File file, File zipFile ) {

	      if (file.isFile()) {
	         String fileName = file.getName();
	         if (fileName != null && fileName.equals(SS3Database.CONF_FILE))
	        	 	return;
	         
             if(Util.isXMLFile(fileName) && zipFile != null) {
            	  ////////////////////////////////// VALIDATION AND CONVERSION BELOW ///////////////////////////////////////

            	 // loading panel
            	 var loading = new JDialog(null, "Loading...", Dialog.ModalityType.MODELESS);
            	 var loadingPanel = new JPanel();
            	 loadingPanel.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
            	 loadingPanel.add(new JLabel("Loading collection file. Please wait."));
            	 loading.add(loadingPanel);
            	 loading.pack();
            	 loading.setLocationRelativeTo(ssPanel);
            	 loading.setVisible(true);
            	 // Force the loading dialog to paint immediately
            	 var root = loading.getRootPane();
            	 var bounds = root.getBounds();
            	 root.paintImmediately(0, 0, bounds.width, bounds.height);

	    		  // Get the parsed XML DOM tree
	    	      Document doc;
	    	      try {
	    	    	  doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(file.getPath());
	    	      } catch (Exception e) {
	    	    	  e.printStackTrace();
	    	    	  return;
	    	      }
	    	      
	    	      SS3SignStreamApplication.deleteValidationFiles();

	    	      File errorsFile = new File(SS3SignStreamApplication.VALIDATION_ERRORS);
	    	      errorsFile.delete();

	    	      File modificationsFile = new File(SS3SignStreamApplication.VALIDATION_MODIFICATIONS);
	    	      modificationsFile.delete();

	    	      // actual validation here
	    	      ArrayList<String> oldVersions = new ArrayList<String>(Arrays.asList("1.0", "2.0"));
	    	      ArrayList<String> validatingVersions = new ArrayList<String>(Arrays.asList("3.0", "3.1", "3.2"));
	    	      SS3ValidatorConverterReturns validator_returns;
	    	      if (doc.getDocumentElement() == null)
	    	    	  return;
	    	      var version = doc.getDocumentElement().getAttribute("DATABASE-VERSION");
	    	      Validator validator = null;
	    	      var broken = false;

	    	      ArrayList<Tuple> completeErrorsList = new ArrayList<Tuple>(); 
	    	      ArrayList<Tuple> completePublicErrorsList = new ArrayList<Tuple>(); 
	    	      ArrayList<Tuple> completeModificationsList = new ArrayList<Tuple>(); 
	    	      ArrayList<Tuple> completePublicModificationsList = new ArrayList<Tuple>(); 
	    	      
	    	      if (version != null && oldVersions.contains(version)) {
	    	    	  JOptionPane.showMessageDialog(
	    	    			  null,
	    	    			  "This collection file is too old for the current version of SignStream to open.\n"
	    	    			  + "Please refer to README.txt under the Install/Usage section for more information about how to open this file.",
	    	    			  "Collection Open Error",
	    	    			  JOptionPane.ERROR_MESSAGE);
	    	    	  return;
	    	      }

	    	      if (version != null && validatingVersions.contains(version)) {
	    	    	  /**
	    	    	   * if we are in here, we are either 3.0 or 3.1
	    	    	   * if 3.0 - do validation for 3.0 and conversion to 3.1
	    	    	   * then do the validation for 3.1 regardless
	    	    	   * 
	    	    	   * 3.2 validation does not exist yet, but should proceed
	    	    	   */

	    	    	  if ("3.0".equals(version)) {
	    	    		  SS3Singleton.setDbOriginallyOldScheme(true);

	    	    		  validator = new SS3Validator();
	    	    		  validator_returns = validator.validate(doc);

	    	    		  var validator_errors = validator_returns.errors;
	    	    		  completeErrorsList.addAll(validator_errors);
	    	    		  var validator_public_errors = validator_returns.publicErrors;
	    	    		  completePublicErrorsList.addAll(validator_public_errors);
	    	    		  var validator_modifications = validator_returns.modifications;
	    	    		  completeModificationsList.addAll(validator_modifications);
	    	    		  var validator_public_modifications = validator_returns.publicModifications;
	    	    		  completePublicModificationsList.addAll(validator_public_modifications);

	    	    		  if (!validator_errors.isEmpty()) {
	    	    			  System.out.println("\n3.0 VALIDATION ERRORS:");
	    	    			  for (var e : validator_errors) {
	    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + "Public: " + e.isPublic() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
	    	    			  }
	    	    		  }

	    	    		  if (!validator_public_modifications.isEmpty()) {
	    	    			  try {
	    	    				  modificationsFile.createNewFile();
	    	    				  for (var c : validator_public_modifications) {
	    	    					  String str = c.getCode() + Util.VALIDATOR_CODES_DELIMITER + c.getMsg();
	    	    					  Files.writeString(modificationsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
	    	    				  }
	    	    			  } catch (IOException e) {
	    	    				  e.printStackTrace();
	    	    			  }

	    	    		  }

	    	    		  if (!validator_public_errors.isEmpty()) {
	    	    			  try {
	    	    				  errorsFile.createNewFile();
	    	    				  for (var e : validator_public_errors) {
	    	    					  String str = e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg();
	    	    					  Files.writeString(errorsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
	    	    				  }
	    	    			  } catch (IOException e) {
	    	    				  e.printStackTrace();
	    	    			  }

	    	    		  }

	    	    		  if (!validator_modifications.isEmpty()) {
	    	    			  System.out.println("\n3.0 VALIDATION MODIFICATIONS:");
	    	    			  for (var e : validator_modifications) {
	    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + "Public: " + e.isPublic() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
	    	    			  }
	    	    		  }

    	    			  // save the new xml 
    	    			  try {
    	    				  boolean saved = XMLWriter.saveDocument(doc.getDocumentElement(), file.getPath());
    	    			  } catch (Exception e1) {
    	    				  e1.printStackTrace();
    	    			  }

	    	    		  //	    			  if (validator_errors.isEmpty()) {	// need to check this, because if there were errors we can't move to 3.1	// GREG TODO: not valid anymore

	    	    		  ////////////////////////////////// 3.0 -> 3.1 CONVERSION BELOW ///////////////////////////////////////
	    	    		  if (! (validator instanceof SS3_1Validator)) {
	    	    			  SS3Validator ss3validator = (SS3Validator) validator;
	    	    			  SS3_1Converter converter = new SS3_1Converter();	// edits doc in place
	    	    			  var converter_returns = converter.convert(ss3validator);

	    	    			  var converter_errors = converter_returns.errors;
	    	    			  completeErrorsList.addAll(converter_errors);
	    	    			  var converter_public_errors = converter_returns.publicErrors;
	    	    			  completePublicErrorsList.addAll(converter_public_errors);
	    	    			  var converter_modifications = converter_returns.modifications;
	    	    			  completeModificationsList.addAll(converter_modifications);

	    	    			  if (!converter_modifications.isEmpty()) {
	    	    				  System.out.println("\n3.1 CONVERTER MODIFICATIONS:");
	    	    				  for (var e : converter_modifications)
	    	    					  //				    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
	    	    					  System.out.println(e.getMsg());
	    	    			  }

	    	    			  if (!converter_errors.isEmpty()) {
	    	    				  System.out.println("\n3.1 CONVERTER ERRORS:");
	    	    				  for (var e : converter_errors)
	    	    					  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
	    	    			  }

	    	    			  if (!converter_public_errors.isEmpty()) {
	    	    				  try {
	    	    					  errorsFile.createNewFile();
	    	    					  for (var e : converter_public_errors) {
	    	    						  String str = e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg();
	    	    						  Files.writeString(errorsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
	    	    					  }
	    	    				  } catch (IOException e) {
	    	    					  e.printStackTrace();
	    	    				  }

	    	    			  }
	    	    		  }

	    	    		  // again save the xml 
	    	    		  try {
	    	    			  boolean saved = XMLWriter.saveDocument(doc.getDocumentElement(), file.getPath());
	    	    		  } catch (Exception e1) {
	    	    			  e1.printStackTrace();
	    	    		  }
	    	    		  
	    	    		  // at this point the DATABASE-VERSION has also been changed from 3.0 -> 3.1
	    	    		  version = "3.1";
	    	    	  }

	    	    	  if ("3.1".equals(version)) {

		    	    	  ///////////////////////// 3.1 VALIDATION BELOW //////////////////////////

		    	    	  // actual validation here
		    	    	  validatingVersions = new ArrayList<String>(Arrays.asList("3.1"));
		    	    	  if (doc.getDocumentElement() == null)
		    	    		  return;
		    	    	  version = doc.getDocumentElement().getAttribute("DATABASE-VERSION");
		    	    	  validator = null;
		    	    	  broken = false;

		    	    	  if (version != null && validatingVersions.contains(version)) {
		    	    		  validator = new SS3_1Validator();
		    	    		  validator_returns = validator.validate(doc); 

		    	    		  var validator_errors = validator_returns.errors;
		    	    		  completeErrorsList.addAll(validator_errors);
		    	    		  var validator_public_errors = validator_returns.publicErrors;
		    	    		  completePublicErrorsList.addAll(validator_public_errors);
		    	    		  var validator_modifications = validator_returns.modifications;
		    	    		  completeModificationsList.addAll(validator_modifications);
		    	    		  var validator_public_modifications = validator_returns.publicModifications;
		    	    		  completePublicModificationsList.addAll(validator_public_modifications);

		    	    		  if (!validator_errors.isEmpty()) {
		    	    			  System.out.println("\n3.1 VALIDATION ERRORS:");
		    	    			  for (var e : validator_errors) {
		    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + "Public: " + e.isPublic() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
		    	    			  }
		    	    		  }

		    	    		  if (!validator_public_modifications.isEmpty()) {
		    	    			  try {
		    	    				  modificationsFile.createNewFile();
		    	    				  for (var c : validator_public_modifications) {
		    	    					  String str = c.getCode() + Util.VALIDATOR_CODES_DELIMITER + c.getMsg();
		    	    					  Files.writeString(modificationsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
		    	    				  }
		    	    			  } catch (IOException e) {
		    	    				  e.printStackTrace();
		    	    			  }

		    	    		  }

		    	    		  if (!validator_public_errors.isEmpty()) {
		    	    			  try {
		    	    				  errorsFile.createNewFile();
		    	    				  for (var e : validator_public_errors) {
		    	    					  String str = e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg();
		    	    					  Files.writeString(errorsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
		    	    				  }
		    	    			  } catch (IOException e) {
		    	    				  e.printStackTrace();
		    	    			  }

		    	    		  }

		    	    		  if (!validator_modifications.isEmpty()) {
		    	    			  System.out.println("\n3.1 VALIDATION MODIFICATIONS:");
		    	    			  for (var e : validator_modifications) {
		    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + "Public: " + e.isPublic() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
		    	    			  }
		    	    		  }

	    	    			  // save the new xml 
	    	    			  try {
	    	    				  boolean saved = XMLWriter.saveDocument(doc.getDocumentElement(), file.getPath());
	    	    			  } catch (Exception e1) {
	    	    				  e1.printStackTrace();
	    	    			  }
		    	    		  

		    	    		  ////////////////////////////////// 3.1 -> 3.2 CONVERSION BELOW ///////////////////////////////////////
		    	    		  
		    	    		  SS3_1Validator ss3_1validator = (SS3_1Validator) validator;
		    	    		  SS3_2Converter converter = new SS3_2Converter();	// edits doc in place
		    	    		  var converter_returns = converter.convert(ss3_1validator);

		    	    		  var converter_errors = converter_returns.errors;
		    	    		  completeErrorsList.addAll(converter_errors);
		    	    		  var converter_public_errors = converter_returns.publicErrors;
		    	    		  completePublicErrorsList.addAll(converter_public_errors);
		    	    		  var converter_modifications = converter_returns.modifications;
		    	    		  completeModificationsList.addAll(converter_modifications);

		    	    		  if (!converter_modifications.isEmpty()) {
		    	    			  System.out.println("\n3.2 CONVERTER MODIFICATIONS:");
		    	    			  for (var e : converter_modifications)
		    	    				  //				    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
		    	    				  System.out.println(e.getMsg());
		    	    		  }

		    	    		  if (!converter_errors.isEmpty()) {
		    	    			  System.out.println("\n3.2 CONVERTER ERRORS:");
		    	    			  for (var e : converter_errors)
		    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
		    	    		  }

		    	    		  if (!converter_public_errors.isEmpty()) {
		    	    			  try {
		    	    				  errorsFile.createNewFile();
		    	    				  for (var e : converter_public_errors) {
		    	    					  String str = e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg();
		    	    					  Files.writeString(errorsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
		    	    				  }
		    	    			  } catch (IOException e) {
		    	    				  e.printStackTrace();
		    	    			  }

		    	    		  }

		    	    		  // again save the xml 
		    	    		  try {
		    	    			  boolean saved = XMLWriter.saveDocument(doc.getDocumentElement(), file.getPath());
		    	    		  } catch (Exception e1) {
		    	    			  e1.printStackTrace();
		    	    		  }
		    	    		  
		    	    		  // at this point the DATABASE-VERSION has also been changed from 3.1 -> 3.2
		    	    		  version = "3.2";
		    	    	  }

		    	    	  //	    			  }
		    	    	  
	    	    	  }
	    	    	  
	    	    	  if ("3.2".equals(version)) {

		    	    	  ///////////////////////// 3.2 VALIDATION BELOW //////////////////////////

		    	    	  // actual validation here
		    	    	  validatingVersions = new ArrayList<String>(Arrays.asList("3.2"));
		    	    	  if (doc.getDocumentElement() == null)
		    	    		  return;
		    	    	  version = doc.getDocumentElement().getAttribute("DATABASE-VERSION");
		    	    	  validator = null;
		    	    	  broken = false;

		    	    	  if (version != null && validatingVersions.contains(version)) {
		    	    		  validator = new SS3_2Validator();
		    	    		  validator_returns = validator.validate(doc); 

		    	    		  var validator_errors = validator_returns.errors;
		    	    		  completeErrorsList.addAll(validator_errors);
		    	    		  var validator_public_errors = validator_returns.publicErrors;
		    	    		  completePublicErrorsList.addAll(validator_public_errors);
		    	    		  var validator_modifications = validator_returns.modifications;
		    	    		  completeModificationsList.addAll(validator_modifications);
		    	    		  var validator_public_modifications = validator_returns.publicModifications;
		    	    		  completePublicModificationsList.addAll(validator_public_modifications);

		    	    		  if (!validator_errors.isEmpty()) {
		    	    			  System.out.println("\n3.2 VALIDATION ERRORS:");
		    	    			  for (var e : validator_errors) {
		    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + "Public: " + e.isPublic() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
		    	    			  }
		    	    		  }

		    	    		  if (!validator_public_modifications.isEmpty()) {
		    	    			  try {
		    	    				  modificationsFile.createNewFile();
		    	    				  for (var c : validator_public_modifications) {
		    	    					  String str = c.getCode() + Util.VALIDATOR_CODES_DELIMITER + c.getMsg();
		    	    					  Files.writeString(modificationsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
		    	    				  }
		    	    			  } catch (IOException e) {
		    	    				  e.printStackTrace();
		    	    			  }

		    	    		  }

		    	    		  if (!validator_public_errors.isEmpty()) {
		    	    			  try {
		    	    				  errorsFile.createNewFile();
		    	    				  for (var e : validator_public_errors) {
		    	    					  String str = e.getCode() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg();
		    	    					  Files.writeString(errorsFile.toPath(), str + System.lineSeparator(), StandardOpenOption.APPEND);
		    	    				  }
		    	    			  } catch (IOException e) {
		    	    				  e.printStackTrace();
		    	    			  }

		    	    		  }

		    	    		  if (!validator_modifications.isEmpty()) {
		    	    			  System.out.println("\n3.2 VALIDATION MODIFICATIONS:");
		    	    			  for (var e : validator_modifications) {
		    	    				  System.out.println(e.getCode() + Util.VALIDATOR_CODES_DELIMITER + "Public: " + e.isPublic() + Util.VALIDATOR_CODES_DELIMITER + e.getMsg());
		    	    			  }

		    	    		  }

	    	    			  // save the new xml 
	    	    			  try {
	    	    				  boolean saved = XMLWriter.saveDocument(doc.getDocumentElement(), file.getPath());
	    	    			  } catch (Exception e1) {
	    	    				  e1.printStackTrace();
	    	    			  }
		    	    	  }
	    	    		  
	    	    	  }
    	    		  
	    	    	  

	    	    	  // AFTER ALL VALIDATIONS/CONVERSIONS - the below shows all of the modifications/errors at once
	    	    	  if (!completePublicModificationsList.isEmpty()) {

	    	    		  String msg = " modifications were";
	    	    		  if (completePublicModificationsList.size() == 1) {
	    	    			  msg = " modification was";
	    	    		  }
	    	    		  JOptionPane.showMessageDialog(
	    	    				  null,
	    	    				  completePublicModificationsList.size() + msg + " made in the collection file being opened.\n"
	    	    						  + "Please verify that the data modifications made are correct.\n\n"

    					  	+ "You may view modifications by clicking on the\n"
    					  	+ "\"Help\" Menu and selecting \"View Collection File Modifications\"",
    					  	"Collection Validation Modifications",
    					  	JOptionPane.WARNING_MESSAGE);
	    	    	  }

	    	    	  if (!completePublicErrorsList.isEmpty()) {

	    	    		  String msg = " errors were";
	    	    		  if (completePublicErrorsList.size() == 1) {
	    	    			  msg = " error was";
	    	    		  }
	    	    		  JOptionPane.showMessageDialog(
	    	    				  null,
	    	    				  completePublicErrorsList.size() + msg + " detected in the collection file being opened.\n\n"
	    	    						  + "It is unknown how such errors will affect the SignStream Application.\n"
	    	    						  + "Continuing to open this file and saving it may result in data corruption or even data loss.\n\n"

    					  	+ "If you choose to continue, you may view errors by clicking on the\n"
    					  	+ "\"Help\" Menu and selecting \"View Collection File Errors\"",
    					  	"Collection Validation Errors",
    					  	JOptionPane.WARNING_MESSAGE);
	    	    	  }

	    	      } else {
	    	    	  JOptionPane.showMessageDialog(
	    	    			  null,
	    	    			  "This collection file is too new for the current version of SignStream to open.\nPlease upgrade SignStream in order to open this collection.",
	    	    			  "Collection Open Error",
	    	    			  JOptionPane.ERROR_MESSAGE);
	    	    	  return;
	    	      }
    			  
	    		  
	    	     var reader = new SS3Reader();
	        	 SS3Database db = reader.readDatabase(file.getPath());		// this is the newly validated and converted xml file
	        	 if (broken) db.setBroken();
	        	 
	        	 // close the loading panel
	        	 loading.dispose();

	        	 try {
	        		 File parentFile = file.getParentFile();
	        		 String directory = parentFile.getName();
	        		 String parentDirectoryPath = parentFile.getParent();
	        		 
	        		 // keep this as `zipfile` (the original collection file) for when saving occurs to just overwrite it
	        		 if(zipFile!=null) {
	            		 directory = zipFile.getName();
	            		 parentDirectoryPath = zipFile.getParent();
	        		 }
	        		 
	        		 db.setFileName(fileName);
	        		 db.setDirectory(directory);
	        		 db.setParentDirectoryPath(parentDirectoryPath);
	        	 } catch(Exception ex) {
	        		 ex.printStackTrace();
					 SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
	        	 }

	             ArrayList<String> mediaFileIDs = new ArrayList<String>();
	             ArrayList<SS3MediaFile> mFiles = db.getMediaFilesCollection();
	             Iterator<SS3MediaFile> iterator = mFiles.iterator();
	             while (iterator.hasNext()) {
	                SS3MediaFile mFile = iterator.next();
	                mediaFileIDs.add(mFile.getId());
	             }

	             db.getExcerptObj().setMediaFiles(mediaFileIDs);

	             SS3Singleton.getSS3DatabaseCollection().addSS3SDatabase(db);
	             db.setReadFromXMLFile(file);

	             DefaultMutableTreeNode[] nodes = new DefaultMutableTreeNode[30];
	             
	             for(int i = 0; i < 30; i++) {
	            	 nodes[i] = null;
	             }
	             
	             String dbNumber = db.getNumber();

	             if(dbNumber == null || dbNumber.trim().length()==0)
	            	 dbNumber = "1"; 
	             
	             int newNodeNumber = Integer.valueOf(dbNumber);
	             
	             nodes[newNodeNumber] = new DefaultMutableTreeNode(db.getExcerptObj()); 
	             
	             for (Enumeration e = node.children(); e.hasMoreElements() ;) {
	                 DefaultMutableTreeNode entNode = (DefaultMutableTreeNode)e.nextElement();
	                 Excerpt excerpt = (Excerpt)entNode.getUserObject();
	                 
	                 SS3Database sDb = SS3Singleton.getSS3DatabaseCollection().getSS3Database(excerpt);
	                 String number = sDb.getNumber();
	                 int n = Integer.valueOf(number);
	                 nodes[n] = entNode; 
	             }
	             
	             node.removeAllChildren();
	             
	             for(int j = 0; j < 30; j++) {
	            	 if(nodes[j] != null)
	            		 node.add(nodes[j]);
	             }
//	         } else if (Util.isXMLFile(fileName)) {
//                // Execute if a database is in older format, unzipped files.
//                // Filename should have mask filename.xml
//	            SS3Parser parser = new SS3Parser();
//	            SS3Database db = parser.parseSS3Database(file.getPath());
//
//	            Excerpt excerpt = new Excerpt(db.getNumber(), db.getExcerpt());
//
//	            ArrayList<String> mediaFileIDs = new ArrayList<String>();
//	            ArrayList<SS3MediaFile> mFiles = db.getMediaFilesCollection();
//	            Iterator<SS3MediaFile> iterator = mFiles.iterator();
//	            while (iterator.hasNext()) {
//	               SS3MediaFile mFile = iterator.next();
//	               mediaFileIDs.add(mFile.getId());
//	            }
//
//	            excerpt.setMediaFiles(mediaFileIDs);
//
//	            SS3Singleton.getSS3DatabaseCollection().addSS3SDatabase(db);
//	            db.setReadFromXMLFile(file);
//
//	            DefaultMutableTreeNode cNode = new DefaultMutableTreeNode(excerpt);
//	           
//	            node.add(cNode);
	         } else {
	            SS3SignStreamApplication.logger.log(
	               Level.INFO,
	               "Parsing binary file [" + file.getName() + "].");
	            try {
	               SignStreamDocument ssDoc = createSSDocument(file.getAbsolutePath());

	               loadLegacyCodingScheme(fileName, ssDoc.getCodingScheme());

	               if (ssDoc != null) {
	                  SS2toSS3Adaptor adaptor = new SS2toSS3Adaptor(ssDoc);
	                  ArrayList<Excerpt> excerptList = ssDoc.getExcerpts();

	                  for (int j = 0; j < excerptList.size(); j++) {
	                     Excerpt excerpt = excerptList.get(j);
	                     excerpt.setPath(file.getAbsolutePath());

	                     SS3Database db = adaptor.loadSS2Database(excerpt);
	                     SS3Singleton.getSS3DatabaseCollection().addSS3SDatabase(db);
	                     db.setExcerpt(excerpt.getExcerpt());

	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Database ["
	                           + db.getShortDescription()
	                           + "] had been added to the database collection. Updating database references.");

//	                     updateDatabaseReferencesToCodingScheme(scheme, db);		// GD: not sure what this is for

	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Database references had been updated. Creating TreeNode that represents the database.");

	                     DefaultMutableTreeNode cNode =
	                        new DefaultMutableTreeNode(excerpt);

	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Tree node had been created.");
	                     
	                     node.insert(cNode, j);
	                     
	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Excerpt [" + excerpt + "] Tree node had been added.");
	                  }
	                  SS3SignStreamApplication.logger.log(
	                     Level.INFO,
	                     "Legacy file had been loaded.");
	               }
	            } catch (IllegalArgumentException ex) {
	               SS3SignStreamApplication.logger.log(
	                  Level.SEVERE,
	                  "File ["
	                     + fileName
	                     + "] is not a valid file name."
	                     + ex.getStackTrace());
	            }
	         }
	      }
	   }

    private SignStreamDocument createSSDocument(String path)
            throws IllegalArgumentException {
        Parser parser;

        if (edu.bu.signstream.common.util.Util.isXMLFile(path)) {
            parser = new XMLParser();
        } else if (
                !edu.bu.signstream.common.util.Util.isMovieFile(path)
                        && !edu.bu.signstream.common.util.Util.isTextFile(path)) {
            parser = new SS2Parser();
        } else {
            throw new IllegalArgumentException();
        }

        SignStreamDocument doc = parser.loadSSDocument(path);

        if (doc != null) {
            CodingSchema codingSchema = new CodingSchema();
            codingSchema.loadCodingSchema(doc.getCodingScheme());
            SS3Singleton.setCodingSchema(codingSchema);
        }

        return doc;
    }

	   private void updateDatabaseReferencesToCodingScheme(
	      SS3CodingScheme cScheme,
	      SS3Database db) {

	      HashMap fields = cScheme.getFields();
	      Set set = fields.keySet();
	      Iterator iterator = set.iterator();

	      while (iterator.hasNext()) {
	         String key = (String) iterator.next();
	         Object value = fields.get(key);
	         SS3Field field = (SS3Field) value;
	         String fieldLegacyID = field.getLegacyID();

	         // update value references. Do not log.
	         Iterator valuesIterator = field.getValuesAsList().iterator();
	         while (valuesIterator.hasNext()) {
	            SS3FieldValue fValue = (SS3FieldValue) valuesIterator.next();
	            String legacyValueID = fValue.getLegacyID();

	            if (legacyValueID != null && legacyValueID.length() > 0) {

	               SS3Track track = db.getTrack(fieldLegacyID);
	               if (track != null) {
	                  ArrayList entities = track.getEntities();
	                  for (int i = 0; i < entities.size(); i++) {
	                     SS3Entity entity = (SS3Entity) entities.get(i);
	                     ArrayList items = entity.getItemsAsList();

	                     for (int j = 0; j < items.size(); j++) {
	                        SS3Item item = (SS3Item) items.get(j);
	                        if (item.getId().equalsIgnoreCase(legacyValueID)) {
	                           item.setId(fValue.getId());
	                           j = items.size();
	                           i = entities.size();
	                        }
	                     }
	                  }
	               }
	            }
	         }

	         // update field references
	         if (fieldLegacyID != null && fieldLegacyID.length() > 0) {
	            SS3Track track = db.getTrack(fieldLegacyID);
	            if (track != null)
	               track.setId(field.getId());
	         }
	      }
	   }

	   private void loadLegacyCodingScheme(String fileName, HashMap codingScheme) {

	      CodingSchemeUpdateDialog updateDialog = new CodingSchemeUpdateDialog(
                  fileName,
                  codingScheme,
                  ssPanel,
                  ssPanel.getParentFrame());
	      updateDialog.setVisible(true);
	   }

	   private boolean isExcerptLoaded(Excerpt excerpt) {
	      Enumeration e = topNode.children();

	      while (e.hasMoreElements()) {
	         Object obj = e.nextElement();
	         try {
	            DefaultMutableTreeNode ex = (DefaultMutableTreeNode) obj;
	            Object uo = ex.getUserObject();
	            Excerpt ex1 = (Excerpt) uo;

	            if (ex1.getExcerpt().equals(excerpt.getExcerpt()))
	               return true;
	         } catch (Exception ex) {
	            ;
	         }
	      }
	      return false;
	   }

	   public void addExcerpt(Excerpt excerpt) {
	      if (isExcerptLoaded(excerpt))
	         return;

	      DefaultMutableTreeNode cNode = new DefaultMutableTreeNode(excerpt);
	      topNode.add(cNode);
	   }

	   public void adddNewDatabase(File file) {
	      DefaultMutableTreeNode node = new DefaultMutableTreeNode(file.getName());
	      topNode.add(node);
	   }

	   private void addDirectory(DefaultMutableTreeNode node, File file, File zipFile) {
	      File[] files = file.listFiles();

	      for (int i = 0; i < files.length; i++) {
	         File nFile = files[i];
	       
	         if (nFile.isFile()
	            && !edu.bu.signstream.common.util.Util.isMovieFile(nFile.getName())
	            && !edu.bu.signstream.common.util.Util.isTextFile(nFile.getName())) {

	            addFile(node, nFile, zipFile);
	         } else if (
	            nFile.isDirectory()
	               && !nFile.getName().equalsIgnoreCase("movies")) {

	            DefaultMutableTreeNode dirNode =
	               new DefaultMutableTreeNode(nFile.getName());
	            addDirectory(dirNode, nFile, zipFile);
	            node.add(dirNode);
	         }
	      }
	   }

	   public DefaultMutableTreeNode createNode(File file, File zipFile) {
		  
		  String cDirName = file.getName();
			
			if(cDirName.startsWith(SS3Database.tempDirectoryPrefix)) {
				cDirName = cDirName.substring(4);				
			}
			
			if(cDirName.endsWith(SS3Database.zipFileExtension)) {			
				cDirName = cDirName.substring(0, cDirName.length()-4);
			}
			
			if(cDirName.endsWith(SS3Database.dbFileExtension)) {
				cDirName = cDirName.substring(0, cDirName.length()-4);
			}
			
			if(cDirName.endsWith(SS3Database.COLLECTION_EXT)) {
				cDirName = cDirName.substring(0, cDirName.length()-4);
			}
		  
	      DefaultMutableTreeNode node = new DefaultMutableTreeNode(cDirName);

	      if (file.isDirectory()) {
	         addDirectory(node, file, zipFile);
	      } else if (file.isFile()) {
	         addFile(node, file, zipFile);
	      }
	      return node;
	   }
	}
