package edu.bu.signstream.time.ui;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.popup.Popup;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

public class TimeRangeEditDialog extends Popup {
    private final TimeRange range;

    private final JTextField offsetField = new JTextField();
    private final JTextField stepField = new JTextField();

    private boolean updating = false;

    public TimeRangeEditDialog(TimeRange range) {
        this.range = range;

        var panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(new EmptyBorder(5, 0, 0, 0));

        panel.add(new JLabel("Edit Time Range " + range.name()));
        panel.add(Box.createVerticalStrut(5));

        var offsetRow = new JPanel(new FlowLayout());
        offsetField.setText(String.format("%,d", range.offset().value()));
        offsetField.setPreferredSize(new Dimension(120, 17));
        offsetField.setEnabled(range != TimeRange.ABSOLUTE);
        offsetField.getDocument().addDocumentListener(new DocumentListener() {
            private void update() {
                if (updating) return;
                try {
                    var offset = Long.parseLong(offsetField.getText().replaceAll(",", ""));
                    SwingUtilities.invokeLater(() -> {
                        updating = true;
                        offsetField.setText(String.format("%,d", offset));
                        updating = false;
                    });
                } catch (Exception ignored) { }
            }
            @Override public void insertUpdate(DocumentEvent e) { update(); }
            @Override public void removeUpdate(DocumentEvent e) { update(); }
            @Override public void changedUpdate(DocumentEvent e) { update(); }
        });
        var offsetAdd = new JButton("+");
        UserInterfaceUtil.setButtonsLNF(offsetAdd);
        offsetAdd.addActionListener(e -> {
            SwingUtilities.invokeLater(() -> {
                try {
                    var offset = Long.parseLong(offsetField.getText().replaceAll(",", ""));
                    offset += range.parent().step();

                    updating = true;
                    offsetField.setText(String.format("%,d", offset));
                    updating = false;

                    applyOffset();
                } catch (Exception ignored) { }
            });
        });
        var offsetSub = new JButton("-");
        UserInterfaceUtil.setButtonsLNF(offsetSub);
        offsetSub.addActionListener(e -> {
            SwingUtilities.invokeLater(() -> {
                try {
                    var offset = Long.parseLong(offsetField.getText().replaceAll(",", ""));
                    offset -= range.parent().step();

                    updating = true;
                    offsetField.setText(String.format("%,d", offset));
                    updating = false;

                    applyOffset();
                } catch (Exception ignored) { }
            });
        });
        offsetAdd.setEnabled(range != TimeRange.ABSOLUTE);
        offsetSub.setEnabled(range != TimeRange.ABSOLUTE);
        offsetRow.add(new JLabel("Offset: "));
        offsetRow.add(offsetField);
        offsetRow.add(new JLabel(range.offset().unit().suffix()));
        offsetRow.add(offsetSub);
        offsetRow.add(offsetAdd);
        panel.add(offsetRow);
        panel.add(Box.createVerticalStrut(5));

        var stepRow = new JPanel(new FlowLayout());
        stepField.setText(String.format("%,d", range.step()));
        stepField.setPreferredSize(new Dimension(120, 17));
        stepField.getDocument().addDocumentListener(new DocumentListener() {
            private void update() {
                if (updating) return;
                try {
                    var step = Long.parseLong(stepField.getText().replaceAll(",", ""));
                    SwingUtilities.invokeLater(() -> {
                        updating = true;
                        stepField.setText(String.format("%,d", step));
                        updating = false;
                    });
                } catch (Exception ignored) { }
            }
            @Override public void insertUpdate(DocumentEvent e) { update(); }
            @Override public void removeUpdate(DocumentEvent e) { update(); }
            @Override public void changedUpdate(DocumentEvent e) { }
        });
        stepRow.add(new JLabel("Step: "));
        stepRow.add(stepField);
        stepRow.add(new JLabel(range.unit().suffix()));

        panel.add(stepRow);
        panel.add(Box.createVerticalStrut(5));

        var apply = new JButton("Apply");
        apply.addActionListener(l -> {
            try {
                applyOffset();
                applyStep();

                SS3Singleton.getMediaToolBar().update();
                SS3Singleton.getSignStreamApplication()
                    .getSignStreamSegmentPanel()
                    .repaint();
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this,
                    e.getMessage(), "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        });

        var close = new JButton("Close");
        close.addActionListener(l -> {
            setVisible(false);
        });

        var row = new JPanel(new FlowLayout());
        row.add(close);
        row.add(apply);
        panel.add(row);

        add(panel);
    }

    private void applyOffset() throws Exception {
        long value;
        try {
            value = Long.parseLong(offsetField.getText().replaceAll(",", ""));
        } catch (NumberFormatException e) {
            throw new Exception("Invalid offset.");
        }

        var offset = new Duration(value, range.offset().unit());

        var player = SS3Singleton.getMediaPlayer();
        if (range == SS3Singleton.getVideoTimeRange()) {
            player.videos().forEach(v -> {
                player.offset(v, offset);
                v.range().offset(offset);
            });
            range.offset(offset);
        } else if (range == SS3Singleton.getAudioTimeRange()) {
            player.audios().forEach(a -> {
                player.offset(a, offset);
                a.range().offset(offset);
            });
            range.offset(offset);
        } else if (range != TimeRange.ABSOLUTE) {
            range.offset(offset);
        }

        var converter = SS3Singleton.getVideoControlManager()
            .getZoomer()
            .getCoordinateTimeConvertor();

        var duration = player.duration().as(TimeUnit.TIMESTAMP);
        converter.setTimeScale(converter.timebase(), converter.timestep(), duration);

        player.seek(player.time());
    }

    private void applyStep() throws Exception {
        long step;
        try {
            step = Long.parseLong(stepField.getText().replaceAll(",", ""));
        } catch (NumberFormatException e) {
            throw new Exception("Invalid step.");
        }

        var player = SS3Singleton.getMediaPlayer();
        if (range == SS3Singleton.getVideoTimeRange()) {
            player.videos().forEach(v -> {
                v.range().step(step);
            });
        } else if (range == SS3Singleton.getAudioTimeRange()) {
            player.audios().forEach(a -> {
                a.range().step(step);
            });
        }

        range.step(step);
    }
}
