package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.panels.table.sortable.TableSorter;

public class CSValuesPanel extends JPanel implements ActionListener {
	private SS3Field csField = null;

	private JTable table;
	private Object[] columnNames = { "Editable", "Value", "Label", "" };

	Object[][] data = null;

	private DefaultTableModel tableModel = null;

	private TableSorter sorter = null;
	private JLabel newValueLbl = new JLabel("New Value:");
	private JLabel newLabelLbl = new JLabel(" Label:");
	private JTextField newValueTxt = new JTextField(10);
	private JTextField newLabelTxt = new JTextField(7);

	private JButton addBtn = new JButton(" Add ");

	public CSValuesPanel(String fieldID, SS3CodingScheme scheme) {
		if (fieldID != null)
			csField = scheme.getField(fieldID);
		
		UserInterfaceUtil.setLabelLNF(newValueLbl);
		UserInterfaceUtil.setLabelLNF(newLabelLbl);		
		UserInterfaceUtil.setTextFieldLNF(newValueTxt);
		UserInterfaceUtil.setTextFieldLNF(newLabelTxt);

		addBtn.addActionListener(this);
		
		//prepareAndStartDAISearch();
	}

	
	private void init() {
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridBagLayout());

		centerPanel.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();

		gbc.fill = GridBagConstraints.HORIZONTAL;		
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridy= 4;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(3, 3, 3, 3);
		centerPanel.add(newValueLbl, gbc);				
		centerPanel.add(Box.createHorizontalStrut(5));
		centerPanel.add(newValueTxt, gbc);
		centerPanel.add(newLabelLbl, gbc);		
		centerPanel.add(Box.createHorizontalStrut(5));
		centerPanel.add(newLabelTxt, gbc);

		gbc.gridx=0;
		gbc.gridy=0;
		gbc.gridwidth = 4;
		gbc.gridheight = 4;		
		
		JScrollPane scrollPane = createValuesTable();
		scrollPane.setPreferredSize(new Dimension(centerPanel.getPreferredSize().width-10, 97));
		centerPanel.add(scrollPane, gbc);
		
		setLayout(new BorderLayout());
		add(centerPanel, BorderLayout.PAGE_START);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		SS3FieldValue fValue = getNewFieldValue();

		if (fValue.getName().length() == 0) {
			String msg = "Value field cannot be empty.";
			
			SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", msg);
		} else {
			String[] row = { "*", fValue.getName(), fValue.getLabel(),
					fValue.getId() };
			tableModel.addRow(row);
			tableModel.fireTableDataChanged();
			sePreferedSize();

			newValueTxt.setText("");
			newLabelTxt.setText("");
		}
	}

	public SS3FieldValue getNewFieldValue() {
		String valueID = Util.getLongUniqueNumber();
		String value = newValueTxt.getText();
		String label = newLabelTxt.getText();

		SS3FieldValue fValue = new SS3FieldValue();
		fValue.setId(valueID);
		fValue.setName(value);
		fValue.setLabel(label);
		return fValue;
	}

	private JScrollPane createValuesTable() {
		ArrayList valueList = new ArrayList();

		if (csField != null) {
			ArrayList values = csField.getValuesAsList();
			for (int i = 0; i < values.size(); i++) {
				SS3FieldValue csValue = (SS3FieldValue) values.get(i);
				String updateable = " "; //csValue.isUpdateable()?"*":"";
				Object[] value = { updateable, csValue.getName(),
						csValue.getLabel(), csValue.getId() };

				valueList.add(value);
			}
		}

		data = new Object[valueList.size()][];

		for (int j = 0; j < valueList.size(); j++) {
			data[j] = (Object[]) valueList.get(j);
		}

		tableModel = new DefaultTableModel(data, columnNames);
		sorter = new TableSorter(tableModel);
		table = new JTable(sorter);

		sorter.setTableHeader(table.getTableHeader());
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);		
		table.getColumnModel().removeColumn(table.getColumnModel().getColumn(3));
		table.setAutoCreateColumnsFromModel(false);

		JScrollPane jsp = new JScrollPane(table);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		return jsp;
	}

	private int sePreferedSize() {
		TableColumn updatable = table.getColumn(columnNames[0]);
		TableColumn value = table.getColumn(columnNames[1]);
		TableColumn label = table.getColumn(columnNames[2]);

		int updateablePrefSize = columnHeaderWidth(updatable); 
		int valuePrefSize = getPreferredWidthForColumn(value);
		int labelPrefSize = getPreferredWidthForColumn(label);

		updatable.setMinWidth(updateablePrefSize);
		updatable.setMaxWidth(updateablePrefSize);
		value.setMinWidth(valuePrefSize);
		label.setMinWidth(labelPrefSize);
		
		return updateablePrefSize + valuePrefSize + labelPrefSize;
	}

	public int getPreferredWidthForColumn(TableColumn col) {
		int hw = columnHeaderWidth(col), // hw = header width
		cw = widestCellInColumn(col); // cw = column width

		return hw > cw ? hw : cw;
	}

	private int columnHeaderWidth(TableColumn col) {
		//TableCellRenderer renderer = col.getHeaderRenderer();
		return 75;
	}

	private int widestCellInColumn(TableColumn col) {
		int c = col.getModelIndex(), width = 0, maxw = 0;

		for (int r = 0; r < table.getRowCount(); ++r) {
			TableCellRenderer renderer = table.getCellRenderer(r, c);

			Component comp = renderer.getTableCellRendererComponent(table,
					table.getValueAt(r, c), false, false, r, c);

			width = comp.getPreferredSize().width;
			maxw = width > maxw ? width : maxw;
		}
		return maxw + 10;
	}

	public String getSelectedFieldValueID() {
		SS3FieldValue fValue = getNewFieldValue();
		String name = fValue.getName();

		if (name != null && name.length() > 0) {
			csField.addValue(fValue);
			return fValue.getId();
		} else {
			updateLoadedCodingSchema();
			int row = table.getSelectedRow();
			return (String) sorter.getValueAt(row, 3);
		}
	}

	public void updateLoadedCodingSchema() {
		Vector data = tableModel.getDataVector();

		for (int i = 0; i < data.size(); i++) {
			Vector obj = (Vector) data.get(i);
			String editable = (String) obj.get(0);

			if (editable.equals("*")) {
				String value = (String) obj.get(1);
				String label = (String) obj.get(2);
				String valueID = (String) obj.get(3);

				SS3FieldValue fValue = new SS3FieldValue();
				fValue.setId(valueID);
				fValue.setName(value);
				fValue.setLabel(label);

				csField.addValue(fValue);
			}
		}
	}

	public ArrayList getValues() {
		ArrayList list = new ArrayList();
		Vector data = tableModel.getDataVector();

		for (int i = 0; i < data.size(); i++) {
			Vector obj = (Vector) data.get(i);
			String editable = (String) obj.get(0);

			if (editable.equals("*")) {
				String value = (String) obj.get(1);
				String label = (String) obj.get(2);
				String valueID = (String) obj.get(3);

				SS3FieldValue fValue = new SS3FieldValue();
				fValue.setId(valueID);
				fValue.setName(value);
				fValue.setLabel(label);
				list.add(fValue);
			}
		}
		return list;
	}
}
