package edu.bu.signstream.ui.panels.codingScheme;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.FieldsGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.NonmanualFieldsView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.AbstractMutableTreeTableNode;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;
import org.jdesktop.swingx.treetable.MutableTreeTableNode;
import org.jdesktop.swingx.treetable.TreeTableNode;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

public class GraphFieldPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private JDialog parent = null;

	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private NonmanualFieldsView nonManualView;
	private SegmentGraphicRepresentation rep;
	private CodingSchemaFieldNode fieldsRootNode;
	private JXTreeTable fieldsTable;
	private JScrollPane scrollPane;

	private TSelectionListener selectionListener = null;
	private MutableTreeTableNode selectedNode = null;
	
	private JButton doneBtn = new JButton("Done");

	private GridBagConstraints gbc = new GridBagConstraints();

	// field ids that might have been used to code data
	private ArrayList<String> fieldIDs = new ArrayList<String>();

	// fields recently created and added to the coding schema
	private ArrayList<SS3Field> recentlyAddedFields = new ArrayList<SS3Field>();

	private AddOrUpdateField newFieldPanel = null;

	public GraphFieldPanel(SignStreamSegmentPanel segmentPanel, Segment segment, NonmanualFieldsView nonManualView,
			JDialog dialog, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.nonManualView = nonManualView;
		this.segment = segment;
		this.parent = dialog;

		doneBtn.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
					enter();
			}
		});

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		selectionListener = new TSelectionListener(this, dbScheme);

		scrollPane = getTreeTableScrollPanel();
		Dimension d = new Dimension(scrollPane.getPreferredSize().width + 260, 250);

		scrollPane.setPreferredSize(d);
		scrollPane.setMinimumSize(d);
		scrollPane.setMaximumSize(d);

		setLayout(new GridBagLayout());

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(1, 1, 1, 1);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 9;
		add(scrollPane, gbc);
		gbc.gridwidth = 1;

		gbc.gridx = 2;
		add(Box.createHorizontalStrut(365), gbc);

		gbc.gridx = 10;
		add(doneBtn, gbc);
	}

	public void setSelectedTreeTableNode(MutableTreeTableNode selectedNode) {
		this.selectedNode = selectedNode;
	}

	public boolean isFieldNameAndLabelUnique(String fieldName, String fieldLabel, int groupID) {
		MutableTreeTableNode node = groupID >= 0 && groupID < 4
				? (MutableTreeTableNode) fieldsRootNode.getChildAt(groupID)
				: fieldsRootNode;

		int numberOfChildren = node.getChildCount();
		for (int i = 0; i < numberOfChildren; i++) {
			MutableTreeTableNode childNode = (MutableTreeTableNode) node.getChildAt(i);
			String existingFieldName = (String) childNode.getValueAt(0);
			String existingFieldLabel = (String) childNode.getValueAt(1);

			if (existingFieldName.equals(fieldName) || existingFieldLabel.equals(fieldLabel)) {
				return false;
			}
		}
		return true;
	}

	public void addNewField(SS3Field field) {
		recentlyAddedFields.add(field);

	    SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		dbScheme.addFieldAndOrder(field);

		int groupID = field.getGroupID();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		MutableTreeTableNode node = groupID >= 0 && groupID < 4
				? (MutableTreeTableNode) fieldsRootNode.getChildAt(groupID)
				: fieldsRootNode;

		Object[] newFieldObject = new Object[] { field.getName(), field.getLabel(), new Boolean(true), " ",
				Color.RED, "" + field.getId() };
		CodingSchemaFieldNode newFieldNode = new CodingSchemaFieldNode(newFieldObject);
		fieldsModel.insertNodeInto(newFieldNode, node, node.getChildCount());

		scrollToAndSelectNode(newFieldNode);
	}

	public void updateField(SS3Field field) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();

		// check if field is still in the same group
		int legacyGroupID = getGroup(selectedNode.getParent());
		int groupID = field.getGroupID();

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		dbScheme.addField(field);

		Object[] newFieldObject = new Object[] { field.getName(), field.getLabel(), new Boolean(true), " ",
				Color.RED, "" + field.getId() };
		fieldsModel.setUserObject(selectedNode, newFieldObject);

		if (legacyGroupID != groupID) {
			fieldsModel.removeNodeFromParent(selectedNode);
			addNewField(field);
		}
	}
	

	public void enter() {
		ArrayList<String> slctFieldIDs = getSelectedFieldIDsNUpdateCodingSchema();

		LabelObject object = segment.getRootLabelObject().getGraphLabelObject();

		SS3Participant currParticipant = segment.getParentField().getSS3GlossWindowSegment().getParticipant();
		ArrayList<LabelObject> children = object.getAllChildren(currParticipant);

		// make all not visible
		for (LabelObject child : children) {
			if (child.getWrapper() instanceof GraphFieldWrapper) {
				child.setVisible(false);
			}
		}

		// set to true only those fields that should be visible
		for (LabelObject child : children) {
			String id = child.getFieldID();

			if (containsID(slctFieldIDs, id)) {
				slctFieldIDs.remove(id);
				child.setVisible(true);
			}
		}

		nonManualView.update(segment);
		nonManualView.getLabelView().loadWrappers();
		nonManualView.getLabelView().updateUI();
		nonManualView.getLabelView().repaint();
        SS3Singleton.getMediaToolBar().deselectLabel();

		parent.dispose();
	}

	private boolean containsID(ArrayList<String> slctFieldIDs, String id) {
		for (int i = 0; i < slctFieldIDs.size(); i++) {
			String fID = slctFieldIDs.get(i);
			id.trim();
			fID.trim();
			if (fID.equals(id))
				return true;
		}
		return false;
	}

	public JScrollPane getTreeTableScrollPanel() {
		populateFieldsTable();

		String[] cNames = { " Field  Names   ", "Field Labels", "Display by Default", "Annotated", "Color", " " };

		DefaultTreeTableModel fieldsModel = new DefaultTreeTableModel(fieldsRootNode, Arrays.asList(cNames)) {
			@Override
			public Class<?> getColumnClass(int column) {

				if (column == 2) {
					return Boolean.class;
				} else if (column == 4) {
					return Color.class;
				} else {
					return String.class;
				}
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
				return column == 2;
			}

			@Override
			public int getChildCount(Object parent) {
				// Only override count if at the root of the tree.
				if (parent == fieldsRootNode) {
					CodingSchemaFieldNode root = (CodingSchemaFieldNode) parent;
					int children = 0;
					for (int i = 0; i < root.getChildCount(); i++) {
						TreeTableNode child = root.getChildAt(i);
						// Nonmanual Fields contain an ArrayList of child IDs in their 5th column
						if (child.getValueAt(5) instanceof ArrayList)
							children++;
					}
					return children;
				} else
					return super.getChildCount(parent);
			}
		};

		fieldsTable = new JXTreeTable(fieldsModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == 4) {
					return new ColorRenderer(true);
				}

				return super.getCellRenderer(row, column);
			}
		};

		TableColumn customCol = fieldsTable.getColumnModel().getColumn(4);
		customCol.setCellEditor(new ColorEditor());

		fieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		fieldsTable.setHorizontalScrollEnabled(true);
		fieldsTable.setFillsViewportHeight(false);
		fieldsTable.setRootVisible(false);

		// do not show field id column
		fieldsTable.getColumnModel().removeColumn(fieldsTable.getColumnModel().getColumn(5));
		fieldsTable.setAutoCreateColumnsFromModel(false);

		UIDefaults uiDefs = UIManager.getDefaults();
		uiDefs.put("Tree.leftChildIndent", new Integer(3));

		fieldsTable.setOpenIcon(null);
		fieldsTable.setClosedIcon(null);
		fieldsTable.setLeafIcon(null);

		fieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		fieldsTable.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		fieldsTable.addTreeSelectionListener(selectionListener);

		fieldsTable.packAll();

		JScrollPane jsp = new JScrollPane(fieldsTable);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		return jsp;
	}

	public int getGroup(TreeTableNode node) {
		if (node == null)
			return 1;

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode rootNode = fieldsModel.getRoot();
		TreeTableNode gphNode = rootNode.getChildAt(0);
		if (node.equals(gphNode)) {
			return 0;
		}
		return 1;
	}

	private class CodingSchemaFieldNode extends AbstractMutableTreeTableNode {
		public CodingSchemaFieldNode(Object[] data) {
			super(data);
			if (data == null) {
				throw new IllegalArgumentException("Node data cannot be null");
			}
		}

		@Override
		public int getColumnCount() {
			return getData().length;
		}

		@Override
		public Object getValueAt(int column) {
			return getData()[column];
		}

		public Object[] getData() {
			return (Object[]) getUserObject();
		}

		private boolean isGroup(TreeTableNode node) {
			DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
			TreeTableNode rootNode = fieldsModel.getRoot();
			TreeTableNode gphNode = null;
			
			if(rootNode != null && rootNode.getChildAt(0) != null) {
				gphNode= rootNode.getChildAt(0);
			}
			
			return node.equals(gphNode);
		}

		@Override
		public void setValueAt(Object value, int column) {
			super.setValueAt(value, column);
			if (column == 2) {
				Boolean isChecked = (Boolean) value;
				getData()[column] = isChecked;

				// for head, face, body, and grammatical nodes - select/unselect
				// all children if group is selected/unselected
				if (isGroup(this)) {
					int childCount = getChildCount();
					for (int i = 0; i < childCount; i++) {
						TreeTableNode node = getChildAt(i);
						if (node != null)
							node.setValueAt(value, column);
					}
				} else if (!isChecked) {
					// if parent node is unselected, all children nodes should
					// be unselected as well
					int childCount = getChildCount();
					for (int i = 0; i < childCount; i++) {
						TreeTableNode node = getChildAt(i);
						node.setValueAt(value, column);
					}
				} else {
					DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
					TreeTableNode rootNode = fieldsModel.getRoot();
					TreeTableNode parentNode = this;
					while (!parentNode.getParent().equals(rootNode)) {
						parentNode = parentNode.getParent();
					}

					if (parentNode != this && !isGroup(parentNode)) {
						getParent().setValueAt(value, column);
					}
				}
			} else if (column == 4) {
				getData()[column] = value;
			}
		}
	}

	private HashMap<String, CodingSchemaFieldNode> getFieldsList(Collection<SS3Field> fieldsToDisplay,
			SegmentGraphicRepresentation rep) {

		HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
		Iterator<SS3Field> iter = fieldsToDisplay.iterator();
		while (iter.hasNext()) {
			SS3Field csField = iter.next();
			fields.put(csField.getId(), csField);
		}

		LabelObject object = segment.getRootLabelObject().getGraphLabelObject();
		ArrayList<LabelObject> leafsOnly = object.getAllChildren();
		ArrayList<String> available = new ArrayList<String>();
		ArrayList<String> visible = new ArrayList<String>();

		Hashtable<String, Color> colors = new Hashtable<String, Color>();
		HashMap<String, CodingSchemaFieldNode> nodes = new HashMap<String, CodingSchemaFieldNode>();
 
		for (int i = 0; i < leafsOnly.size(); i++) {
			LabelObject obj = leafsOnly.get(i);
			FieldWrapper wrapper = obj.getWrapper();
			String id = wrapper.getField().getFieldID();

			Color color = wrapper.getField().getColor();
			if (color != null)
				colors.put(id, color);

			if (obj.isVisible())
				visible.add(id);

			if (!available.contains(id) && wrapper.getField().getFieldSize() > 0)
				available.add(id);
		}

		if (fields != null) {
			Set<String> csKeys = fields.keySet();
			Object[] sortedKeys = Sorter.sortIntegerSet(csKeys);

			for (int i = 0; i < sortedKeys.length; i++) {
				SS3Field csField = fields.get(sortedKeys[i]);

				String fieldID = "" + csField.getId();

				Boolean checked = visible.contains(fieldID) ? new Boolean(true) : new Boolean(false);

				String dt = available.contains(fieldID) ? "  *  " : "";

				Color color = csField.getColor();
				Object col = colors.get(fieldID);

				if (col != null)
					color = (Color) col;

				Object[] values = new Object[] { csField.getName(), csField.getLabel(), checked, dt, Color.RED, fieldID };

				CodingSchemaFieldNode node = new CodingSchemaFieldNode(values);
				nodes.put(fieldID, node);
			}
		}
		return nodes;
	}

	private void populateFieldsTable() {
		fieldsRootNode = new CodingSchemaFieldNode(
				new Object[] { "Field Name", "Field Label", "true", "Data", "Color.BLUE", "" });

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		HashMap<String, SS3Field> fields = dbScheme.getDependentRootFields();
		HashMap<String, CodingSchemaFieldNode> gphNodes = getFieldsList(dbScheme.getSortedFields(), rep);

		FieldsGroup group = dbScheme.loadGphFieldGroup();

		Boolean checked = null;
		String dt = "";
		ArrayList<CodingSchemaFieldNode> nodes = new ArrayList<CodingSchemaFieldNode>();
		ArrayList<String> ids = group.getFieldIDs();

		for (String id : ids) {

			fieldIDs.add(id);

			CodingSchemaFieldNode node = gphNodes.get(id);
			if (node != null) {
				Object[] values = (Object[]) node.getUserObject();
				checked = (Boolean) values[2];
				String dt1 = (String) values[3];

				if (dt1.trim().length() > 0) {
					dt = dt1;
				}

				nodes.add(node);
			}
		}

		Object[] value = new Object[] { group.getGroupName(), "", true, dt, Color.RED, group.getFieldIDs() }; // temporary fix
		CodingSchemaFieldNode gNode = new CodingSchemaFieldNode(value);
		fieldsRootNode.add(gNode);

		for (CodingSchemaFieldNode node1 : nodes) {
			gNode.add(node1);
		}

		if (fields != null) {
			Set<String> csKeys = fields.keySet();
//			Object[] sortedKeys = Sorter.sortIntegerSet(csKeys);
			Object[] sortedKeys = csKeys.toArray();

			for (int i = 0; i < sortedKeys.length; i++) {
				SS3Field childField = fields.get(sortedKeys[i]);
				addChild(fieldsRootNode, childField);
			}
		}
	}

	private void addChild(CodingSchemaFieldNode parent, SS3Field childField) {
		CodingSchemaFieldNode child = createFieldNode(childField);
		parent.add(child);
		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		ArrayList<SS3Field> depFields = dbScheme.getDependentFields(childField.getId());

		for (int i = 0; i < depFields.size(); i++) {
			SS3Field f = depFields.get(i);
			addChild(child, f);
		}
	}

	private CodingSchemaFieldNode createFieldNode(SS3Field field) {
		LabelObject object = segment.getRootLabelObject().getGlossLabelObject();
		ArrayList<LabelObject> list = object.getAllChildren();
		ArrayList<String> available = new ArrayList<String>();
		ArrayList<String> visible = new ArrayList<String>();

		for (int i = 0; i < list.size(); i++) {
			LabelObject obj = list.get(i);
			FieldWrapper wrapper = obj.getWrapper();
			String id = wrapper.getField().getFieldID();

			if (obj.isVisible())
				visible.add(id);

			if (!available.contains(id) && wrapper.getField().getFieldSize() > 0)
				available.add(id);
		}

		String fieldID = "" + field.getId();
		boolean checked = visible.contains(fieldID) ? true : false;
		boolean data = available.contains(fieldID);
		String dataStr = data ? "*" : "";

		Object[] fieldNodeObj = new Object[] { field.getName(), field.getLabel(), checked, dataStr, Color.RED,
				field.getId() };
		return new CodingSchemaFieldNode(fieldNodeObj);
	}

	public void setSelectedField(String fieldID, boolean selected) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();
		
		if (fieldsRootNode.getChildAt(0) != null) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(0);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(5);

				if (nodeID.equals(fieldID)) {
					// this will either check or uncheck the field, then refresh the panel to
					// reflect the change
					childNode.setValueAt(selected, 2);
				}
			}
		}
		
	}

	public void scrollToAndSelectFieldByID(String fieldID) {
		if (fieldID == null || fieldID.isEmpty()) {
			scrollPane.getVerticalScrollBar().setValue(0);	// scroll to top
			return;
		}

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(5);

				if (nodeID.equals(fieldID)) {
					scrollToAndSelectNode(childNode);
				}
			}
		}
	}

	private void scrollToAndSelectNode(MutableTreeTableNode node) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeNode[] nodes = fieldsModel.getPathToRoot(node);
		TreePath path = new TreePath(nodes);

		fieldsTable.expandPath(path.getParentPath());
		int row = fieldsTable.getRowForPath(path);
		fieldsTable.scrollRowToVisible(row);
		fieldsTable.scrollPathToVisible(path);
		fieldsTable.setRowSelectionInterval(row, row);
	}

	public ArrayList<String> getSelectedFieldIDsNUpdateCodingSchema() {
		ArrayList<String> visibleFieldIDs = new ArrayList<String>();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				MutableTreeTableNode childNode = (MutableTreeTableNode) groupNode.getChildAt(i);
				Boolean display = (Boolean) childNode.getValueAt(2);

				if (display.booleanValue()) {
					visibleFieldIDs.add((String) childNode.getValueAt(5));
				}
			}
		}

		// always set gloss fields to visible
		getSelectedGlossFldIDs(4, visibleFieldIDs);

		int numberOfChildrenAfterGroupedNonManuals = fieldsRootNode.getChildCount();
		for (int counter = 5; counter < numberOfChildrenAfterGroupedNonManuals; counter++) {
			MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode.getChildAt(counter);
			Boolean display = (Boolean) node.getValueAt(2);
			if (display.booleanValue()) {
				visibleFieldIDs.add((String) node.getValueAt(5));
			}
		}
		return visibleFieldIDs;
	}

	private void getSelectedGlossFldIDs(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode.getChildAt(index);
		Object obj = node.getValueAt(2);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(5));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(2);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(5));
				}
			}
		}
	}

	public void discard() {

		fieldsTable.removeTreeSelectionListener(selectionListener);
		if (newFieldPanel != null) {
			newFieldPanel.discard();
		}
	}

	public Segment getSegment() {
		return segment;
	}
	
}

class TSelectionListener implements TreeSelectionListener {
	private GraphFieldPanel panel = null;
	private SS3CodingScheme scheme = null;

	public TSelectionListener(GraphFieldPanel graphFieldPanel, SS3CodingScheme scheme) {
		this.panel = graphFieldPanel;
		this.scheme = scheme;
	}

	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		MutableTreeTableNode selectedNode = (MutableTreeTableNode) tree.getLastSelectedPathComponent();

		panel.setSelectedTreeTableNode(selectedNode);

		if (selectedNode == null) {
			return;
		} else if (selectedNode.isLeaf()) {
			String fieldID = (String) selectedNode.getValueAt(5);
			SS3Field field = scheme.getField(fieldID);

			Color selectedColor = (Color) selectedNode.getValueAt(4);
			Segment segment = panel.getSegment();
			field.setColor(selectedColor);

			LabelObject object = segment.getRootLabelObject().getNonManualLabelObject();
			ArrayList<LabelObject> leafsOnly = object.getAllChildren();

			for (int i = 0; i < leafsOnly.size(); i++) {
				LabelObject obj = leafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

			LabelObject gObject = segment.getRootLabelObject().getGlossLabelObject();
			ArrayList<LabelObject> gLeafsOnly = gObject.getAllChildren();

			for (int i = 0; i < gLeafsOnly.size(); i++) {
				LabelObject obj = gLeafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

			if (selectedNode.isLeaf())
				selectedNode = (MutableTreeTableNode) selectedNode.getParent();
		}
	}
}

