package edu.bu.signstream.ui.panels.search;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.FieldsGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;

public class SearchRule extends SearchExpression implements ActionListener, DocumentListener {

	private static final long serialVersionUID = 1L;

	private SearchExpressionData data;
	private boolean validDataFlag;
	
	// GUI COMPONENTS
	
	private JButton negateRuleButton;
	private JPanel rulePanel;
	
	private JButton negateParticipantButton;
	private JComboBox<String> participantCB;
	
	private JButton negateFieldAndValueButton;
	private JPanel fieldAndValuePanel;
	
	private JButton negateFieldButton;
	private JComboBox<String> fieldCB;
	
	private JButton negateValueButton;
	private JComboBox<String> valueCB;
	private JTextField typedValue;
	private JCheckBox exactWord;
	private JCheckBox matchCase;
	
	private JLabel frameLabel;
	private JButton frameButton;
	
	// ACTION COMMANDS
	
	private final int REMOVE = 0;
	private final int NEGATE_RULE = 11;
	private final int NEGATE_PARTICIPANT = 12;
	private final int NEGATE_FIELD_VALUE = 13;
	private final int NEGATE_FIELD = 14;
	private final int NEGATE_VALUE = 15;
	private final int PARTICIPANT_SELECT = 20;
	private final int FIELD_SELECT = 30;
	private final int VALUE_SELECT = 40;
	private final int EXACT_WORD = 60;
	private final int MATCH_CASE = 70;
	private final int FRAME = 80;
	
	// LABELS, DIFFERENT CASES

	private final String anyParticipantLabel = "(Any Participant)";
	private final String anyFieldLabel = "(Any Field)";
	private final String anyValueLabel = "(Any Value)";
	private final String specifyLabel = "specify";

	private final String origRuleLabel = "";
	private final String negateRuleLabel = "NOT";
	private final String origParticipantLabel = "Participant";
	private final String negateParticipantLabel = "NOT Participant";
	private final String origFieldAndValueLabel = "";
	private final String negateFieldAndValueLabel = "NOT";
	private final String origFieldLabel = "Field";
	private final String negateFieldLabel = "NOT Field";
	private final String origValueLabel = "Value";
	private final String negateValueLabel = "NOT Value";
	private final String origExactWordLabel = "Exact Word";
	private final String origMatchCaseLabel = "Match Case";
	private final String origFrameLabel = "Frame";
	private final String firstFrameButtonLabel = "ANY";
	private final String secondFrameButtonLabel = "START";
	private final String thirdFrameButtonLabel = "END";

	static final String ANY_GLOSS_FIELD_NAME = "any gloss";
	static final String DOM_GLOSS_FIELD_NAME = "dom gloss";
	static final String NDOM_GLOSS_FIELD_NAME = "ndom gloss";

    static final String ANY_GES_FIELD_NAME = "any ges";
    static final String DOM_GES_FIELD_NAME = "dom ges";
    static final String NDOM_GES_FIELD_NAME = "ndom ges";

	// OTHER PARAMETERS
	
	private final int LIMIT_NOT_OPERATORS = 1;
	private int currentNotOperators;
	
	private ArrayList<SS3Participant> currentParticipants;
	private HashMap<String, SS3Field> currentFields;
	private HashMap<String, SS3FieldValue> currentValues;

	public SearchRule(SS3Database database, SearchGroup parentGroup) {
		super(database, parentGroup);
		
		this.currentNotOperators = 0;
		
		data = new SearchExpressionData();
		searchTreeNode = new SearchTreeNode(data, null);
		
		createUI();
	}
	
	@Override
	public void createUI() { 
		
		// color-coding
		
		java.awt.Color ruleColor = java.awt.Color.CYAN.darker();
		int rulePanelBorderWidth = 3;
		java.awt.Color fieldAndValueColor = java.awt.Color.MAGENTA.darker();
		int fieldAndValuePanelBorderWidth = 4;
		
		java.awt.Color parentColor = parentGroup.getBackground();
		setBackground(parentColor);
		
		// instantiate components

		removeButton.addActionListener(this);
		removeButton.setActionCommand("" + REMOVE);
		
		negateRuleButton = new JButton(origRuleLabel);
		negateRuleButton.setBackground(ruleColor);
		negateRuleButton.setOpaque(true);
		negateRuleButton.addActionListener(this);
		negateRuleButton.setActionCommand("" + NEGATE_RULE);		
		
		rulePanel = new JPanel();
		rulePanel.setBorder(BorderFactory.createLineBorder(ruleColor, rulePanelBorderWidth));
		
		negateParticipantButton = new JButton(origParticipantLabel);
		negateParticipantButton.addActionListener(this);
		negateParticipantButton.setActionCommand("" + NEGATE_PARTICIPANT);
		
		participantCB = new JComboBox<String>();
		resetParticipantCB(null);
		
		negateFieldAndValueButton = new JButton(origFieldAndValueLabel);
		negateFieldAndValueButton.setBackground(fieldAndValueColor);
		negateFieldAndValueButton.setOpaque(true);
		negateFieldAndValueButton.addActionListener(this);
		negateFieldAndValueButton.setActionCommand("" + NEGATE_FIELD_VALUE);
		
		fieldAndValuePanel = new JPanel();
		fieldAndValuePanel.setBorder(BorderFactory.createLineBorder(fieldAndValueColor, fieldAndValuePanelBorderWidth));
		
		negateFieldButton = new JButton(origFieldLabel);
		negateFieldButton.addActionListener(this);
		negateFieldButton.setActionCommand("" + NEGATE_FIELD);

		fieldCB = new JComboBox<String>();
		resetFieldCB(null);
		
		negateValueButton = new JButton(origValueLabel);
		negateValueButton.addActionListener(this);
		negateValueButton.setActionCommand("" + NEGATE_VALUE);

		valueCB = new JComboBox<String>();	
		resetValueCB(null, null);
				
		typedValue = new JTextField(15);
		typedValue.getDocument().addDocumentListener(this);
		typedValue.addKeyListener(new KeyAdapter() {
			@Override public void keyTyped(KeyEvent e) {
				if (anyValueLabel.equals(valueCB.getSelectedItem())) {
					valueCB.setSelectedItem(specifyLabel);
				}
			}
		});
	
		exactWord = new JCheckBox(origExactWordLabel);
		exactWord.addActionListener(this);
		exactWord.setActionCommand("" + EXACT_WORD);
		matchCase = new JCheckBox(origMatchCaseLabel);
		matchCase.addActionListener(this);
		matchCase.setActionCommand("" + MATCH_CASE);
		
		frameLabel = new JLabel(origFrameLabel);
		frameButton = new JButton(firstFrameButtonLabel);
		frameButton.addActionListener(this);
		frameButton.setActionCommand("" + FRAME);
		
		// setLayout, GridBagLayout, sub-panels, add all to this panel
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridy = 0;
		c.gridx = 0;
		c.anchor = GridBagConstraints.WEST;
		add(removeButton, c);
		
		c.gridx++;
		add(negateRuleButton, c);
		c.weightx = 1.0;

		rulePanel.setLayout(new GridBagLayout());
		GridBagConstraints c2 = new GridBagConstraints();
		
		c2.gridx = 0;
		c2.gridy = 0;
		c2.anchor = GridBagConstraints.WEST;
		rulePanel.add(negateParticipantButton, c2);
		
		c2.gridy = 1;
		rulePanel.add(participantCB, c2);
		
		c2.gridy = 0;				
		c2.gridx++;
		rulePanel.add(negateFieldAndValueButton, c2);
		
		fieldAndValuePanel.setLayout(new GridBagLayout());
		GridBagConstraints c3 = new GridBagConstraints();
		
		c3.gridx = 0;
		c3.gridy = 0;
		c3.anchor = GridBagConstraints.WEST;
		fieldAndValuePanel.add(negateFieldButton, c3);
		
		c3.gridy = 1;
		fieldAndValuePanel.add(fieldCB, c3);
		
		c3.gridy = 0;
		c3.gridx++;
		fieldAndValuePanel.add(negateValueButton, c3);
		
		c3.gridy = 1;
		fieldAndValuePanel.add(valueCB, c3);
		
		c3.gridx++;
		c3.gridwidth = 2;
		fieldAndValuePanel.add(typedValue, c3);
		
		c3.gridwidth = 1;
		c3.gridy = 0;
		fieldAndValuePanel.add(exactWord, c3);
		
		c3.gridx++;
		fieldAndValuePanel.add(matchCase, c3);
		
		c3.gridy = 0;
		c3.gridx++;
		fieldAndValuePanel.add(frameLabel, c3);
		
		c3.gridy = 1;
		fieldAndValuePanel.add(frameButton, c3);
		
		c2.gridheight = 2;
		c2.gridy = 0;
		c2.gridx++;
		rulePanel.add(fieldAndValuePanel, c2);

		c.gridheight = 2;
		c.gridy = 0;
		c.gridx++;
//		c.fill = GridBagConstraints.BOTH;
		add(rulePanel, c);
		
		// disable non-default rule components (value text & frame button)
		
		enableTextValueComponents(false);
		enableFrameComponents(false);		
		
		// set default data points in the data object (already a member of this rule's node)
		
		setDefaultDataPoints();
	}

	@Override
	public void loadSavedSearchTreeNode(SearchTreeNode savedSearchTreeNode) {
		
		if ( savedSearchTreeNode == null || !savedSearchTreeNode.isRuleNode())
			return;
		
		searchTreeNode = savedSearchTreeNode;
		data = savedSearchTreeNode.getData();
		boolean invalidValue = false;
		
		boolean negateRule = data.isNegateRule();
		boolean negateParticipant = data.isNegateParticipant();
		boolean negateFieldAndValue = data.isNegateFieldAndValue();
		boolean negateField = data.isNegateField();
		boolean negateValue = data.isNegateValue();
		
		if (negateRule) {
			negateRuleButton.setSelected(negateRule);
			negateRuleButton.doClick();
		}
		if (negateParticipant) {
			negateParticipantButton.setSelected(negateParticipant);
			negateParticipantButton.doClick();
		}
		if (negateFieldAndValue) {
			negateFieldAndValueButton.setSelected(negateFieldAndValue);
			negateFieldAndValueButton.doClick();
		}
		if (negateField) {
			negateFieldButton.setSelected(negateField);
			negateFieldButton.doClick();
		}
		if (negateValue) {
			negateValueButton.setSelected(negateValue);
			negateValueButton.doClick();
		}

		String participantName = data.getParticipantName();
		String fieldName = data.getFieldName();
		String valueName = data.getValueName();
		String valueText = data.getValueText();
		boolean exactWordFlag = data.isExactWord();
		boolean matchCaseFlag = data.isMatchCase();
		String frameSearchBy = data.getFrameSearchBy();
		
		if (participantName != null) {
			participantCB.setSelectedItem(participantName);
			if ( !participantName.equals("any") && participantCB.getSelectedItem().equals("any")) {
				participantCB.addItem(participantName);
				participantCB.setSelectedItem(participantName);
				invalidValue = true;
			}
		}
		
		if (fieldName != null) {
			fieldCB.setSelectedItem(fieldName);
			if ( !fieldName.equals("any") && fieldCB.getSelectedItem().equals("any")) {
				fieldCB.addItem(fieldName);
				fieldCB.setSelectedItem(fieldName);
				invalidValue = true;
			}
		}
		
		if (valueName != null) {
			valueCB.setSelectedItem(valueName);
			if ( !valueName.equals("any") && valueCB.getSelectedItem().equals("any")) {
				valueCB.addItem(valueName);
				valueCB.setSelectedItem(valueName);
				invalidValue = true;
			} else if (valueCB.getSelectedItem().equals("specify")) {
				enableTextValueComponents(true);
				typedValue.setText(valueText);
				exactWord.setSelected(exactWordFlag);
				matchCase.setSelected(matchCaseFlag);
			}
		}
		
		if (frameSearchBy.equals(firstFrameButtonLabel))
			frameButton.setText(firstFrameButtonLabel);
		else if (frameSearchBy.equals(secondFrameButtonLabel))
			frameButton.setText(secondFrameButtonLabel);
		else if (frameSearchBy.equals(thirdFrameButtonLabel))
			frameButton.setText(thirdFrameButtonLabel);
		
		if (invalidValue)
			disableAllComponents();
		
	}
	
	private void resetParticipantCB(String selectedParticipantName) {
		participantCB.removeActionListener(this);
		
		participantCB.removeAllItems();
		currentParticipants = database.getSelectedParticipants();
		for (SS3Participant participant: currentParticipants) {
			String participantName = participant.getFullName();
			participantCB.addItem(participantName);
		}
		participantCB.addItem(anyParticipantLabel);

		if (selectedParticipantName != null) {
			participantCB.setSelectedItem(selectedParticipantName);
		} else {
			participantCB.setSelectedItem(anyParticipantLabel);
		}
		
		participantCB.addActionListener(this);
		participantCB.setActionCommand("" + PARTICIPANT_SELECT);
	}
	
	private void resetFieldCB(String selectedFieldName) {
		fieldCB.removeActionListener(this);
		
		fieldCB.removeAllItems();
		
		SS3CodingScheme dbScheme = database.getSS3CodingScheme();
		
		currentFields = dbScheme.getFields();
		currentFields.remove("40001");
		currentFields.remove("40002");
		// above are graph fields "eye brows" and "eye aperture" as of 2020-05-19 -- this is a hack until a better solution (GD)
				
		ArrayList<FieldsGroup> fieldsGroups = dbScheme.loadNonManualGroups();
		for (FieldsGroup group: fieldsGroups) {
			ArrayList<String> fieldIDs = group.getFieldIDs();
			for (String fieldID: fieldIDs) {
				SS3Field field = currentFields.get(fieldID);
				String fieldName = field.getName();
				fieldCB.addItem(fieldName);
			}
		}
		
		FieldsGroup customFieldsGroup = dbScheme.loadCustomFieldGroups().get(0);
		ArrayList<String> fieldIDs = customFieldsGroup.getFieldIDs();
		for (String fieldID: fieldIDs) {
			SS3Field field = currentFields.get(fieldID);
			String fieldName = field.getName();
			fieldCB.addItem(fieldName);
		}

		fieldCB.addItem(ANY_GLOSS_FIELD_NAME);

		boolean allSameHand = true;
		if (participantCB.getSelectedItem().equals("any")) {
			ArrayList<SS3Participant> participants = database.getSelectedParticipants();
			String hand = participants.get(0).getDominantHand();
			for (int i = 1; i < participants.size(); i++) {
				if (!hand.equals(participants.get(i).getDominantHand())) {
					allSameHand = false;
					break;
				}
			}
		}

		if (allSameHand) {
			fieldCB.addItem(DOM_GLOSS_FIELD_NAME);
			fieldCB.addItem(NDOM_GLOSS_FIELD_NAME);
		}

		fieldCB.addItem(ANY_GES_FIELD_NAME);

		if (allSameHand) {
            fieldCB.addItem(DOM_GES_FIELD_NAME);
            fieldCB.addItem(NDOM_GES_FIELD_NAME);
        }
		
		fieldCB.addItem(currentFields.get(Util.FREE_TEXT_FIELD_ID).getName());
		
		fieldCB.addItem(anyFieldLabel);
		
		String toSelect = selectedFieldName != null ? selectedFieldName : anyFieldLabel;
		fieldCB.setSelectedItem(toSelect);
		
		fieldCB.addActionListener(this);
		fieldCB.setActionCommand("" + FIELD_SELECT);
	}
	
	private void resetValueCB(SS3Field selectedField, String selectedValueName) {
		valueCB.removeActionListener(this);
		
		valueCB.removeAllItems();			
		if (selectedField != null) {
			currentValues = selectedField.getValues();
			for (SS3FieldValue value: currentValues.values()) {
				String valueName = value.getName();
				valueCB.addItem(valueName);
			}
		} else {
			currentValues = null;
		}
		valueCB.addItem(specifyLabel);
		valueCB.addItem(anyValueLabel);

		String toSelect = selectedValueName != null ? selectedValueName : anyValueLabel;
		valueCB.setSelectedItem(toSelect);

		valueCB.addActionListener(this);
		valueCB.setActionCommand("" + VALUE_SELECT);
	}
	
	private void setDefaultDataPoints() {
		
		validDataFlag = true;
		
		data.setValidDataFlag(validDataFlag);
		
		data.setNegateRuleFlag(false);
		data.setNegateParticipantFlag(false);
		data.setNegateFieldAndValueFlag(false);
		data.setNegateFieldFlag(false);
		data.setNegateValueFlag(false);
		
		data.setParticipantID(null);
		data.setAnyParticipant(true);
		data.setFieldID(null);
		data.setAnyField(true);
		data.setValueID(null);
		data.setAnyValue(true);
		
		data.setValueText("");
		data.setExactWord(false);
		data.setMatchCase(false);
		
		data.setFrameSearchBy(firstFrameButtonLabel);
	}

	public void updateSearchRuleAndTreeNode() {
		String selectedParticipant = (String) participantCB.getSelectedItem();

		String selectedFieldName = (String) fieldCB.getSelectedItem();

		String selectedValueName = (String) valueCB.getSelectedItem();
		
		boolean found = true;
		if (!anyParticipantLabel.equals(selectedParticipant)) {
			found = false;
			for (SS3Participant participant: database.getSelectedParticipants()) {
				if (selectedParticipant.equals(participant.getFullName())) {
					found = true;
					break;
				}
			}
		}
		
		if (found) {
			resetParticipantCB(selectedParticipant);
		} else {
			disableAllComponents();
			return;
		}
		
		found = true;
		SS3Field selectedField = null;
		if (!anyFieldLabel.equals(selectedFieldName)) {
			found = false;
			for (SS3Field field: database.getSS3CodingScheme().getFields().values()) {
				if (selectedFieldName.equals(field.getName())) {
					found = true;
					selectedField = field;
					break;
				}
			}
		}
		
		if (found) {
			resetFieldCB(selectedFieldName);
		} else {
			disableAllComponents();
			return;
		}
		
		found = true;
		if (!anyValueLabel.equals(selectedValueName) && !specifyLabel.equals(selectedValueName)) {
			found = false;
			if (selectedField != null) {
				for (SS3FieldValue value: selectedField.getValuesAsList()) {
					String valueName = value.getName();
					if (selectedValueName.equals(valueName)) {
						found = true;
						break;
					}
				}
			}
		}
		
		if (found) {
			resetValueCB(selectedField, selectedValueName);
		} else {
			disableAllComponents();
			return;
		}
		
		enableDefaultComponents(true);
		
		if (selectedValueName.equals(specifyLabel))
			enableTextValueComponents(true);
		
		if (parentGroup instanceof BinarySearchGroup)
			enableFrameComponents(true);
		
		revalidate();
		repaint();
	}
	
	public void enableDefaultComponents(boolean enable) {
		data.setValidDataFlag(enable);
		validDataFlag = enable;
		
		negateRuleButton.setEnabled(enable);
		negateParticipantButton.setEnabled(enable);
		participantCB.setEnabled(enable);
		negateFieldAndValueButton.setEnabled(enable);
		negateFieldButton.setEnabled(enable);
		fieldCB.setEnabled(enable);
		negateValueButton.setEnabled(enable);
		valueCB.setEnabled(enable);
	}
	
	public void disableAllComponents() {
		enableDefaultComponents(false);
		enableTextValueComponents(false);
		enableFrameComponents(false);
		
		revalidate();
		repaint();
	}
	
	public void enableRemoveButton(boolean enable) {
		removeButton.setEnabled(enable);
	}
	
	public void enableNegateRuleButton(boolean enable) {
		negateRuleButton.setEnabled(enable);
	}
	
	public void enableNegateFieldAndValueButton(boolean enable) {
		negateFieldAndValueButton.setEnabled(enable);
	}
	
	public void enableTextValueComponents(boolean enable) {
		exactWord.setEnabled(enable);
		matchCase.setEnabled(enable);		
	}
	
	public void enableFrameComponents(boolean enable) {
		frameLabel.setEnabled(enable);
		frameButton.setEnabled(enable);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		
		switch(Integer.parseInt(command)) {
		case REMOVE:
			parentGroup.getSearchTreeNode().remove(searchTreeNode);
			parentGroup.removeExpression(this);
			break;
		case NEGATE_RULE:
			String text = negateRuleButton.getText();
			if (text.equals("")) {
				if (currentNotOperators >= LIMIT_NOT_OPERATORS)
					return;
				
				negateRuleButton.setText(negateRuleLabel);
				currentNotOperators++;
				data.setNegateRuleFlag(true);
			} else {
				if (currentNotOperators < LIMIT_NOT_OPERATORS)
					return;
				
				negateRuleButton.setText(origRuleLabel);
				currentNotOperators--;
				data.setNegateRuleFlag(false);
			}
			break;
		case NEGATE_PARTICIPANT:
			text = negateParticipantButton.getText();
			if (text.equals(origParticipantLabel)) {
				if (currentNotOperators >= LIMIT_NOT_OPERATORS)
					return;
				
				negateParticipantButton.setText(negateParticipantLabel);
				currentNotOperators++;
				data.setNegateParticipantFlag(true);
			} else {
				if (currentNotOperators < LIMIT_NOT_OPERATORS)
					return;
				
				negateParticipantButton.setText(origParticipantLabel);
				currentNotOperators--;
				data.setNegateParticipantFlag(false);
			}
			break;
		case NEGATE_FIELD_VALUE:
			text = negateFieldAndValueButton.getText();
			if (text.equals("")) {
				if (currentNotOperators >= LIMIT_NOT_OPERATORS)
					return;
				
				negateFieldAndValueButton.setText(negateFieldAndValueLabel);
				currentNotOperators++;
				data.setNegateFieldAndValueFlag(true);
			} else {
				if (currentNotOperators < LIMIT_NOT_OPERATORS)
					return;
				
				negateFieldAndValueButton.setText(origFieldAndValueLabel);
				currentNotOperators--;
				data.setNegateFieldAndValueFlag(false);
			}
			break;
		case NEGATE_FIELD:
			text = negateFieldButton.getText();
			if (text.equals(origFieldLabel)) {
				if (currentNotOperators >= LIMIT_NOT_OPERATORS)
					return;
				
				negateFieldButton.setText(negateFieldLabel);
				currentNotOperators++;
				data.setNegateFieldFlag(true);
			} else {
				if (currentNotOperators < LIMIT_NOT_OPERATORS)
					return;
				
				negateFieldButton.setText(origFieldLabel);
				currentNotOperators--;
				data.setNegateFieldFlag(false);
			}
			break;
		case NEGATE_VALUE:
			text = negateValueButton.getText();
			if (text.equals(origValueLabel)) {
				if (currentNotOperators >= LIMIT_NOT_OPERATORS)
					return;
				
				negateValueButton.setText(negateValueLabel);
				currentNotOperators++;
				data.setNegateValueFlag(true);
			} else {
				if (currentNotOperators < LIMIT_NOT_OPERATORS)
					return;
				
				negateValueButton.setText(origValueLabel);
				currentNotOperators--;
				data.setNegateValueFlag(false);
			}
			break;
		case PARTICIPANT_SELECT:
			String participantName = (String) participantCB.getSelectedItem();
			
			SS3Participant selectedParticipant = null;
			for (SS3Participant participant: currentParticipants) {
				if (participantName.equals(participant.getFullName())) {
					selectedParticipant = participant;
					break;
				}
			}
			
			if (selectedParticipant != null) {
				data.setParticipantID(selectedParticipant.getId());
				data.setParticipantName(selectedParticipant.getFullName());
				data.setAnyParticipant(false);
			} else {
				data.setParticipantID(null);
				data.setAnyParticipant(true);
			}

			resetFieldCB(null);
			
			break;
		case FIELD_SELECT:
			String fieldName = (String) fieldCB.getSelectedItem();

			valueCB.removeActionListener(this);
			valueCB.removeAllItems();

			if (fieldName.equals(ANY_GLOSS_FIELD_NAME)
                || fieldName.equals(DOM_GLOSS_FIELD_NAME)
                || fieldName.equals(NDOM_GLOSS_FIELD_NAME)
                || fieldName.equals(ANY_GES_FIELD_NAME)
                || fieldName.equals(DOM_GES_FIELD_NAME)
                || fieldName.equals(NDOM_GES_FIELD_NAME)) {
				data.setFieldName(fieldName);
			} else {

				SS3Field selectedField = null;
				for (SS3Field field : currentFields.values()) {
					if (fieldName.equals(field.getName())) {
						selectedField = field;
						break;
					}
				}

				if (selectedField != null) {
					data.setAnyField(false);
					data.setFieldID(selectedField.getId());
					data.setFieldName(selectedField.getName());

					currentValues = selectedField.getValues();
					for (SS3FieldValue value : currentValues.values()) {
						valueCB.addItem(value.getName());
					}

					data.setValueText("");
					data.setAnyValue(true);
				} else {
					data.setFieldID(null);
					data.setAnyField(true);

					data.setValueText("");
					data.setAnyValue(true);
				}
			}

			valueCB.addItem(anyValueLabel);
			valueCB.addItem(specifyLabel);
			
			valueCB.addActionListener(this);		
			valueCB.setActionCommand("" + VALUE_SELECT);
			
			valueCB.setSelectedItem(anyValueLabel);
			
			break;
		case VALUE_SELECT:
			String valueName = (String) valueCB.getSelectedItem();
			
			if (specifyLabel.equals(valueName)) {
				enableTextValueComponents(true);
			} else {
				exactWord.setSelected(false);
				matchCase.setSelected(false);
				typedValue.setText("");
				enableTextValueComponents(false);
			}
			
//			SS3FieldValue selectedValue = null;
//			if (currentValues != null) {
//				for (SS3FieldValue value: currentValues.values()) {
//					if (valueName.equals(value.getName())) {
//						selectedValue = value;
//						break;
//					}
//				}
//			}
			
			if (!valueName.equals(anyValueLabel)) {
				data.setAnyValue(false);
				data.setValueText("");
				data.setValueName(valueName);
				// not using getName() because that is only for human-readable purposes
				// getLabel() is what is actually saved in each SS3Item text, to match in the search engine
			} else {
				data.setValueText("");
				data.setAnyValue(true);
			}
			
			break;
		case EXACT_WORD:
			boolean isSelected = exactWord.isSelected();
			data.setExactWord(isSelected);
			break;
		case MATCH_CASE:
			isSelected = matchCase.isSelected();
			data.setMatchCase(isSelected);
			break;
		case FRAME:
			text = frameButton.getText();
			if (text.equals(firstFrameButtonLabel)) {
				frameButton.setText(secondFrameButtonLabel);
				data.setFrameSearchBy(secondFrameButtonLabel);
			} else if (text.equals(secondFrameButtonLabel)) {
				frameButton.setText(thirdFrameButtonLabel);
				data.setFrameSearchBy(thirdFrameButtonLabel);
			} else {
				frameButton.setText(firstFrameButtonLabel);
				data.setFrameSearchBy(firstFrameButtonLabel);
			}
			break;
		}
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		handleTypedValueEdit();
	}
	@Override
	public void removeUpdate(DocumentEvent e) {
		handleTypedValueEdit();
	}
	@Override
	public void changedUpdate(DocumentEvent e) {
		handleTypedValueEdit();
	}	
	private void handleTypedValueEdit() {
		String text = typedValue.getText();
		data.setValueText(text);
	}
}
