package edu.bu.signstream.ui.panels.signbank;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.logging.Level;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;

import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletDialog;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankResultNode;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SignBankLookupPanel extends JPanel implements ActionListener {
	
	private static final long serialVersionUID = 1L;
	
	private SignBankPanel parent = null;
	private AssociatedFieldsPanel assocFieldsPanel = null;
	
	private JLabel signLabel = null;
	private JTextField signLabelTextField = null;
	
	private JButton clearButton = null;
	private JCheckBox exactWordCheck = null;
	private JButton lookupButton = null;
	
	private JRadioButton oneHandedRadio = null;
	private JRadioButton twoHandedRadio = null;
	private JRadioButton eitherRadio = null;
	
	private JCheckBox includeHS = null;
	
	private JLabel startHSLabel = null;
	private JLabel endHSLabel = null;
	private JLabel domStartHSLabel = null;
	private JLabel nondomStartHSLabel = null;
	private JLabel domEndHSLabel = null;
	private JLabel nondomEndHSLabel = null;
	
	private HandShapePanel domStartHSPanel = null;
	private HandShapePanel nondomStartHSPanel = null;
	private HandShapePanel domEndHSPanel = null;
	private HandShapePanel nondomEndHSPanel = null;
	HandShapePalletDialog palletDialog = null;
	
	private BufferedImage domStartHSImage = null;
	private BufferedImage nondomStartHSImage = null;
	private BufferedImage domEndHSImage = null;
	private BufferedImage nondomEndHSImage = null;
	
	private String domStartHSImageLabel = null;
	private String nondomStartHSImageLabel = null;
	private String domEndHSImageLabel = null;
	private String nondomEndHSImageLabel = null;
	
	private JButton domStartHSClearButton = null;
	private JButton nondomStartHSClearButton = null;
	private JButton domEndHSClearButton = null;
	private JButton nondomEndHSClearButton = null;
	
	private SignBankCollection signBankCollection = null;
	
	private final String dominantString = "Dom";
	private final String nondominantString = "Non-Dom";
	
	private final int CLEAR_SIGN_LABEL = 0;
	private final int LOOKUP = 10;
	private final int ONE_HANDED = 30;
	private final int TWO_HANDED = 40;
	private final int EITHER = 41;
	private final int CLEAR_DOM_START_HS = 50;
	private final int CLEAR_NONDOM_START_HS = 60;
	private final int CLEAR_DOM_END_HS = 70;
	private final int CLEAR_NONDOM_END_HS = 80;

	public SignBankLookupPanel(AssociatedFieldsPanel assocFieldsPanel, String variantLabel) { // No UI option to secretly perform search
		this.assocFieldsPanel = assocFieldsPanel;
	}

	public SignBankLookupPanel(SignBankPanel parent, AssociatedFieldsPanel assocFieldsPanel, String variantLabel) {
		this.parent = parent;
		this.assocFieldsPanel = assocFieldsPanel;
		
		createUI(variantLabel);
	}
	
	private void createUI(String variantLabel) {

	    SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
	    		.getSS3Database().getSS3CodingScheme();
		
		ArrayList<HandShapeImage> domImages, nondomImages;
		if (assocFieldsPanel.isDominantHandRight()) {
			domImages = assocFieldsPanel.getHandShapeFormBean().getSelectedDomHandShapeImages();
			nondomImages = assocFieldsPanel.getHandShapeFormBean().getSelectedNdomHandShapeImages();
		} else {
			domImages = assocFieldsPanel.getHandShapeFormBean().getSelectedNdomHandShapeImages();
			nondomImages = assocFieldsPanel.getHandShapeFormBean().getSelectedDomHandShapeImages();
		}
		int domImagesSize = domImages.size();
		int nondomImagesSize = nondomImages.size();
		
		if (domImagesSize > 0) {
			SS3FieldValue domStartFieldValue = domImages.get(0).getSS3FieldValue();
			domStartHSImageLabel = domStartFieldValue.getLabel();
			domStartHSImage = (BufferedImage) domImages.get(0).getImage();
			
			SS3FieldValue domEndFieldValue = domImages.get(domImagesSize - 1).getSS3FieldValue(); 
			domEndHSImageLabel = domEndFieldValue.getLabel();
			domEndHSImage = (BufferedImage) domImages.get(domImagesSize - 1).getImage();
		}
		
		if (nondomImagesSize > 0) {
			SS3FieldValue nondomStartFieldValue = nondomImages.get(0).getSS3FieldValue();
			nondomStartHSImageLabel = nondomStartFieldValue.getLabel();
			nondomStartHSImage = (BufferedImage) nondomImages.get(0).getImage();
			
			SS3FieldValue nondomEndFieldValue = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue();
			nondomEndHSImageLabel = nondomEndFieldValue.getLabel();
			nondomEndHSImage = (BufferedImage) nondomImages.get(nondomImagesSize - 1).getImage();
		}

		
		signLabel = new JLabel("Search: ");
		signLabelTextField = new JTextField(15);
		signLabelTextField.setText(variantLabel);		
		
		clearButton = new JButton("Clear");
		clearButton.addActionListener(this);
		clearButton.setActionCommand("" + CLEAR_SIGN_LABEL);
		
		exactWordCheck = new JCheckBox("Search Must Match Exact Word");
		
		lookupButton = new JButton("Lookup");
		lookupButton.addActionListener(this);
		lookupButton.setActionCommand("" + LOOKUP);
		
		oneHandedRadio = new JRadioButton("1-Handed");
		oneHandedRadio.addActionListener(this);
		oneHandedRadio.setActionCommand("" + ONE_HANDED);
		
		twoHandedRadio = new JRadioButton("2-Handed");
		twoHandedRadio.addActionListener(this);
		twoHandedRadio.setActionCommand("" + TWO_HANDED);
		
		eitherRadio = new JRadioButton("Either");
		eitherRadio.addActionListener(this);
		eitherRadio.setActionCommand("" + EITHER);

		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(oneHandedRadio);
		buttonGroup.add(twoHandedRadio);
		buttonGroup.add(eitherRadio);
		
		includeHS = new JCheckBox("Include Handshapes in Search");
		if (domStartHSImage != null || domEndHSImage != null || nondomStartHSImage != null || nondomEndHSImage != null) {
			setSelectedHsCheckbox(true);
		}
		
		startHSLabel = new JLabel("Start HS");
		endHSLabel = new JLabel("End HS");
		
		domStartHSLabel = new JLabel(dominantString);
		nondomStartHSLabel = new JLabel(nondominantString);
		domEndHSLabel = new JLabel(dominantString);
		nondomEndHSLabel = new JLabel(nondominantString);
		
		domStartHSPanel = new HandShapePanel(dbScheme, Util.DOM_HANDSHAPE_FIELD_ID, domStartHSImage, domStartHSImageLabel);
		nondomStartHSPanel = new HandShapePanel(dbScheme, Util.NDOM_HANDSHAPE_FIELD_ID, nondomStartHSImage, nondomStartHSImageLabel);
		domEndHSPanel = new HandShapePanel(dbScheme, Util.DOM_HANDSHAPE_FIELD_ID, domEndHSImage, domEndHSImageLabel);
		nondomEndHSPanel = new HandShapePanel(dbScheme, Util.NDOM_HANDSHAPE_FIELD_ID, nondomEndHSImage, nondomEndHSImageLabel);

		String clear = "Clear";
		domStartHSClearButton = new JButton(clear);
		domStartHSClearButton.addActionListener(this);
		domStartHSClearButton.setActionCommand("" + CLEAR_DOM_START_HS);
		
		nondomStartHSClearButton = new JButton(clear);
		nondomStartHSClearButton.addActionListener(this);
		nondomStartHSClearButton.setActionCommand("" + CLEAR_NONDOM_START_HS);
		
		domEndHSClearButton = new JButton(clear);
		domEndHSClearButton.addActionListener(this);
		domEndHSClearButton.setActionCommand("" + CLEAR_DOM_END_HS);
		
		nondomEndHSClearButton = new JButton(clear);
		nondomEndHSClearButton.addActionListener(this);
		nondomEndHSClearButton.setActionCommand("" + CLEAR_NONDOM_END_HS);
		
		JPanel firstRow = new JPanel();
		firstRow.add(signLabel);
		firstRow.add(signLabelTextField);
		firstRow.add(clearButton);
		
		JPanel secondRow = new JPanel();
		secondRow.setLayout(new FlowLayout(FlowLayout.RIGHT));
		secondRow.add(exactWordCheck);
		secondRow.add(lookupButton);
		
		JPanel thirdRow = new JPanel();
		thirdRow.add(oneHandedRadio);
		thirdRow.add(twoHandedRadio);
		thirdRow.add(eitherRadio);
		
		JPanel fourthRow = new JPanel();
		fourthRow.add(includeHS);
		
		JPanel hsPanelRow = new JPanel();
		JPanel startHSPanel = new JPanel();
		JPanel endHSPanel = new JPanel();
		
		startHSPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridwidth = 2;
		
		c.gridx = 0;
		c.gridy = 0;
		startHSPanel.add(startHSLabel, c);
		
		c.gridwidth = 1;
		
		c.gridy++;
		c.gridx = 0;
		startHSPanel.add(domStartHSLabel, c);
		c.gridx = 1;
		startHSPanel.add(nondomStartHSLabel, c);

		c.gridy++;
		c.gridx = 0;
		startHSPanel.add(domStartHSPanel, c);
		c.gridx = 1;
		startHSPanel.add(nondomStartHSPanel, c);
		
		c.gridy++;
		c.gridx = 0;
		startHSPanel.add(domStartHSClearButton, c);
		c.gridx = 1;
		startHSPanel.add(nondomStartHSClearButton, c);
		
		
		endHSPanel.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		
		c.gridwidth = 2;
		
		c.gridx = 0;
		c.gridy = 0;
		endHSPanel.add(endHSLabel, c);
		
		c.gridwidth = 1;
		
		c.gridy++;
		c.gridx = 0;
		endHSPanel.add(domEndHSLabel, c);
		c.gridx = 1;
		endHSPanel.add(nondomEndHSLabel, c);

		c.gridy++;
		c.gridx = 0;
		endHSPanel.add(domEndHSPanel, c);
		c.gridx = 1;
		endHSPanel.add(nondomEndHSPanel, c);
		
		c.gridy++;
		c.gridx = 0;
		endHSPanel.add(domEndHSClearButton, c);
		c.gridx = 1;
		endHSPanel.add(nondomEndHSClearButton, c);
		
		hsPanelRow.setLayout(new GridLayout(1, 2));
		hsPanelRow.add(startHSPanel);
		hsPanelRow.add(endHSPanel);
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(firstRow);
		add(secondRow);
		add(thirdRow);
		add(fourthRow);
		add(hsPanelRow);
		
		// "Either" one handed or two handed is the default sign bank search parameter
		eitherRadio.doClick();
	}
	
	public void setSignBankCollection(SignBankCollection collection) {
		this.signBankCollection = collection;
	}
	
	public SignBankCollection getSignBankCollection() {
		return signBankCollection;
	}

	private void lookup() {
		
		String signLabelText = signLabelTextField.getText();
		boolean exactWord = exactWordCheck.isSelected();
		
		boolean searchHS = includeHS.isSelected();
		
		boolean oneHanded = oneHandedRadio.isSelected();
		boolean twoHanded = twoHandedRadio.isSelected();
		boolean eitherHanded = eitherRadio.isSelected();
		
		// interpret "either" as search oneHanded and also search twoHanded, pass both to lookup
		if (eitherHanded) {
			oneHanded = true;
			twoHanded = true;
		}

		// get labels from the selected hs images
		domStartHSImageLabel = domStartHSPanel.getHSImageLabel();
		nondomStartHSImageLabel = nondomStartHSPanel.getHSImageLabel();
		domEndHSImageLabel = domEndHSPanel.getHSImageLabel();
		nondomEndHSImageLabel = nondomEndHSPanel.getHSImageLabel();
		
		// validate entry before beginning lookup
		if (signLabelText == null) {
			return;
		}
		signLabelText = signLabelText.trim();
		
		if (!exactWord) {
			signLabelText = Util.trimAsterisksAndPlusSigns(signLabelText);
		}
		
		// call lookup function in the collection object, get back a list of resulting SignBankOccurrence's
		ArrayList<SignBankOccurrence> resultOccurrences = 
				signBankCollection.lookup(signLabelText, exactWord, searchHS, oneHanded, twoHanded, 
				domStartHSImageLabel, nondomStartHSImageLabel, domEndHSImageLabel, nondomEndHSImageLabel);

		SignBankMainGloss previousMainGloss = null, currentMainGloss = null;
		
		ArrayList<DefaultMutableTreeNode> mainGlossNodes = new ArrayList<DefaultMutableTreeNode>();
		
		DefaultMutableTreeNode mainGlossNode = null;
		SignBankResultNode occurrenceNode = null;
		
		for (SignBankOccurrence occurrence: resultOccurrences) {
			currentMainGloss = occurrence.getParentVariant().getParentMainGloss();
			
			// if we have moved on to a different parent gloss...
			if (previousMainGloss == null || !currentMainGloss.equals(previousMainGloss)) {
				
				// start over with a new main gloss node, and add it immediately to the list
				mainGlossNode = new DefaultMutableTreeNode(currentMainGloss.getMainGlossLabel());
				mainGlossNodes.add(mainGlossNode);
			}
			
			// add a new SignBankResultNode on each pass
			occurrenceNode = new SignBankResultNode(occurrence);
			mainGlossNode.add(occurrenceNode);
			
			previousMainGloss = currentMainGloss;
		}
		
		// sort the list of main gloss nodes alphabetically by label
		Collections.sort(mainGlossNodes, new Comparator<DefaultMutableTreeNode>() {
			@Override
			public int compare(DefaultMutableTreeNode o1, DefaultMutableTreeNode o2) {
				DefaultMutableTreeNode node1 = o1;
				DefaultMutableTreeNode node2 = o2;
				
				String label1 = (String) node1.getUserObject();
				String lowerLabel1 = label1.toLowerCase();
				String label2 = (String) node2.getUserObject();
				String lowerLabel2 = label2.toLowerCase();
				
				return lowerLabel1.compareTo(lowerLabel2);
			}
		});
		
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode();
		for (DefaultMutableTreeNode node: mainGlossNodes) {
			rootNode.add(node);
		}
		
		parent.updateResultsListPanel(rootNode);
		parent.setVariantPanelVisible(false);
		SwingUtilities.getWindowAncestor(this).pack();
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case CLEAR_SIGN_LABEL:
			signLabelTextField.setText(null);
			signLabelTextField.requestFocus();
			parent.setResultsPanelToNull();
			break;
		
		case LOOKUP:
			parent.setResultsPanelToNull();
			lookup();
			break;

		case ONE_HANDED:
			nondomStartHSPanel.setVisible(false);
			nondomStartHSClearButton.setVisible(false);
			nondomStartHSLabel.setVisible(false);
			
			domStartHSLabel.setVisible(false);

			nondomStartHSPanel.removeAll();
			
			nondomEndHSPanel.setVisible(false);
			nondomEndHSClearButton.setVisible(false);
			nondomEndHSLabel.setVisible(false);
			
			domEndHSLabel.setVisible(false);
			
			nondomEndHSPanel.removeAll();
			
			if (SwingUtilities.getWindowAncestor(parent) != null)
				SwingUtilities.getWindowAncestor(parent).pack();
			break;
			
		case TWO_HANDED:
		case EITHER:
			nondomStartHSPanel.setVisible(true);
			nondomStartHSClearButton.setVisible(true);
			nondomStartHSLabel.setVisible(true);
			
			domStartHSLabel.setVisible(true);
			
			nondomEndHSPanel.setVisible(true);
			nondomEndHSClearButton.setVisible(true);
			nondomEndHSLabel.setVisible(true);
			
			domEndHSLabel.setVisible(true);

			if (SwingUtilities.getWindowAncestor(parent) != null)
				SwingUtilities.getWindowAncestor(parent).pack();	
			break;
			
		case CLEAR_DOM_START_HS:
			domStartHSPanel.removeAll();
			break;
			
		case CLEAR_NONDOM_START_HS:
			nondomStartHSPanel.removeAll();
			break;
			
		case CLEAR_DOM_END_HS:
			domEndHSPanel.removeAll();
			break;
			
		case CLEAR_NONDOM_END_HS:
			nondomEndHSPanel.removeAll();
			break;
		}
		
		revalidate();
		repaint();
	}
	
	public void setSelectedHsCheckbox(boolean b){
		includeHS.setSelected(b);
	}
	
	private class HandShapePanel extends JPanel implements MouseListener, HandShapePalletClient {
		private static final long serialVersionUID = 1L;
		
		SS3CodingScheme codingScheme = null;
		String fieldID = null;
		String hsImageLabel = null;
		BufferedImage bufImage = null;
		
		public HandShapePanel(SS3CodingScheme codingScheme, String fieldID, BufferedImage image, String label) {
			this.codingScheme = codingScheme;
			this.fieldID = fieldID;
			this.bufImage = image;
			this.hsImageLabel = label;
			
			addMouseListener(this);
			
			if (image != null)
				add(new JLabel(new ImageIcon(image)));
			
			int size = 50;
			setSize(new Dimension(size, size));
			setMaximumSize(new Dimension(size, size));
			setMinimumSize(new Dimension(size, size));
			setPreferredSize(new Dimension(size, size));			
			setBorder(BorderFactory.createLineBorder(Color.BLACK));
		}
		
		public String getFieldID() {
			return fieldID;
		}
		
		public String getHSImageLabel() {
			return hsImageLabel;
		}
		
		public BufferedImage getBufferedImage() {
			return bufImage;
		}
		
		// just need to clear the image label here before calling parent removeAll()
		@Override
		public void removeAll() {
			hsImageLabel = "";
			super.removeAll();
		}

		@Override
		public void mousePressed(MouseEvent e) {
			if (palletDialog == null || palletDialog.isVisible() == false) {
				palletDialog = new HandShapePalletDialog(
						codingScheme.getFieldImg(fieldID), this, codingScheme, (Window) parent.getRootPane().getParent());
				palletDialog.setVisible(true);
			}
		}

		@Override
		public void mouseClicked(MouseEvent e) {}
		@Override
		public void mouseReleased(MouseEvent e) {}
		@Override
		public void mouseEntered(MouseEvent e) {}
		@Override
		public void mouseExited(MouseEvent e) {}

		@Override
		public void setItem(Object image) {
			if ( ! (image instanceof SS3FieldValue) )
				return;

			SS3FieldValue fieldValue = (SS3FieldValue) image;
            SS3FieldTypeImg firstImage = null;
            String hsFilePath = SSResources.resourcesPath + "/images/handShapes/";
            
            if (fieldValue != null) {
                firstImage = codingScheme.getImage(fieldValue.getIid());
                hsFilePath += firstImage.getPath();
                hsImageLabel = fieldValue.getLabel();
            }

            File hsFile = new File(hsFilePath);
            if (hsFile.exists() && !hsFile.isDirectory()) {
	            	try {
	            		bufImage = ImageIO.read(hsFile);
	            	} catch (IOException e) {
	            		e.printStackTrace();
						SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
	            	}
            }
			
			super.removeAll();
			add(new JLabel(new ImageIcon(bufImage)));
			SignBankLookupPanel.this.setSelectedHsCheckbox(true);
			palletDialog.dispose();
			
			revalidate();
			repaint();
		}

		@Override
		public void setItem(Object obj, boolean bothHands) {}
	}	
}
