package edu.bu.signstream.ui.textgrid;

import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.textgrid.TextGrid;
import edu.bu.signstream.common.textgrid.TextGridParser;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldPanel;
import edu.bu.signstream.ui.popup.Popup;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.URI;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Paths;

public class OpenTextGridDialog extends Popup {
    private final JLabel fileLabel = new JLabel("No file selected.");
    private final JButton file = new JButton("Select...");
    private final JLabel charsetsLabel = new JLabel("Character Encoding");
    private final JComboBox<Charset> charsets = new JComboBox<>(new Charset[]{
        StandardCharsets.US_ASCII,
        StandardCharsets.UTF_8,
        StandardCharsets.UTF_16
    });
    private final int charsetsDefaultIndex = 1;
    private final JButton open = new JButton("Open");

    private URI uri;
    private Charset charset;

    public OpenTextGridDialog(CSFieldPanel csFieldPanel) {
        setAlwaysOnTop(true);
        csFieldPanel.dialog().block();

        var panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        var gbc = new GridBagConstraints();
        gbc.weightx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = GridBagConstraints.REMAINDER;

        panel.add(fileLabel, gbc);
        panel.add(file, gbc);
        file.addActionListener(e -> {
            var file = UserInterfaceUtil.selectFile(
                SS3Singleton.getSignStreamApplication(),
                new File(SSResources.rootPath),
                null,
                UserInterfaceUtil.ALL_FILES_FILTER,
                "Import TextGrid File"
            );

            if (file != null) {
                var path = Paths.get(file.getPath()).normalize();
                uri = path.toUri();
                fileLabel.setText(path.getFileName().toString());
                open.setEnabled(true);
            }
        });

        panel.add(charsetsLabel, gbc);
        panel.add(charsets, gbc);
        charsets.setSelectedIndex(charsetsDefaultIndex);
        charset = (Charset)charsets.getSelectedItem();
        charsets.addActionListener(e -> charset = (Charset) charsets.getSelectedItem());

        open.setEnabled(false);
        panel.add(open, gbc);
        open.addActionListener(e -> {
            setVisible(false);

            TextGrid grid;
            try {
                grid = TextGridParser.parse(uri, charset);
                grid.validate();
            } catch (Exception ex) {
                var message = "Error importing TextGrid file:\n" + ex.getMessage();
                JOptionPane.showMessageDialog(this, message, "Import Error", JOptionPane.ERROR_MESSAGE);
                csFieldPanel.dialog().unblock();
                return;
            }

            new ImportTextGridDialog(csFieldPanel, grid).display(csFieldPanel);
        });

        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                csFieldPanel.dialog().unblock();
            }
        });

        add(panel);
    }
}
